// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2007 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: ank
// date: 19.01.2006

#include <ShaderFragments.hlsl>

// ================================================================================================
const float4    LIGHT_COLOR         : GLOBAL;
const float4    PickColor          : GLOBAL;
const float4    LIGHT_INTENSITY     : GLOBAL;

texture2D texture0;

// ================================================================================================
sampler2D sampler0 = sampler_state
{
	Texture = (texture0);
	AddressU = Clamp;
	AddressV = Clamp;
    MinFilter = Linear;
	MagFilter = Linear;
  	MipFilter = None;
  	MaxAnisotropy = 1;
};

// ================================================================================================
sampler2D sampler1 = sampler_state
{
	Texture = (texture0);
	AddressU = Clamp;
	AddressV = Clamp;
    MinFilter = Point;
	MagFilter = Point;
  	MipFilter = None;
  	MaxAnisotropy = 1;
};

// ================================================================================================
VS_PosColUV0 VS_SimpleTransform(VS_PosColUV0 p_vsIn)
{
    VS_PosColUV0 o;
    o.vPosition = mul(p_vsIn.vPosition, matWorldViewProjection);
    o.vColor = p_vsIn.vColor * LIGHT_COLOR;
    o.vTex = p_vsIn.vTex;
    return o;
};

// ================================================================================================
float4 PS_ModulateTexWithColor(PS_ColUV0 p_psIn) : COLOR
{
    // return float4(1,0,0,1);
	return tex2D(sampler0, p_psIn.vTex) * p_psIn.vColor;
}

// ================================================================================================
technique Billboard
{
	pass p0
	{
		FillMode            = <GROUND_FILL_MODE>;
		ShadeMode           = Flat;
	    CullMode            = CW;
	    ZEnable             = true;
	    ZWriteEnable        = true;
	    AlphaTestEnable     = true;
	    AlphaRef            = 0;
	    AlphaFunc           = Greater;
	    AlphaBlendEnable    = true;
	    SrcBlend            = SrcAlpha;
	    DestBlend           = InvSrcAlpha;
	    Lighting            = false;

		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_SimpleTransform();
		PixelShader  = compile  SHAD_PS_MINVER_1_1  PS_ModulateTexWithColor();
	}
}



// ================================================================================================
VS_PosColUV0 VS_SimpleTransformNoLight(VS_PosColUV0 p_vsIn)
{
    VS_PosColUV0 o;
    o.vPosition = mul(p_vsIn.vPosition, matWorldViewProjection);
    o.vColor = p_vsIn.vColor;
    o.vTex = p_vsIn.vTex;
    return o;
};

// ================================================================================================
technique BillboardUnlit
{
	pass p0
	{
		FillMode            = <GROUND_FILL_MODE>;
		ShadeMode           = Flat;
	    CullMode            = CW;
	    ZEnable             = true;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = true;
	    AlphaRef            = 0;
	    AlphaFunc           = Greater;
	    AlphaBlendEnable    = true;
	    SrcBlend            = SrcAlpha;
	    DestBlend           = InvSrcAlpha;
	    Lighting            = false;

		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_SimpleTransformNoLight();
		PixelShader  = compile  SHAD_PS_MINVER_1_1  PS_ModulateTexWithColor();
	}
}



// ================================================================================================
VS_PosColUV0 VS_SimpleTransformPick(VS_PosUV0 p_vsIn)
{
    VS_PosColUV0 o;
    o.vPosition = mul(p_vsIn.vPosition, matWorldViewProjection);
    o.vColor = PickColor;
    o.vTex = p_vsIn.vTex;
    return o;
};

// ================================================================================================
float4 PS_BillboardPick(PS_ColUV0 p_psIn) : COLOR
{
    float4 col = tex2D(sampler1, p_psIn.vTex);
    col.xyz = p_psIn.vColor;
    return col;
    //if (col.a > 0.5)
    //    return p_psIn.vColor;
    //else
    //    return float4(0,0,0,0);
}

// ================================================================================================
technique BillboardPick
{
	pass p0
	{
	    FillMode            = Solid;
		ShadeMode           = Flat;
	    CullMode            = CW;
	    ZEnable             = true;
	    ZWriteEnable        = true;
	    AlphaTestEnable     = true;
	    AlphaRef            = 128;
	    AlphaFunc           = Greater;
	    AlphaBlendEnable    = false;
	    Lighting            = false;

		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_SimpleTransformPick();
		PixelShader  = compile  SHAD_PS_MINVER_1_1  PS_BillboardPick();
	}
}

