// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2007 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: ank
// date: 18.01.2006

#include <ShaderFragments.hlsl>

// ================================================================================================
const float4x4  TEX1_TRANSFORM      : GLOBAL;
const float4x4  TEX2_TRANSFORM      : GLOBAL;

texture tex0<string NTM = "base";>;
texture tex1<int NTMIndex = 0;>;
texture texMask<int NTMIndex = 1;>;

// ================================================================================================
technique Clouds //_v1
{
	pass p0
	{
		FillMode            = <GROUND_FILL_MODE>;
		ShadeMode           = Gouraud;
	    CullMode            = CW;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = true;
	    SrcBlend            = SrcAlpha;
	    DestBlend           = InvSrcAlpha;
	    Lighting            = false;
        ZEnable             = false;
        ZWriteEnable        = false;
	    
	    SET_FILTER(0, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
	    AddressU[0]         = Wrap;
	    AddressV[0]         = Wrap;
	    ColorOp[0]          = SelectArg1;
		ColorArg1[0]        = Texture;
		ColorArg2[0]        = Current;
		AlphaOp[0]          = SelectArg1;
		AlphaArg1[0]        = Texture;
		AlphaArg2[0]        = Current;
		Texture[0]          = (tex0);
        TexCoordIndex[0]    = 0;
        TextureTransformFlags[0] = Count2;
        TextureTransform[0] = <TEX1_TRANSFORM>;
        
        SET_FILTER(1, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
        AddressU[1]         = Wrap;
	    AddressV[1]         = Wrap;
        ColorOp[1]          = BlendTextureAlpha;
		ColorArg1[1]        = Current;
		ColorArg2[1]        = Texture;
		AlphaOp[1]          = Modulate;
		AlphaArg1[1]        = Current;
		AlphaArg2[1]        = Texture;
		Texture[1]          = (tex1);
        TexCoordIndex[1]    = 0;
        TextureTransformFlags[1] = Count2;
        TextureTransform[1] = <TEX2_TRANSFORM>;
        
        SET_FILTER(2, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
        AddressU[2]         = Clamp;
	    AddressV[2]         = Clamp;
        ColorOp[2]          = SelectArg1;
		ColorArg1[2]        = Current;
		AlphaOp[2]          = Modulate;
		AlphaArg1[2]        = Texture;
		AlphaArg2[2]        = Current;
		Texture[2]          = (texMask);
        TexCoordIndex[2]    = 1;
        TextureTransformFlags[2] = Disable;
	}
}

// ================================================================================================
// ================================================================================================
sampler2D sampler0 = sampler_state
{
	Texture = (tex0);
	AddressU = Wrap;
	AddressV = Wrap;
};

// ================================================================================================
sampler2D sampler1 = sampler_state
{
	Texture = (tex1);
	AddressU = Wrap;
	AddressV = Wrap;
};

// ================================================================================================
sampler2D sampleMask = sampler_state
{
	Texture = (texMask);
	AddressU = Clamp;
	AddressV = Clamp;
};

// ================================================================================================
struct VS_InClouds
{
    float4 Position : POSITION;
    float4 TexCoord0: TEXCOORD0;
    float4 TexCoord1: TEXCOORD1;
};

// ================================================================================================
struct VS_OutClouds
{
    float4 Position  : POSITION;
    float2 TexCoord0: TEXCOORD0;
    float2 TexCoord1: TEXCOORD1;
    float2 TexCoord2: TEXCOORD2;
};

// ================================================================================================
VS_OutClouds VS_CloudsTransform(VS_InClouds i)
{
    VS_OutClouds o = (VS_OutClouds)0;
    o.Position  = mul(float4(i.Position.xyz, 1), matWorldProjection);	// ka, warum hier nur WorldProjection genommen wird
    o.TexCoord0 = mul(TEX1_TRANSFORM, i.TexCoord0);
    o.TexCoord1 = mul(TEX2_TRANSFORM, i.TexCoord0);
    o.TexCoord2 = i.TexCoord1;
    return o;
}

// ================================================================================================
float4 PS_ColorCloudsOld(VS_OutClouds i) : COLOR
{
    float4 result = float4(1,0,0,1);
    float4 colTex1 = tex2D(sampler0, i.TexCoord0);
    float4 colTex2 = tex2D(sampler1, i.TexCoord1);
    float4 colMask = tex2D(sampleMask, i.TexCoord2);
    
    result.rgb = colTex1; //colTex2.a*colTex2.rgb + (1-colTex2.a)*colTex1.rgb;
    result.a = colTex1.a; //*colTex2.a*colMask.a;
    
    return result;
}

// ================================================================================================
float4 PS_ColorClouds(VS_OutClouds i) : COLOR
{
    float4 col = float4(0,0,0,0);
    float4 tex0 = tex2D(sampler0, 1 * i.TexCoord0) * 2 - 1;
    float4 tex1 = tex2D(sampler1, 2 * i.TexCoord0) * 2 - 1;
    float4 tex2 = tex2D(sampler0, 4 * i.TexCoord0) * 2 - 1;
    float4 tex3 = tex2D(sampler1, 8 * i.TexCoord0) * 2 - 1;
    
    float4 mask = tex2D(sampleMask, i.TexCoord0);
    
    // col.a = mask.a*tex0.a*tex1.a;
    
    // build up cloud noise
    col = tex0 + 0.5 * tex1 - 0.25 * tex2 + 0.125 * tex3;
    //col = col - 1 + 0.5;    // cloudcover = 0.5 ...
    //col = max(0, col);
    
    // generate clouds by cloudcover value
    float cloudcover = 0.73;
    col = min(max(col * 0.5 + 0.5 - cloudcover, 0.0), 1.0);
    
    // exponentiate for fluffyness
    float sharpness = 0.94;
    col = 1.0 - pow(sharpness, col*255.0);
    float light = col.a; //1.0 / exp(col.a * 0.4);
    col.xyz = light;
    
    return col;
}
// ================================================================================================
/*
float4 PS_ColorClouds_PS3(VS_OutClouds i) : COLOR
{
    float cloud_cover = 0.45;
    float cloud_sharpness = 0.94;
    
    // read out perlin ovates
    float3 tex = tex2D(Octave0, i.TexCoord0 * OctaveScale.x) * OctaveWeight.x;
    tex += tex2D(Octave0, i.TexCoord0 * OctaveScale.x) * OctaveWeight.x;
    tex += tex2D(Octave0, i.TexCoord0 * OctaveScale.x) * OctaveWeight.x;
    tex += tex2D(Octave0, i.TexCoord0 * OctaveScale.x) * OctaveWeight.x;
    // todo weitere octaven  
    
    // bring noise interval from -1/1 to 0/1 and subtract cloud cover
    tex = max(tex * 0.5 + 0.5 - cloud_cover, 0.0);  
    // exponentiate clouds t make them more solid
    tex = 1 - pow(cloud_sharpness, tex * 255.0);
    
    // tex.r == current cloud density...
    float density = 0;
    float3 endTracePos = float3(i.TexCoord0, -tex.r);
    float3 traceDir = endTracePos - sunPos;
    traceDir = normalize(traceDir);
    float3 curTracePos = sunPos + traceDir * 1.25;
    tex = 1.0 - pow(cloud_sharpness, tex * 255.0);
    traceDir *= 2.0;
    
    for (int i=0; i<64; i++)
    {
        curTracePos += traceDir;
        float4 tex2 = tex2D(densityFieldTex, curTracePos.xy) * 255.0;
        density += 0.1 * step(curTracePos.z*2, tex2.r*2);
    }
    
    float light = 1.0 / exp(scattering * 0.4);
    return float4(light, light, light, tex.r);
}
*/

// ================================================================================================
technique Clouds_v2
{
	pass p0
	{
		FillMode            = <GROUND_FILL_MODE>;
		ShadeMode           = Gouraud;
	    CullMode            = CW;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = true;
	    SrcBlend            = SrcAlpha;
	    DestBlend           = InvSrcAlpha;
	    Lighting            = false;
        ZEnable             = false;
        ZWriteEnable        = false;
        
        // org version....
 		////VertexShader = compile  SHAD_VS_MINVER_1_1  VS_CloudsTransform();
 		//PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_ColorClouds();
 		
 		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_CloudsTransform();
 		PixelShader  = compile  SHAD_PS_MINVER_2_0 PS_ColorCloudsOld();
    }
};