// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2007 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: ank
// date: 18.01.2006

#include <ShaderFragments.hlsl>

// ================================================================================================
const float4    LIGHT_COLOR          : GLOBAL;
const float4    TEX_BLEND_FACTOR     : GLOBAL;
const float4    SKYDOME_COLOR        : GLOBAL;
const float3    SUN_LIGHT_DIRECTION_WS : GLOBAL;

#if defined(ENABLE_NORMALMAPPING)
  const float4    SUN_LIGHT_AMBIENT      : GLOBAL;
  const float4    SUN_LIGHT_DIFFUSE      : GLOBAL;
  const float3    CAMERA_POS_WS          : GLOBAL;
  const float     GLOSSINESS             : GLOBAL = 20;  // todo: als param bergeben
#endif

// ================================================================================================
struct VS_Out
{
    float4 vPosition    : POSITION;
    float4 vColor       : COLOR;
    float2 vTex0        : TEXCOORD0;
    float2 vTex1        : TEXCOORD1;

#if defined(ENABLE_NORMALMAPPING)
    float3 LightDirTS   : TEXCOORD2;
    //float3 vWSPosition  : TEXCOORD3;
    float3 CamDirTS     : TEXCOORD3;
#endif    
};

// ================================================================================================
struct PS_In
{	
	float4 vColor   : COLOR;
	float2 vTex0    : TEXCOORD0;
    float2 vTex1    : TEXCOORD1;

#if defined(ENABLE_NORMALMAPPING)
    float3 LightDirTS   : TEXCOORD2;
    //float3 vWSPosition  : TEXCOORD3;
    float3 CamDirTS     : TEXCOORD3;
#endif    
};

// ================================================================================================
struct VS_PosCol
{
    float4 vPosition    : POSITION;
    float4 vColor       : COLOR;
};

// ================================================================================================
VS_PosCol VS_Skydome(float4 p_vPosition : POSITION)
{
    VS_PosCol o;
    o.vPosition = mul(p_vPosition, matWorldViewProjection);
    o.vColor = SKYDOME_COLOR * LIGHT_COLOR;
    return o;
};

// ================================================================================================
VS_Out VS_Ground(VS_PosColUV0 p_vsIn)
{
    VS_Out o;
    o.vPosition = mul(p_vsIn.vPosition, matWorldViewProjection);
    o.vTex0 = p_vsIn.vTex;
    o.vTex1 = p_vsIn.vTex;
    
#if defined(ENABLE_NORMALMAPPING) // LIGHT_COLOR == pre-computed diffuse lighting
    o.vColor = p_vsIn.vColor;

    o.LightDirTS  = normalize( mul( SUN_LIGHT_DIRECTION_WS, matInverseWorld) );
    o.LightDirTS.x = -o.LightDirTS.x;               // cause texCoords.x values are inverse to world.x values 
    
    //o.vWSPosition = p_vsIn.vPosition;
    o.CamDirTS = mul (CAMERA_POS_WS.xyz, matInverseWorld);  // transform cam-pos in objects space
    o.CamDirTS -= p_vsIn.vPosition.xyz;             // and get direction from current pos
    o.CamDirTS.x = -o.CamDirTS.x;                   // cause texCoords.x values are inverse to world.x values 
    o.CamDirTS = normalize(o.CamDirTS);

#else
    o.vColor = p_vsIn.vColor * LIGHT_COLOR;
#endif

    return o;
};

// ================================================================================================
float4 PS_TextureLerp(PS_In p_psIn) : COLOR
{	
    float4 col1 = tex2D(Sampler1, p_psIn.vTex1);
    float fSnowArea = col1.a;
        
#if !defined(ENABLE_NORMALMAPPING)
    float4 col0 = tex2D(Sampler0, p_psIn.vTex0);
#else
    float3 CamDirTS = normalize(p_psIn.CamDirTS);
    
    //float3 CamDirTS = mul (CAMERA_POS_WS.xyz, matInverseWorld);  // transform cam-pos in objects space
    //CamDirTS -= p_psIn.vWSPosition.xyz;                 // and get direction from current pos
    //CamDirTS = normalize(CamDirTS);
    
  #if defined(ENABLE_PARALLAXMAPPING)

  	static const float Scale =  0.03; //0.012;
	static const float Bias  = -0.02; //-0.01;

	float fCurrentHeight = tex2D(Sampler2, p_psIn.vTex0).w;
	float fHeight        = fCurrentHeight * Scale + Bias;
	p_psIn.vTex0        += CamDirTS.xy * fHeight;
  #endif
  
    float4 col0 = tex2D(Sampler0, p_psIn.vTex0);
    
    // Normalmap
    float3 vNormal = tex2D(Sampler2, p_psIn.vTex0);
    vNormal.xy  = vNormal.xy * 2 - 1;               // decode normals
    vNormal.z   = vNormal.z - 0.5;
    vNormal.xyz = normalize(vNormal.xyz);
    
    // Ambient and Diffuse Lighting here
    float3 ambdiffuse  = clamp(dot(vNormal, -p_psIn.LightDirTS),0,1) * SUN_LIGHT_DIFFUSE + SUN_LIGHT_AMBIENT;
    
    // Specular Lighting here
    float fSpecLevel  = col0.w;
        
    // 3ds max version using float-vectors
    float3 fHalf = normalize(CamDirTS - p_psIn.LightDirTS);
    float3 specular = saturate( pow( clamp(dot(fHalf, vNormal),0,1), GLOSSINESS));
    specular *= fSpecLevel;
    specular *= SUN_LIGHT_DIFFUSE;
    
    // todo: beleuchtung berechnen
    float4 vSnowNormal = tex2D(Sampler3, p_psIn.vTex1);
    float fSnowSpec = vSnowNormal.a;
    float3 snowNorm = vSnowNormal * 2 - 1;
    snowNorm = normalize(snowNorm);
    
    float3 snowDiffuse = 0.51*saturate(dot(snowNorm, -p_psIn.LightDirTS)) * SUN_LIGHT_DIFFUSE;
    col1.xyz *= (float3(0.50,0.51,0.55) + snowDiffuse) ;

    const float cfSnowGlossiness = 30.0;
    const float cfSnowSpecLevel  = 0.6;
    
    float3 fHalfSnow = normalize(p_psIn.CamDirTS - p_psIn.LightDirTS);
    float3 snowSpecular = saturate(fSnowSpec * cfSnowSpecLevel * pow( clamp(dot(snowNorm,fHalfSnow),0,1), cfSnowGlossiness)) * SUN_LIGHT_DIFFUSE;
    
    // calc final lighting here
    col0.xyz = p_psIn.vColor.xyz * col0.xyz * ambdiffuse + specular;
    col1    *= p_psIn.vColor;  // snow-layer uses just the old values...
    col1.xyz+= snowSpecular;

#endif    

    //float4 f = float4(TEX_BLEND_FACTOR.x, TEX_BLEND_FACTOR.x, TEX_BLEND_FACTOR.x, 1);
    float factorAlpha = (0.25+TEX_BLEND_FACTOR.x*0.75)*clamp((TEX_BLEND_FACTOR.x - (1 - fSnowArea)) / 0.18,0,1);
    factorAlpha = min(1.0f, factorAlpha);
    float4 f = float4(factorAlpha, factorAlpha, factorAlpha, 1);
    
    float4 fColor;

    // by mad: da textur alpha == heighmap
#if !defined(ENABLE_NORMALMAPPING)
    fColor.xyz = lerp(col0, col1, f) * p_psIn.vColor;
#else
    fColor.xyz = lerp(col0, col1, f);
#endif
    
    fColor.w   = 1;
	return fColor;
}

// ================================================================================================
struct VS_SkyIn
{
    float4 vPosition : POSITION;
    float2 vTex      : TEXCOORD0;
    float3 vNormal   : NORMAL0;
};

// ================================================================================================
struct VS_SkyOut
{
    float4 vPosition : POSITION;
    float2 vTex      : TEXCOORD0;
    float3 vVertex   : TEXCOORD1;
};

// ================================================================================================
VS_SkyOut VS_Skydome_2(VS_SkyIn p_In)
{
    VS_SkyOut o = (VS_SkyOut)0;
    o.vPosition =  mul(p_In.vPosition, matWorldViewProjection);
    o.vTex      = p_In.vTex;
    o.vVertex   = p_In.vPosition;
    return o;
}

// ================================================================================================
const float4 Zenith  : GLOBAL = { 0.00, 0.34, 0.71, 0.00 };
const float4 Horizon : GLOBAL = { 0.70, 0.40, 0.20, 0.00 };
const float  weather : GLOBAL = 1.f;

// ================================================================================================
float4 PS_Skydome_2(VS_SkyOut p_In) : COLOR
{	
	float3 norm = normalize(p_In.vVertex).xyz;
	float3 Sun = normalize(-SUN_LIGHT_DIRECTION_WS.xyz);
	float  SunDot = dot(Sun, norm);
    
    float  time_of_day = acos(Sun.z);
    float4 SunPos = float4(Sun.xyz*2.f, 0.f);

	//float2 cycle = {fmod(SunPos.w, 100.0), 0};
	float fact_clouds = saturate(SunPos.z * 4.0) * 0.3;
   	fact_clouds *= time_of_day;
        
	vector rays = 0.7f * pow( max(0.0, SunDot), 60.0f );
	//rays *= clouds2.a;
	
	vector light = 0.8f * pow( max(0.0001, SunDot), 360.0f );
	
	//float Curve = 0.25f * (2.0f - SunPos.z);
	float Curve = 0.4f * (2.0f - SunPos.z);

	vector sky = Zenith * (1-Curve) + Horizon * Curve;
	sky = sky * (1+SunDot) + Zenith * -SunDot;
	// sky *= (1.0-clouds_alpha*(1.0-time_of_day));
	sky *= saturate(SunPos.z+2.0);

	float4 color = light + rays + sky;

    // scale down color by weather (0->1) ... if (0) "*=1" / if (1) "*=0.5"
    color *= 1.0 - 0.5 * weather;
	
	color.a = 1.0f;
	return color;
}

// ================================================================================================
technique ExtArea
{
	pass p0
	{
		FillMode            = <GROUND_FILL_MODE>;
		ShadeMode           = Gouraud;
	    CullMode            = CW;
	    ZEnable             = true;
	    ZWriteEnable        = true;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = true;
	    SrcBlend            = SrcAlpha;
	    DestBlend           = InvSrcAlpha;
	    Lighting            = false;
	    
	    TexCoordIndex[0]            = 0;
	    TextureTransformFlags[0]    = Disable;
	    TexCoordIndex[1]            = 1;
	    TextureTransformFlags[1]    = Disable;
		
		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_Ground();
#if !defined(ENABLE_NORMALMAPPING)
		PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_TextureLerp();
#else
		PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_TextureLerp();
#endif
	}
}

// ================================================================================================
technique Skydome
{
	pass p0
	{
		FillMode            = <GROUND_FILL_MODE>;
		ShadeMode           = Gouraud;
	    CullMode            = CW;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    		
		VertexShader = compile  vs_1_1  VS_Skydome();
		PixelShader  = NULL;
	}
}


// ================================================================================================
technique Skydome2
{
	pass p0
	{
		FillMode            = <GROUND_FILL_MODE>;
		ShadeMode           = Gouraud;
	    CullMode            = CW;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    		
		VertexShader = compile vs_1_1 VS_Skydome_2();
		PixelShader  = compile ps_2_0 PS_Skydome_2();
	}
}