// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2009 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: ank
// date: 07.11.2006

#include "GlobalFX.h"
#include "Skinning.h"

#define SetStates \
    FillMode            = <GROUND_FILL_MODE>; \
	ShadeMode           = Gouraud; \
	ZEnable             = true; \
	ZWriteEnable        = true; \
	AlphaTestEnable     = true; \
	AlphaRef            = 100; \
	AlphaFunc           = greater; \
	AlphaBlendEnable    = false; \
	SrcBlend            = SrcAlpha; \
	DestBlend           = InvSrcAlpha; \
	Lighting            = false

// ================================================================================================

const float     g_fShellDist: GLOBAL = 0.2;     // global fur length
const float     g_fFurAlpha   : GLOBAL = 1;     // global alpha value for fur
const float     g_fBrightness : GLOBAL = 0.02;  // 0.07 global fur brightness 20/255
float           g_fGradientFactor : GLOBAL =  0.35; // Verschiebung des Gradientens

#define FUROFFSET 0.02
#define MAX_PASS 10.0

#define LIMITPASSES
#if defined(LIMITPASSES)
  const float     g_fLimitPass  : GLOBAL = MAX_PASS;  // PassLimit
#endif

// ================================================================================================
struct VS_PosColSpecUV0Pass
{
    float4 vPosition    : POSITION;
    float4 vAmb         : COLOR;
    float4 vDiff        : COLOR1;
    float2 vTex         : TEXCOORD0;
    float fPass         : TEXCOORD1;
    float4 vSpec        : TEXCOORD2;
    
 #if defined(ENABLE_SHADOWMAPPING)
    float4 LightVPS     : TEXCOORD3;
 #endif
};

// ================================================================================================
struct VS_PosNormUV0
{
    float4 vPosition: POSITION;
    float3 vNormal  : NORMAL;
    float2 vTex     : TEXCOORD0;
};


// ================================================================================================
// float4 PS_FurBase(VS_PosColSpecUV0Pass i) : COLOR
float4 PS_FurBase(float2 vTex, float4 vColor) : COLOR
{	
    float4 col = tex2D(SamplerBase, vTex);    // base color
    
    // calc layers ----------------------------------------
    float4 col2 = tex2D(Sampler2, vTex);      // layer 0
    float4 col3 = tex2D(Sampler3, vTex);      // layer 1
    
    // BaseColor already used within VS
    col *= ColorBase;
    col2 *= ColorLayer0;
    col3 *= ColorLayer1;
    
    col = ModulateSrcAlphaInvSrcAlpha(col2, col);
    col = ModulateSrcAlphaInvSrcAlpha(col3, col);
    
    col *= vColor; // BaseColor
    // ----------------------------------------------------
    
    return col;
}

// ================================================================================================
VS_PosColSpecUV0Pass VS_Fur(uniform float p_fPass, VS_PosNormUV0 i)
{    
    float4 PosWorld = normalize(mul(i.vPosition, matWorld));
    float3 NormalWorld = normalize(mul(i.vNormal, matWorld));

    VS_PosColSpecUV0Pass o;
#if !defined(LIMITPASSES)
    PosWorld.xyz += NormalWorld * g_fShellDist * p_fPass;
    o.fPass = p_fPass;
#else
    o.fPass = p_fPass * MAX_PASS / g_fLimitPass;            // Welchen Pass wrde das entsprechen?
    PosWorld.xyz += NormalWorld * g_fShellDist * o.fPass;   // mit den "neuen" Pass weiterrechnen
#endif

    o.vPosition = mul(PosWorld, matViewProj);
    o.vAmb  = CalcAmbient();
    o.vDiff = CalcDiffuse(NormalWorld, LightDir);
    o.vSpec = CalcSpecular(NormalWorld, LightDir, normalize(PosWorld - float3(CamPos.xyz)));
    o.vTex = i.vTex;

 #if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(PosWorld);
 #endif
    
    return o;
}


// ================================================================================================
// Pass = 0
// ================================================================================================
float4 PS_FurPass0(VS_PosColSpecUV0Pass i) : COLOR
{
#if defined(ENABLE_SHADOWMAPPING)
    // by mad: erweitert fr schatten
    float fCurrentDepth = GetDepthInLightSpace(i.LightVPS);
    float fShadow = GetObjectInShadowFactor(fCurrentDepth, i.LightVPS, ShadowMap);

    float4 col = PS_FurBase(i.vTex, i.vAmb + fShadow*i.vDiff);
#else
    float4 col = PS_FurBase(i.vTex, i.vAmb + i.vDiff);
#endif

    // Umstand, um z.B. durchscheinende Kanten zu verschleiern (einfach je nach Strke der Fellfarbe,
    // diese auf die Basistextur setzen (einfaches lineares interpolieren)
    // Aber auch, um im LoD-Fall die Farbe des Fells noch deutlich zu machen.
    float4 colFur = tex2D(Sampler0, i.vTex * 3);// noise tex
    float4 colFurDens = tex2D(Sampler1, i.vTex);// density map
    
    float3 colBaseAndFur = col.rgb * ColorFur.rgb;
    col.rgb = col.rgb + (colBaseAndFur - col.rgb) * colFurDens.r;
    
#if defined(ENABLE_SHADOWMAPPING)
    return col + fShadow*i.vSpec;
#else
    return col + i.vSpec;
#endif
}

// ================================================================================================
// Pass > 0
// ================================================================================================
float4 PS_Fur(VS_PosColSpecUV0Pass i) : COLOR
{	
    // by mad: erweitert fr schatten
 #if defined(ENABLE_SHADOWMAPPING)
    float fCurrentDepth = GetDepthInLightSpace(i.LightVPS);
    float fShadow = GetObjectInShadowFactor(fCurrentDepth, i.LightVPS, ShadowMap);

    float4 col = PS_FurBase(i.vTex, i.vAmb + fShadow*i.vDiff);
 #else
    float4 col = PS_FurBase(i.vTex, i.vAmb + i.vDiff);
 #endif
    
    float4 colFur = tex2D(Sampler0, i.vTex * 3);// noise tex
    float4 colFurDens = tex2D(Sampler1, i.vTex);// density map
    
    col *= ColorFur; // fur has its own additional color
    col *= 1.5 - colFur.r;
    float f = i.fPass/10; // x divided by num passes
    
    col.a *= (1 - f) * colFurDens.r * g_fFurAlpha;
    
    // debugging:
    // col.a *= (1-f) * g_fFurAlpha;
    // col.a = saturate(col.a * colFurDens.a); -> wrde gehn
    // col.a = saturate(col.a * colFurDens.r); -> liefert artefakte?! O.o
    
    col.rgb += f * (1-colFur.r) * g_fBrightness;
 #if defined(ENABLE_SHADOWMAPPING)
    col.xyz += fShadow*i.vSpec;
 #else
    col.xyz += saturate(i.vSpec).xyz;
 #endif
    return col;
}

// ================================================================================================
// more natural fur
// ================================================================================================
struct VS_PosColSpecUV0Pass_Nat
{
    float4 vPosition    : POSITION;
    float4 vAmb         : COLOR;
    float4 vDiff        : COLOR1;
    float2 vTex         : TEXCOORD0;
    float fPass         : TEXCOORD1;
    float fLyrDepth     : TEXCOORD2;
    float4 vSpec        : TEXCOORD3;
 #if defined(ENABLE_SHADOWMAPPING)
    float4 LightVPS     : TEXCOORD4;
 #endif
};


// ================================================================================================
VS_PosColSpecUV0Pass_Nat VS_FurNatural(uniform float fPassIndex, VS_PosNormUV0 i)
{
    float4 PosWorld    = mul(i.vPosition, matWorld);
    float3 NormalWorld = normalize(mul(i.vNormal, matWorld));

    VS_PosColSpecUV0Pass_Nat o;
  
#define LINEAR
#if defined(LINEAR)    
  #if !defined(LIMITPASSES)
    PosWorld.xyz += NormalWorld * g_fShellDist * fPassIndex;
    o.fPass       = fPassIndex;
  #else
    o.fPass       = fPassIndex * MAX_PASS / g_fLimitPass;
    PosWorld.xyz += NormalWorld * g_fShellDist * o.fPass;
  #endif
    
    o.fLyrDepth   = clamp(o.fPass/MAX_PASS,0,1); // linear
#else
  #if !defined(LIMITPASSES)
    o.fPass          = fPassIndex;
    const float fMax = MAX_PASS * g_fShellDist;
  #else
    o.fPass          = fPassIndex * MAX_PASS / g_fLimitPass;
    const float fMax = MAX_PASS * g_fShellDist;
  #endif

    o.fLyrDepth   = clamp(sqrt(o.fPass/MAX_PASS), 0, 1); // quad
    PosWorld.xyz += NormalWorld * fMax * o.fLyrDepth;
#endif

    o.vPosition = mul(PosWorld, matViewProj);

    o.vAmb  = CalcAmbient();
    o.vDiff = CalcDiffuse(NormalWorld, LightDir);
    o.vSpec  = CalcSpecular(NormalWorld, LightDir, normalize(PosWorld - float3(CamPos.xyz)));
    o.vTex   = i.vTex;

 #if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(PosWorld);
 #endif
     
    return o;
}

// ================================================================================================
float4 PS_FurNatural(VS_PosColSpecUV0Pass_Nat i) : COLOR0
{   
    // by mad: erweitert fr schatten
#if defined(ENABLE_SHADOWMAPPING)
    float fCurrentDepth = GetDepthInLightSpace(i.LightVPS);
    float fShadow = GetObjectInShadowFactor(fCurrentDepth, i.LightVPS, ShadowMap);
#endif    
    float4 colFur      = tex2D(Sampler0, i.vTex); // verschiebungsinfo
    float4 colFurDens  = tex2D(Sampler1, i.vTex); // density map
    
    float2 offset = colFur.gb * 2 - 1;            // offset bestimmen
    i.vTex -= FUROFFSET * i.fLyrDepth * offset;
    
    colFur.ra = tex2D(Sampler0, i.vTex * 3).ra;   // noise tex
    
#if defined(ENABLE_SHADOWMAPPING)
    float4 col = PS_FurBase(i.vTex, i.vAmb + fShadow*i.vDiff);
#else
    float4 col = PS_FurBase(i.vTex, i.vAmb + i.vDiff);
#endif

    if (i.fPass > 0 )
    {

//#define WII_FUR
#if !defined(WII_FUR)

        if (colFurDens.r < i.fLyrDepth)
            discard;
        // ... alternativ:
        //colFur.a *= colFurDens.r;
        
        //float gradientFur = 1; // -> alternativ:
        //float gradientFur = tex2D(SamplerGradient, float2(i.fLyrDepth,i.fLyrDepth));
        float gradientFur = pow(1 - i.fLyrDepth, g_fGradientFactor);    // 1 -> 0; wird durch pow mehr zu weiss gedrckt
        colFur.a *= gradientFur;
        
        // PoreMap * Farbe * Fellfarbe; Noise als Alpha
        col     *= ColorFur;
        col.rgb *= colFur.r;
        col.a   *= colFur.a * g_fFurAlpha;
        
        // Specular + Brightness draufaddieren
  #if defined(ENABLE_SHADOWMAPPING)
        col.rgb += g_fBrightness * i.fLyrDepth + fShadow * i.vSpec; //max(g_fBrightness, i.vSpec);
  #else
        col.rgb += g_fBrightness * i.fLyrDepth + i.vSpec; //max(g_fBrightness, i.vSpec);
  #endif
#else
        // Stage #1:
        col.rgb = g_fBrightness + (col.rgb + i.vSpec);
        col *= ColorFur; // extra
        
        // Stage #2:
        col = col * colFur;
#endif        
    }
  
    return col;
}


// ================================================================================================
technique Fur
<
	bool UsesNIRenderState = true;
>
{
	/*pass p0
	{
		SetStates;
	    		
		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_Fur(0);
		PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurPass0();
	}*/
	pass p1
	{
	    SetStates; // tmp
	    
	    AlphaRef            = 0;
	    AlphaBlendEnable    = true;
	    CullMode            = cw;
	    ZWriteEnable        = false;
	    
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_Fur(1);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p2
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_Fur(2);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p3
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_Fur(3);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p4
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_Fur(4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p5
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_Fur(5);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p6
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_Fur(6);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p7
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_Fur(7);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p8
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_Fur(8);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p9
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_Fur(9);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p10
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_Fur(10);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
}


// ================================================================================================
// Same as Fur - but only the 1st pass is needed
// ================================================================================================
technique FurLoD
<
    int BonesPerPartition = MAX_BONES;
    string NBTMethod = "none";
    bool UsesNIRenderState = true;
>	
{
	pass p0
	{
		SetStates;
	    		
		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_Fur(0);
		PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurPass0();
	}
}


// ================================================================================================
technique NaturalFur
<
	bool UsesNIRenderState = true;
>
{
	/*pass p0
	{
		SetStates;
	    		
		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurNatural(0);
		PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}*/
	
	pass p1
	{
	    SetStates; // tmp
	    
	    AlphaRef            = 0;
	    AlphaBlendEnable    = true;
	    CullMode            = cw;
	    ZWriteEnable        = false;
	    
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurNatural(1);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p2
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurNatural(2);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p3
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurNatural(3);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p4
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurNatural(4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p5
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurNatural(5);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p6
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurNatural(6);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p7
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurNatural(7);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p8
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurNatural(8);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p9
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurNatural(9);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p10
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurNatural(10);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
}

// ================================================================================================
// Same as FurNatural - but only the 1st pass is needed
// ================================================================================================
technique NaturalFurLoD
<
    int BonesPerPartition = MAX_BONES;
    string NBTMethod = "none";
    bool UsesNIRenderState = true;
>	
{
	pass p0
	{
		SetStates;
	    		
		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurNatural(0);
		PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
}


// ================================================================================================
VS_PosColSpecUV0Pass VS_FurSkinning(uniform float p_fPass, uniform int NumBones, VS_InSkinning i)
{
    VS_OutNormal os = Skinning(i.Pos, i.BlendWeights, i.BlendIndices, i.Normal, NumBones);

    VS_PosColSpecUV0Pass o;
#if !defined(LIMITPASSES)
    os.Pos += os.Normal * g_fShellDist * p_fPass;
    o.fPass = p_fPass;
#else
    o.fPass = p_fPass * MAX_PASS / g_fLimitPass;    // Welchen Pass wrde das entsprechen?
    os.Pos += os.Normal * g_fShellDist * o.fPass;   // mit den "neuen" Pass weiterrechnen
#endif
    
    o.vPosition = mul(float4(os.Pos, 1), matViewProj);
    o.vAmb  = CalcAmbient();
    o.vAmb *= ColorBase;
    o.vDiff = CalcDiffuse(os.Normal, LightDir);
    o.vDiff *= ColorBase;
    
    o.vSpec = CalcSpecular(os.Normal, LightDir, normalize(os.Pos - float3(CamPos.xyz)));
    o.vTex = i.Tex;
 
 #if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(float4(os.Pos, 1));
 #endif
    
    return o;
}

// ================================================================================================
VS_PosColSpecUV0Pass_Nat VS_FurSkinningNatural(uniform float fPassIndex, uniform int NumBones, VS_InSkinning i)
{
    VS_OutNormal os = Skinning(i.Pos, i.BlendWeights, i.BlendIndices, i.Normal, NumBones);
    
    VS_PosColSpecUV0Pass_Nat o;

#define LINEAR
#if defined(LINEAR)    
  #if !defined(LIMITPASSES)
    os.Pos  += os.Normal * g_fShellDist * fPassIndex;
    o.fPass = fPassIndex;
  #else
    o.fPass = fPassIndex * MAX_PASS / g_fLimitPass;
    os.Pos += os.Normal * g_fShellDist * o.fPass;
  #endif
    
    o.fLyrDepth   = clamp(o.fPass/MAX_PASS,0,1); // linear
#else
  #if !defined(LIMITPASSES)
    o.fPass          = fPassIndex;
    const float fMax = MAX_PASS * g_fShellDist;
  #else
    o.fPass          = fPassIndex * MAX_PASS / g_fLimitPass;
    const float fMax = MAX_PASS * g_fShellDist;
  #endif

    o.fLyrDepth = clamp(sqrt(o.fPass/MAX_PASS), 0, 1); // quad
    os.Pos     += os.Normal * fMax * o.fLyrDepth;
#endif

    o.vPosition = mul(float4(os.Pos, 1), matViewProj);

    o.vAmb  = CalcAmbient();
    o.vAmb *= ColorBase;
    o.vDiff = CalcDiffuse(os.Normal, LightDir);
    o.vDiff *= ColorBase;
    
    o.vSpec  = CalcSpecular(os.Normal, LightDir, normalize(os.Pos - float3(CamPos.xyz)));
    o.vTex  = i.Tex;

 #if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(float4(os.Pos, 1));
 #endif
    
    return o;
}


// ================================================================================================
technique FurSkinning
<
    int BonesPerPartition = MAX_BONES;
    string NBTMethod = "none";
    bool UsesNIRenderState = true;
>	
{
	/*pass p0
	{
		SetStates;
	    		
		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinning(0, 4);
		PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurPass0();
	}*/
	pass p1
	{
	    SetStates; // tmp
	    
	    AlphaRef            = 0;
	    AlphaBlendEnable    = true;
	    CullMode            = cw;
	    ZWriteEnable        = false;
	    
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinning(1, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p2
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinning(2, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p3
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinning(3, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p4
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinning(4, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p5
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinning(5, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p6
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinning(6, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p7
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinning(7, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p8
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinning(8, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p9
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinning(9, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
	pass p10
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinning(10, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_Fur();
	}
}

// ================================================================================================
// Same as FurSkinning - but only the 1st pass is needed
// ================================================================================================
technique FurSkinningLoD
<
    int BonesPerPartition = MAX_BONES;
    string NBTMethod = "none";
    bool UsesNIRenderState = true;
>	
{
	pass p0
	{
		SetStates;
	    		
		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinning(0, 4);
		PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurPass0();
	}
}

// ================================================================================================
technique NaturalFurSkinning
<
    int BonesPerPartition = MAX_BONES;
    string NBTMethod = "none";
    bool UsesNIRenderState = true;
>	
{
	/*pass p0
	{
		SetStates;
	    		
		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinning(0, 4);
		PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurPass0();
	}*/
	pass p1
	{
	    SetStates; // tmp
	    
	    AlphaRef            = 0;
	    AlphaBlendEnable    = true;
	    CullMode            = cw;
	    ZWriteEnable        = false;
	    
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinningNatural(1, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p2
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinningNatural(2, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p3
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinningNatural(3, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p4
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinningNatural(4, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p5
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinningNatural(5, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p6
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinningNatural(6, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p7
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinningNatural(7, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p8
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinningNatural(8, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p9
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinningNatural(9, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
	pass p10
	{
	    VertexShader = compile  SHAD_VS_MINVER_1_1  VS_FurSkinningNatural(10, 4);
	    PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_FurNatural();
	}
}

// ================================================================================================
// Same as FurSkinning - but only the 1st pass is needed
// ================================================================================================
technique NaturalFurSkinningLoD
<
    int BonesPerPartition = MAX_BONES;
    string NBTMethod = "none";
    bool UsesNIRenderState = true;
>	
{
	pass p0
	{
		SetStates;
	    		
		VertexShader = compile  SHAD_VS_MINVER_2_0 VS_FurSkinningNatural(0, 4);
		PixelShader  = compile  SHAD_PS_MINVER_2_0 PS_FurNatural();
	}
}