// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2018 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: jow
// date: 26.10.2018

#ifndef GLOBALPARAMETERS_HLSL
#define GLOBALPARAMETERS_HLSL

// ================================================================================================
#if defined(SUPPORTS_SHADERMODEL_3_0) && !defined(MIN_SHADER_VERSION)
#	define HALF_LAMBERT_SHADING
#   define ADVANCED_SHADING
#endif // ...

#define DIFFUSE
#define SPECULAR
#define ALBEDO

// ================================================================================================
// defined stuff
const float4    MaterialAmbient : MATERIALAMBIENT = {0.1f, 0.1f, 0.1f, 1.0f};
const float4    MaterialDiffuse : MATERIALDIFFUSE = {0.8f, 0.8f, 0.8f, 1.0f};
const float4    MaterialSpecular: MATERIALSPECULAR = {0.5f, 0.5f, 0.5f, 1.0f};
const float4    MaterialEmissive: MATERIALEMISSIVE = {0, 0, 0, 1.0f};
const float     MaterialPower   : MATERIALPOWER;

// Matrices
const float4x4 matWorld					: WORLD;
const float4x4 matView					: VIEW;
const float4x4 matProjection            : PROJECTION;
const float4x4 matWorldView				: WORLDVIEW;
const float4x4 matWorldProjection		: WORLDPROJECTION;
const float4x4 matViewProj				: VIEWPROJECTION;
const float4x4 matWorldViewProjection   : WORLDVIEWPROJECTION;
const float4x3 matInverseWorld			: INVWORLD;
const float4x4 matTex					: TEXTRANSFORMBASE;

// global stuff
const DWORD     GROUND_FILL_MODE	: GLOBAL = 3; // solid
const float4    LightDir			: GLOBAL = {0.0f, 0.0f, -1.0f, 1.0f};
const float4    LightAmbient		: GLOBAL = {0.3f, 0.3f, 0.3f, 1.0f};
const float4    LightDiffuse		: GLOBAL = {1.0f, 1.0f, 1.0f, 1.0f};
const float4    LightSpecular		: GLOBAL = {0.1f, 0.1f, 0.1f, 1.0f};
const float4    CamPos				: GLOBAL;
const float		Time				: GLOBAL = 0.0f;
const float3	MouseWorldPosition	: GLOBAL = {0.0f, 0.0f, 0.0f};
const float		GridRadius			: GLOBAL = 800.0f;
const float		GridRadiusFalloff	: GLOBAL = 200.0f;
const float3	MouseWorldPositionGrid	: GLOBAL = {0.0f, 0.0f, 0.0f};

//const float4    DeepSeaColor    : GLOBAL = {0.0f, 0.5f ,0.75f, 1.0f};
const float4    WATERGROUND_COLOR : GLOBAL = {0.0f, 0.5f ,0.75f, 1.0f};
const float     AnimalDepth     : GLOBAL = 0;
const float     LayerLerpFac    : GLOBAL = 0;
const bool      CaustEnhanceFac : GLOBAL = 2;   // Verstrkungsfaktor fr Caustics
const float     WaterHeight     : GLOBAL = 1000;
const float     DeepSeaFactor   : GLOBAL = 0.003;

const DWORD     MIN_FILTERTYPE  : GLOBAL = 2;
const DWORD     MAG_FILTERTYPE  : GLOBAL = 2;
const DWORD     MIP_FILTERTYPE  : GLOBAL = 2;
const DWORD     MAX_ANISOTROPY  : GLOBAL = 1;

const float     g_fNBTValue     : GLOBAL = 1;   // Value to regulate the Lighting between NBT- and VertexLighting
const float     g_fGlossiness   : GLOBAL = 1;

const float4    ColorBase       : GLOBAL = {1,1,1,1};
const float4    ColorFur        : GLOBAL = {1,1,1,1};
const float4    ColorLayer0     : GLOBAL = {1,1,1,1};
const float4    ColorLayer1     : GLOBAL = {1,1,1,1};
const float4    ColorLayer2     : GLOBAL = {1,1,1,1};

const float     g_fSnowCoverage     : GLOBAL = 0;   // how much snow is on the object
const float     g_fSnowConstFactor  : GLOBAL = 0;
const float     g_fSecondColorBlend : GLOBAL = 0.f;
const float     g_fSecondColorDecay : GLOBAL = 0.f;
const int       numLayer            : GLOBAL = 0;

const float     FogStart    : GLOBAL;
const float     FogEnd      : GLOBAL;
const float     FogDensity  : GLOBAL;
const float4    FogColor    : GLOBAL;

// Vertex-Color Animationen pro Object
const float		g_VertexAnimationScale	: GLOBAL = 1.0f;			// Value to increase/decrease the animation effect on a per object basis
const int		g_HasVertexColor		: GLOBAL = 0;				// 0 = has no vertex color animation; 1 = has vertex color animation

// Vertex-Color Animationen globale Parameter
const float		g_WindMaxDisplacement	: GLOBAL = 20.0f;			// Max displacement in wind direction (in wirld space)
const float		g_WindAnimationSpeed	: GLOBAL = 2.0f;			// Speed of the animation
const float		g_WindHeightFactor		: GLOBAL = 1.0f / 600.0f;	// Vertices at different heights get displaced differently
const float		g_WindWaveLength		: GLOBAL = 1000.0f;			// The distance of the wave animation in worldspace
const float2	g_WindDirection			: GLOBAL = {1.0f, 1.0f};	// The direction of the wind in worldspace

// Matrix Palette
static const int MAX_BONES = 20;
const float4x4 matWorldArray[MAX_BONES] : SKINBONEMATRIX4;

// ================================================================================================
texture BaseMap<string NTM = "base";>;
texture ShaderMap0<int NTMIndex = 0;>;
texture ShaderMap1<int NTMIndex = 1;>;
texture ShaderMap2<int NTMIndex = 2;>;
texture ShaderMap3<int NTMIndex = 3;>;
texture ShaderMap4<int NTMIndex = 4;>;
texture ShaderMap5<int NTMIndex = 6;>;  // shadowmap uses slot 5
texture ShaderMap6<int NTMIndex = 7;>;

sampler2D SamplerBase = sampler_state
{
    Texture = (BaseMap);
    AddressU = Wrap;
    AddressV = Wrap;
    MinFilter = (MIN_FILTERTYPE);
    MagFilter = (MAG_FILTERTYPE);
    MipFilter = (MIP_FILTERTYPE);
    MaxAnisotropy = (MAX_ANISOTROPY);
};

sampler2D Sampler0 = sampler_state
{
    Texture = (ShaderMap0);
    AddressU = Wrap;
    AddressV = Wrap;
    MinFilter = (MIN_FILTERTYPE);
    MagFilter = (MAG_FILTERTYPE);
    MipFilter = (MIP_FILTERTYPE);
    MaxAnisotropy = (MAX_ANISOTROPY);
};

sampler2D Sampler1 = sampler_state
{
    Texture = (ShaderMap1);
    AddressU = Wrap;
    AddressV = Wrap;
    MinFilter = (MIN_FILTERTYPE);
    MagFilter = (MAG_FILTERTYPE);
    MipFilter = (MIP_FILTERTYPE);
    MaxAnisotropy = (MAX_ANISOTROPY);
};

sampler2D Sampler2 = sampler_state
{
    Texture = (ShaderMap2);
    AddressU = Wrap;
    AddressV = Wrap;
    MinFilter = (MIN_FILTERTYPE);
    MagFilter = (MAG_FILTERTYPE);
    MipFilter = (MIP_FILTERTYPE);
    MaxAnisotropy = (MAX_ANISOTROPY);
};

sampler2D Sampler3 = sampler_state
{
    Texture = (ShaderMap3);
    AddressU = Wrap;
    AddressV = Wrap;
    MinFilter = (MIN_FILTERTYPE);
    MagFilter = (MAG_FILTERTYPE);
    MipFilter = (MIP_FILTERTYPE);
    MaxAnisotropy = (MAX_ANISOTROPY);
};

sampler2D Sampler4 = sampler_state
{
    Texture = (ShaderMap4);
    AddressU = Wrap;
    AddressV = Wrap;
    MinFilter = (MIN_FILTERTYPE);
    MagFilter = (MAG_FILTERTYPE);
    MipFilter = (MIP_FILTERTYPE);
    MaxAnisotropy = (MAX_ANISOTROPY);
};

sampler2D SamplerSnow = sampler_state
{
    Texture = (ShaderMap5);
    AddressU = Wrap;
    AddressV = Wrap;
    MinFilter = (MIN_FILTERTYPE);
    MagFilter = (MAG_FILTERTYPE);
    MipFilter = (MIP_FILTERTYPE);
    MaxAnisotropy = (MAX_ANISOTROPY);
};

sampler2D SamplerCaustics = sampler_state
{
    Texture = (ShaderMap6);
    AddressU = Wrap;
    AddressV = Wrap;
    MinFilter = (MIN_FILTERTYPE);
    MagFilter = (MAG_FILTERTYPE);
    MipFilter = (MIP_FILTERTYPE);
    MaxAnisotropy = (MAX_ANISOTROPY);
};

#endif
