// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2007 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: ank
// date: 19.01.2006

#include "GlobalFX.h"
#include <ShaderFragments.hlsl>
#include <shadowmap.hlsl>

// ================================================================================================
//const float4    LIGHT_COLOR        : GLOBAL;
const float4    LIGHT_COLOR_AMB      : GLOBAL;
const float4    LIGHT_COLOR_DIFF     : GLOBAL;
const bool      AlphaBlend          : GLOBAL = false;

// ================================================================================================
VS_PosColUV0LightVPS VS_SimpleTransform(VS_PosColUV0 p_vsIn)
{
    VS_PosColUV0LightVPS o;
    o.vPosition = mul(p_vsIn.vPosition, matWorldViewProjection);
    o.vColor = p_vsIn.vColor; // * LIGHT_COLOR;
    o.vTex = p_vsIn.vTex;
    
#if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(p_vsIn.vPosition);
#endif

    return o;
};

// ================================================================================================
float4 PS_ModulateTexWithColor(VS_PosColUV0LightVPS p_psIn) : COLOR
{
#if defined(ENABLE_SHADOWMAPPING)
    // by mad: um schatten erweitert
    float fCurrentDepth = GetDepthInLightSpace(p_psIn.LightVPS);
    float fShadow = GetObjectInShadowFactor(fCurrentDepth, p_psIn.LightVPS, ShadowMap);
    
    float4 col = clamp(LIGHT_COLOR_AMB + fShadow*LIGHT_COLOR_DIFF, 0, 1);
#else
    float4 col = clamp(LIGHT_COLOR_AMB + LIGHT_COLOR_DIFF, 0, 1);
#endif

    //float4 texCoord = float4(p_psIn.vTex.xy, 0, -0.5);
    //col.rgb *= tex2D(Sampler0, texCoord) * p_psIn.vColor;
    //col.a   *= tex2Dbias(Sampler0, texCoord).a * p_psIn.vColor.a;
    float4 texCol = tex2D(Sampler0, p_psIn.vTex);
    col.rgb *= texCol * p_psIn.vColor;
    col.a    = texCol.a;
    return col; 
}

// ================================================================================================
technique Grass
{
	pass p0
	{
		FillMode            = <GROUND_FILL_MODE>;
		//ShadeMode           = Flat;
	    CullMode            = None;
	    ZEnable             = true;
	    ZWriteEnable        = <!AlphaBlend>;    // write only to z if AlphaTesting
	    AlphaTestEnable     = <!AlphaBlend>;
	    AlphaRef            = 80;
	    AlphaFunc           = greater;
	    AlphaBlendEnable    = <AlphaBlend>;
	    SrcBlend            = SrcAlpha;
	    DestBlend           = InvSrcAlpha;
	    Lighting            = false;

		VertexShader = compile  SHAD_VS_MINVER_2_0  VS_SimpleTransform();
		PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_ModulateTexWithColor();
	}
}

// ================================================================================================
// new grass shader
// ================================================================================================
// const int    CURRENT_GRASS_LAYER   : GLOBAL = 0; // texture selection
// const int    NUM_GRASS_LAYER       : GLOBAL = 1;
// const float2 GRASS_ANIM_OFFSET     : GLOBAL;     // "flowing direction"

// const float  FAR_DISTANCE... (fr Hhe!)
const float     TIME : GLOBAL;
const float     WIND : GLOBAL;

// ================================================================================================
struct VS_AnimGrass
{
    float4 vPosition    : POSITION;
    float4 vColor       : COLOR;
    float2 vTex         : TEXCOORD0;
    float3 vDirection   : NORMAL0;
};

// ================================================================================================
VS_PosColUV0LightVPS VS_AnimatedGrassTransform(VS_AnimGrass p_vsIn)
{
    VS_PosColUV0LightVPS o;
    
    float fWindFactor = WIND * 1.5 + 0.5;
    float4 vPosition = p_vsIn.vPosition;
    //vPosition.xy    += p_vsIn.vDirection.xy * cos(TIME) * max(0.2 ,p_vsIn.vDirection.z);
    vPosition.xy    += p_vsIn.vDirection.xy * fWindFactor * sin(TIME * p_vsIn.vDirection.z);
    
    o.vPosition = mul(vPosition, matWorldViewProjection);
    o.vColor    = p_vsIn.vColor; // * LIGHT_COLOR;
    o.vTex      = p_vsIn.vTex;
    
#if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(p_vsIn.vPosition);
#endif

    return o;
};

// ================================================================================================
technique AnimatedGrass
{
	pass p0
	{
		FillMode            = <GROUND_FILL_MODE>;
		//ShadeMode           = Flat;
	    CullMode            = None;
	    ZEnable             = true;
	    ZWriteEnable        = <!AlphaBlend>;    // write only to z if AlphaTesting
	    AlphaTestEnable     = <!AlphaBlend>;
	    AlphaRef            = 60;
	    AlphaFunc           = greater;
	    AlphaBlendEnable    = <AlphaBlend>;
	    SrcBlend            = SrcAlpha;
	    DestBlend           = InvSrcAlpha;
	    Lighting            = false;

		VertexShader = compile  SHAD_VS_MINVER_2_0  VS_AnimatedGrassTransform();
		PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_ModulateTexWithColor();
	}
}
