// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2008 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author:   
// modifier: mad

#include <ShaderFragments.hlsl>

const float3    SUN_LIGHT_DIRECTION_WS : GLOBAL;
const float4    SUN_LIGHT_AMBIENT : GLOBAL;
const float4    SUN_LIGHT_DIFFUSE : GLOBAL;
// const float3 SUN_LIGHT_SPECULAR : GLOBAL;
const float4    GROUND_BLEND_COLOR_MIN : GLOBAL;
const float4    GROUND_BLEND_COLOR_MAX : GLOBAL;

const float     CAUST_ITERATION_FACTOR : GLOBAL;

// const bool FOG_ENABLE : GLOBAL;
const float3    CAMERA_POS_WS : GLOBAL;

const bool      FogTypeLinear : GLOBAL = false; // false = EXPONENTIAL

#define USE_BLENDED_SNOW
#include <GroundTile_Shared.hlsl>

// ================================================================================================
float4 PSDebug( PSIn i_PSIn ) : COLOR
{
	return i_PSIn.Color;
}

// ================================================================================================
float ComputeFogValue( float3 i_PosWS )
{
    float fFog;
    if (FogTypeLinear)
	{
	    float FarD = FogEnd;
	    float FadeRange = FarD - FogStart;
	    fFog =  1 - min(1, max(0, (FarD - i_PosWS.z) / FadeRange));  // -> DX SDK
	}
	else
	{
	    // exp fog version	
	    float eTPODTD = exp(i_PosWS.z * FogDensity);
	    fFog = 1 - (1/eTPODTD);
    }
    return fFog;
}

// ================================================================================================
VSOut VSTransformEnlightenGernerateUVDeepSea( VSIn i_VSIn )
{
	VSOut Out = (VSOut) 0; 
	
	// Output transformed vertex position:
	Out.Pos = mul(float4( i_VSIn.Pos, 1.0f ), matWorldViewProjection); 

	Out.PosVS = mul(float4(i_VSIn.Pos, 1.0f), matWorldView);
	Out.PosVS.z /= Out.PosVS.w;

	Out.PosVS.w = Out.Pos.z / Out.Pos.w;
	
	float4 worldPos = mul(float4(i_VSIn.Pos,1.0f), matWorld);
	worldPos.z /= worldPos.w;
	
	// apply deep Sea bluishness
	// Out.Color.xy *= saturate( 1.0f - i_VSIn.WaterHeight.x );
	Out.DeepSeaWeightFog = 0.0f;
    Out.DeepSeaWeightFog.x = i_VSIn.WaterHeight.x;
//	Out.DeepSeaWeightFog.y = ComputeFogValue( i_VSIn.Pos );
	Out.DeepSeaWeightFog.y = ComputeFogValue( Out.Pos.xyz );
	Out.DeepSeaWeightFog.z = worldPos.z;
		
    UpdateLighting(i_VSIn, Out);	
	
	// Propagate texture coordinates:
	// Out.TexBlend = i_VSIn.TexBlendNCWeight.xy;
	// Out.TexMaterial00 = i_VSIn.TexSurface;
	// Out.TexMaterial01 = i_VSIn.TexSurface;
	// Out.TexMaterial02 = i_VSIn.TexSurface;
	
	//Out.TexBlend_Material00.xy = i_VSIn.TexBlendNCWeight.xy;
	Out.TexBlend_Material00.xy = i_VSIn.TexBlend;
	Out.TexBlend_Material00.zw = i_VSIn.TexSurface;
	Out.TexMaterial01_02 = i_VSIn.TexSurface.xyxy;

 #if defined(ENABLE_SHADOWMAPPING)
	Out.LightVPS = PerformTransformInLightSpace(mul(float4(i_VSIn.Pos, 1.0f ), matWorld));
 #endif
 	
	return Out;
}

// ================================================================================================
VSOut VSTransformGrid( VSIn i_VSIn )
{
	VSOut Out = (VSOut) 0; 
	
	// Output transformed vertex position:
	Out.Pos = mul(float4( i_VSIn.Pos, 1.0f ), matWorldViewProjection); 
	
	float4 worldPos = mul(float4(i_VSIn.Pos,1.0f), matWorld);
	worldPos.z /= worldPos.w;
	Out.TexBlend_Material00.xy = worldPos.xy;
 	
	return Out;
}

// ================================================================================================
float4 PSBlendCaustics( PSIn i_PSIn ) : COLOR
{
    float fUnderWaterDepth = saturate((WaterHeight - i_PSIn.DeepSeaWeightFog.z)*DeepSeaFactor);

	float4 color = tex2D( Caust, i_PSIn.TexBlend_Material00.zw * CAUST_ITERATION_FACTOR ) * fUnderWaterDepth * 2.0f;
	color.xyz *= SUN_LIGHT_DIFFUSE;
	return color;

	//return tex2D( Caust, i_PSIn.TexBlend_Material00.zw * CAUST_ITERATION_FACTOR ) * i_PSIn.DeepSeaWeightFog.x * 2.0f;
}

// ================================================================================================
float4 PSBlendDeepSea( PSIn i_PSIn ) : COLOR
{
	// <Artist DeapSea>:
	// edit the first three float values
	// to setup the deep sea color to blend to
	// 0.0f, 0.0f, 1.0f is simple blue

	float fUnderWaterDepth = saturate((WaterHeight - i_PSIn.DeepSeaWeightFog.z)*DeepSeaFactor);
	return float4(SUN_LIGHT_DIFFUSE * WATERGROUND_COLOR.xyz, fUnderWaterDepth);
	
	//return float4(WATERGROUND_COLOR.xyz, i_PSIn.DeepSeaWeightFog.x);
}

// ================================================================================================
float4 PSBlendFog( PSIn i_PSIn ) : COLOR
{
	// <Artist DeapSea>:
	// edit the first three float values
	// to setup the deep sea color to blend to
	// 0.0f, 0.0f, 1.0f is simple blue
	
	float4 C = FogColor;
	C.w = i_PSIn.DeepSeaWeightFog.y;
	
	return C;
}

// ================================================================================================
float4 PSGrid( PSIn i_PSIn ) : COLOR
{
    float4 color = float4(1,1,1,0);
    
    if (!GRID_TYPE)
        discard;

    color = GridColor(i_PSIn.TexBlend_Material00.xy, float4(0,1,0,1));

    if (color.a <= 0.f)
        discard;

	// by jow
	float dist = length(MouseWorldPositionGrid - i_PSIn.TexBlend_Material00.xy);
	float factor = 1.0f - saturate((dist - GridRadius) / GridRadiusFalloff);

	color.a = lerp(0.0f, color.a, factor);

    return color;
    //return i_PSIn.Color;
}

// ================================================================================================
// techniques
// ================================================================================================
technique GroundTileSingleSetDeepSea
{
	pass MaterialSet00
	{
		FillMode = <GROUND_FILL_MODE>;
		
		RS_Z_ALPHA_SOLID_SET
		SETUP_TEXTURE_STAGES
		
		// VertexShader = compile SHAD_VS_MINVER_2_0 VSDebug();
		// PixelShader = compile SHAD_PS_MINVER_2_0 PSDebug();
		
		FogColor = <FogColor>;
		
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformEnlightenGernerateUVDeepSea();
		//PixelShader = compile SHAD_PS_MINVER_1_1 PSDebug();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsSolidSet();
	}

#ifdef USE_BLENDED_NORMALS
	pass BlendNormal
	{
		RS_Z_BLENDED_NORMALS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendNormals();
	}
#endif

#ifdef USE_BLENDED_SNOW	
	pass BlendSnow
	{
		RS_Z_BLENDED_SNOW
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendSnow();
	}
#endif
	
	pass BlendCaustics
	{
		RS_Z_BLENDED_CAUSTICS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendCaustics();
	}
	
	pass BlendDeepSea
	{
		RS_Z_BLENDED_DEEP_SEA
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendDeepSea();
	}
	
	pass BlendFog
	{
		RS_Z_BLENDED_FOG
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendFog();
	}
	
	pass Grid
	{
		//FillMode = WireFrame;
		AlphaBlendEnable = true;
		ZEnable = true;
		
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformGrid();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSGrid();
	}
    
}

// ================================================================================================
technique GroundTileDoubleSetDeepSea
{
	pass MaterialSet00
	{
		FillMode = <GROUND_FILL_MODE>;
		
		RS_Z_ALPHA_SOLID_SET
		SETUP_TEXTURE_STAGES
		
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformEnlightenGernerateUVDeepSea();
		//PixelShader = compile SHAD_PS_MINVER_1_1 PSDebug();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsSolidSet();
	}	
	
	pass MaterialSet01
	{
		RS_Z_ALPHA_BLENDED_SET
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet01();
	}

#ifdef USE_BLENDED_NORMALS
	pass BlendNormal
	{
		RS_Z_BLENDED_NORMALS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendNormals();
	}
#endif

#ifdef USE_BLENDED_SNOW	
	pass BlendSnow
	{
		RS_Z_BLENDED_SNOW
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendSnow();
	}
#endif
	
	pass BlendCaustics
	{
		RS_Z_BLENDED_CAUSTICS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendCaustics();
	}
	
	pass BlendDeepSea
	{
		RS_Z_BLENDED_DEEP_SEA
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendDeepSea();
	}
	
	pass BlendFog
	{
		RS_Z_BLENDED_FOG
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendFog();
	}

    pass Grid
	{
		//FillMode = WireFrame;
		AlphaBlendEnable = true;
		ZEnable = true;
		
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformGrid();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSGrid();
	}
    	
}

// ================================================================================================
technique GroundTileTrippleSetDeepSea
{
	pass MaterialSet00
	{
		FillMode = <GROUND_FILL_MODE>;
			
		RS_Z_ALPHA_SOLID_SET
		SETUP_TEXTURE_STAGES
	
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformEnlightenGernerateUVDeepSea();
		//PixelShader = compile SHAD_PS_MINVER_1_1 PSDebug();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsSolidSet();
	}	
	
	pass MaterialSet01
	{
		RS_Z_ALPHA_BLENDED_SET
			
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet01();
	}
	
	pass MaterialSet02
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet02();
	}
	
#ifdef USE_BLENDED_NORMALS
	pass BlendNormal
	{
		RS_Z_BLENDED_NORMALS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendNormals();
	}
#endif

#ifdef USE_BLENDED_SNOW	
	pass BlendSnow
	{
		RS_Z_BLENDED_SNOW
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendSnow();
	}
#endif
	
	pass BlendCaustics
	{
		RS_Z_BLENDED_CAUSTICS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendCaustics();
	}
	
	pass BlendDeepSea
	{
		RS_Z_BLENDED_DEEP_SEA
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendDeepSea();
	}
	
	pass BlendFog
	{
		RS_Z_BLENDED_FOG
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendFog();
	}
	
    pass Grid
	{
		//FillMode = WireFrame;
		AlphaBlendEnable = true;
		ZEnable = true;
		
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformGrid();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSGrid();
	}
    	
}

// ================================================================================================
technique GroundTileQuadSetDeepSea
{
	pass MaterialSet00
	{
		FillMode = <GROUND_FILL_MODE>;
			
		RS_Z_ALPHA_SOLID_SET
		SETUP_TEXTURE_STAGES
	
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformEnlightenGernerateUVDeepSea();
		//PixelShader = compile SHAD_PS_MINVER_1_1 PSDebug();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsSolidSet();
		// PixelShader = compile SHAD_PS_MINVER_2_0 PSRednerBlendSet03();
	}	
	
	pass MaterialSet01
	{
		RS_Z_ALPHA_BLENDED_SET
			
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet01();
	}
	
	pass MaterialSet02
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet02();
	}
	
	pass MaterialSet03
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet03();
	}
	
#ifdef USE_BLENDED_NORMALS
	pass BlendNormal
	{
		RS_Z_BLENDED_NORMALS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendNormals();
	}
#endif // USE_BLENDED_NORMALS

#ifdef USE_BLENDED_SNOW	
	pass BlendSnow
	{
		RS_Z_BLENDED_SNOW
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendSnow();
	}
#endif // USE_BLENDED_SNOW
	
	pass BlendCaustics
	{
		RS_Z_BLENDED_CAUSTICS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendCaustics();
	}
	
	pass BlendDeepSea
	{
		RS_Z_BLENDED_DEEP_SEA
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendDeepSea();
	}
	
	pass BlendFog
	{
		RS_Z_BLENDED_FOG
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendFog();
	}
	
	pass Grid
	{
		//FillMode = WireFrame;
		AlphaBlendEnable = true;
		ZEnable = true;
		
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformGrid();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSGrid();
	}
    
}

// ================================================================================================
technique GroundTileQuintSetDeepSea
{
	pass MaterialSet00
	{
		FillMode = <GROUND_FILL_MODE>;
			
		RS_Z_ALPHA_SOLID_SET
		SETUP_TEXTURE_STAGES
	
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformEnlightenGernerateUVDeepSea();
		//PixelShader = compile SHAD_PS_MINVER_1_1 PSDebug();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsSolidSet();
	}	
	
	pass MaterialSet01
	{
		RS_Z_ALPHA_BLENDED_SET
				
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet01();
	}
	
	pass MaterialSet02
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet02();
	}
	
	pass MaterialSet03
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet03();
	}
	
	pass MaterialSet04
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet04();
	}

#ifdef USE_BLENDED_NORMALS
	pass BlendNormal
	{
		RS_Z_BLENDED_NORMALS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendNormals();
	}
#endif // USE_BLENDED_NORMALS

#ifdef USE_BLENDED_SNOW		
	pass BlendSnow
	{
		RS_Z_BLENDED_SNOW
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendSnow();
	}
#endif //USE_BLENDED_SNOW
	
	pass BlendCaustics
	{
		RS_Z_BLENDED_CAUSTICS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendCaustics();
	}
	
	pass BlendDeepSea
	{
		RS_Z_BLENDED_DEEP_SEA
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendDeepSea();
	}
	
	pass BlendFog
	{
		RS_Z_BLENDED_FOG
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendFog();
	}
	
	pass Grid
	{
		//FillMode = WireFrame;
		AlphaBlendEnable = true;
		ZEnable = true;
		
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformGrid();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSGrid();
	}
    
}

// ================================================================================================
technique GroundTileSextSetDeepSea
{
	pass MaterialSet00
	{
		FillMode = <GROUND_FILL_MODE>;
			
		RS_Z_ALPHA_SOLID_SET
		SETUP_TEXTURE_STAGES
	
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformEnlightenGernerateUVDeepSea();
		//PixelShader = compile SHAD_PS_MINVER_1_1 PSDebug();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsSolidSet();
	}	
	
	pass MaterialSet01
	{
		RS_Z_ALPHA_BLENDED_SET
				
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet01();
	}
	
	pass MaterialSet02
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet02();
	}
	
	pass MaterialSet03
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet03();
	}
	
	pass MaterialSet04
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet04();
	}
	
	pass MaterialSet05
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet05();
	}

#ifdef USE_BLENDED_NORMALS
	pass BlendNormal
	{
		RS_Z_BLENDED_NORMALS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendNormals();
	}
#endif // USE_BLENDED_NORMALS

#ifdef USE_BLENDED_SNOW		
	pass BlendSnow
	{
		RS_Z_BLENDED_SNOW
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendSnow();
	}
#endif // USE_	
	
	pass BlendCaustics
	{
		RS_Z_BLENDED_CAUSTICS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendCaustics();
	}
	
	pass BlendDeepSea
	{
		RS_Z_BLENDED_DEEP_SEA
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendDeepSea();
	}
	
	pass BlendFog
	{
		RS_Z_BLENDED_FOG
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendFog();
	}
	
	pass Grid
	{
		//FillMode = WireFrame;
		AlphaBlendEnable = true;
		ZEnable = true;
		
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformGrid();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSGrid();
	}
    
}

// ================================================================================================
technique GroundTileSeptSetDeepSea
{
	pass MaterialSet00
	{
		FillMode = <GROUND_FILL_MODE>;
			
		RS_Z_ALPHA_SOLID_SET
		SETUP_TEXTURE_STAGES
	
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformEnlightenGernerateUVDeepSea();
		//PixelShader = compile SHAD_PS_MINVER_1_1 PSDebug();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsSolidSet();
	}	
	
	pass MaterialSet01
	{
		RS_Z_ALPHA_BLENDED_SET
				
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet01();
	}
	
	pass MaterialSet02
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet02();
	}
	
	pass MaterialSet03
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet03();
	}
	
	pass MaterialSet04
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet04();
	}
	
	pass MaterialSet05
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet05();
	}
	
	pass MaterialSet06
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet06();
	}

#ifdef USE_BLENDED_NORMALS
	pass BlendNormal
	{
		RS_Z_BLENDED_NORMALS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendNormals();
	}
#endif // USE_BLENDED_NORMALS

#ifdef USE_BLENDED_SNOW		
	pass BlendSnow
	{
		RS_Z_BLENDED_SNOW
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendSnow();
	}
#endif

	pass BlendCaustics
	{
		RS_Z_BLENDED_CAUSTICS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendCaustics();
	}
	
	pass BlendDeepSea
	{
		RS_Z_BLENDED_DEEP_SEA
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendDeepSea();
	}
	
	pass BlendFog
	{
		RS_Z_BLENDED_FOG
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendFog();
	}
	
	pass Grid
	{
		//FillMode = WireFrame;
		AlphaBlendEnable = true;
		ZEnable = true;
		
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformGrid();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSGrid();
	}
    
}

// ================================================================================================
technique GroundTileOctSetDeepSea
{
	pass MaterialSet00
	{
		FillMode = <GROUND_FILL_MODE>;
			
		RS_Z_ALPHA_SOLID_SET
		SETUP_TEXTURE_STAGES
	
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformEnlightenGernerateUVDeepSea();
		//PixelShader = compile SHAD_PS_MINVER_1_1 PSDebug();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsSolidSet();
	}	
	
	pass MaterialSet01
	{
		RS_Z_ALPHA_BLENDED_SET
				
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet01();
	}
	
	pass MaterialSet02
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet02();
	}
	
	pass MaterialSet03
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet03();
	}
	
	pass MaterialSet04
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet04();
	}
	
	pass MaterialSet05
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet05();
	}
	
	pass MaterialSet06
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet06();
	}
	
	pass MaterialSet07
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet07();
	}
	
#ifdef USE_BLENDED_NORMALS
	pass BlendNormal
	{
		RS_Z_BLENDED_NORMALS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendNormals();
	}
#endif // USE_BLENDED_NORMALS

#ifdef USE_BLENDED_SNOW	
	pass BlendSnow
	{
		RS_Z_BLENDED_SNOW
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendSnow();
	}
#endif

	pass BlendCaustics
	{
		RS_Z_BLENDED_CAUSTICS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendCaustics();
	}
	
	pass BlendDeepSea
	{
		RS_Z_BLENDED_DEEP_SEA
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendDeepSea();
	}
	
	pass BlendFog
	{
		RS_Z_BLENDED_FOG
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendFog();
	}
	
	pass Grid
	{
		//FillMode = WireFrame;
		AlphaBlendEnable = true;
		ZEnable = true;
		
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformGrid();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSGrid();
	}
    
}

// ================================================================================================
technique GroundTileNonSetDeepSea
{
	pass MaterialSet00
	{
		FillMode = <GROUND_FILL_MODE>;
			
		RS_Z_ALPHA_SOLID_SET
		SETUP_TEXTURE_STAGES
	
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformEnlightenGernerateUVDeepSea();
		//PixelShader = compile SHAD_PS_MINVER_1_1 PSDebug();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsSolidSet();
	}	
	
	pass MaterialSet01
	{
		RS_Z_ALPHA_BLENDED_SET
				
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet01();
	}
	
	pass MaterialSet02
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet02();
	}
	
	pass MaterialSet03
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet03();
	}
	
	pass MaterialSet04
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet04();
	}
	
	pass MaterialSet05
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet05();
	}
	
	pass MaterialSet06
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet06();
	}
	
	pass MaterialSet07
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet07();
	}
	
	pass MaterialSet08
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet08();
	}

#ifdef USE_BLENDED_NORMALS
	pass BlendNormal
	{
		RS_Z_BLENDED_NORMALS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendNormals();
	}
#endif // USE_BLENDED_NORMALS

#ifdef USE_BLENDED_SNOW		
	pass BlendSnow
	{
		RS_Z_BLENDED_SNOW
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendSnow();
	}
#endif
	
	pass BlendCaustics
	{
		RS_Z_BLENDED_CAUSTICS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendCaustics();
	}
	
	pass BlendDeepSea
	{
		RS_Z_BLENDED_DEEP_SEA
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendDeepSea();
	}
	
	pass BlendFog
	{
		RS_Z_BLENDED_FOG
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendFog();
	}
	
	pass Grid
	{
		//FillMode = WireFrame;
		AlphaBlendEnable = true;
		ZEnable = true;
		
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformGrid();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSGrid();
	}
    
}

// ================================================================================================
technique GroundTileDecSetDeepSea
{
	pass MaterialSet00
	{
		FillMode = <GROUND_FILL_MODE>;
			
		RS_Z_ALPHA_SOLID_SET
		SETUP_TEXTURE_STAGES
	
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformEnlightenGernerateUVDeepSea();
		//PixelShader = compile SHAD_PS_MINVER_1_1 PSDebug();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsSolidSet();
	}	
	
	pass MaterialSet01
	{
		RS_Z_ALPHA_BLENDED_SET
				
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet01();
	}
	
	pass MaterialSet02
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet02();
	}
	
	pass MaterialSet03
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet03();
	}
	
	pass MaterialSet04
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet04();
	}
	
	pass MaterialSet05
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet05();
	}
	
	pass MaterialSet06
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet06();
	}
	
	pass MaterialSet07
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet07();
	}
	
	pass MaterialSet08
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet08();
	}
	
	pass MaterialSet09
	{
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendMaterialsBlendSet09();
	}
	
#ifdef USE_BLENDED_NORMALS
	pass BlendNormal
	{
		RS_Z_BLENDED_NORMALS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendNormals();
	}
#endif // USE_BLENDED_NORMALS

#ifdef USE_BLENDED_SNOW	
	pass BlendSnow
	{
		RS_Z_BLENDED_SNOW
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendSnow();
	}
#endif
	
	pass BlendCaustics
	{
		RS_Z_BLENDED_CAUSTICS
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendCaustics();
	}
	
	pass BlendDeepSea
	{
		RS_Z_BLENDED_DEEP_SEA
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendDeepSea();
	}
	
	pass BlendFog
	{
		RS_Z_BLENDED_FOG
		PixelShader = compile SHAD_PS_MINVER_2_0 PSBlendFog();
	}
	
	pass Grid
	{
		//FillMode = WireFrame;
		AlphaBlendEnable = true;
		ZEnable = true;
		
		VertexShader = compile SHAD_VS_MINVER_2_0 VSTransformGrid();
		PixelShader = compile SHAD_PS_MINVER_2_0 PSGrid();
	}
    
}
