// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2008 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author:   mad

#include "GlobalFX.h"

texture ShadowTex;

#define USE_INDIVIDUAL_SHADOW_TEXTURE_SLOT
#include <shadowmap.hlsl>

// ================================================================================================
#if defined(USE_SURFACE_NORMALTEXTURES) && defined(USE_BLENDED_NORMALS)
    #undef USE_BLENDED_NORMALS
#endif

#if defined(USE_SURFACE_NORMALTEXTURES) || defined(USE_BLENDED_NORMALS)
    #define USE_PS_LIGHTING 
#else
    #undef USE_PS_LIGHTING
#endif

// ================================================================================================
const int       GRID_TYPE     : GLOBAL = 0;

// ================================================================================================
struct VSIn
{
   float3 Pos				: POSITION;
   float3 Normal			: NORMAL;
   // by jow: Umstellung von COLOR auf TEXCOORD, da Gamebryo die Colors in einzelne bytes packt und es damit zu starken Rundungsfehlern kommt
   float4 TexBlendWeight	: COLOR;		// Blend Map weight in r
   float2 TexBlend			: TEXCOORD0;
   float2 TexSurface		: TEXCOORD1;
   float2 WaterHeight		: TEXCOORD2;
   //float4 TexBlendNCWeight	: COLOR;	// Blend Map UVs
   //float2 TexSurface		: TEXCOORD0;
   //float2 WaterHeight		: TEXCOORD1;
};

// ================================================================================================
struct VSOut
{
	float4 Pos				    : POSITION;
	float4 Color			    : COLOR;
	float4 PosVS				: TEXCOORD0;	// x,y,z = VS Position, w = Projected depth
	float4 DeepSeaWeightFog	    : TEXCOORD1;
    float4 TexBlend_Material00 	: TEXCOORD2;	// (xy)Blendmap, (zw)Material 3 * Stage
	float4 TexMaterial01_02	    : TEXCOORD3;	// (xy)Material (3 * Stage) + 1, (zw)Material 3 * Stage +2
	float3 WorldUp              : TEXCOORD4;
	
 #if defined(ENABLE_SHADOWMAPPING)
    float4 LightVPS              : TEXCOORD5;    // Schatten
 
    #ifdef USE_PS_LIGHTING
	    float3 LightDirTS        : TEXCOORD6;
   
		#ifdef ENABLE_SPECULAR_REFLECTIONS
			float3 CamDirTS			 : TEXCOORD7;
		#endif
   
//   #ifdef ENABLE_PARALLAX
//	    float2 vParallaxOffsetTS : TEXCOORD8;
//   #endif
	#endif
  
 #else  // ENABLE_SHADOWMAPPING
  
   #ifdef USE_PS_LIGHTING
	    float3 LightDirTS        : TEXCOORD5;
   
     #ifdef ENABLE_SPECULAR_REFLECTIONS
	    float3 CamDirTS			 : TEXCOORD6;
     #endif
   
//   #ifdef ENABLE_PARALLAX
//	    float2 vParallaxOffsetTS : TEXCOORD7;
//   #endif
   #endif // USE_PS_LIGHTING

#endif   // ENABLE_SHADOWMAPPING
};

// ================================================================================================
struct PSIn
{
	float4 Color 				: COLOR;
	float4 PosVS				: TEXCOORD0;	// x,y,z = VS Position, w = Projected depth
	float4 DeepSeaWeightFog		: TEXCOORD1;
	float4 TexBlend_Material00 	: TEXCOORD2;	// (xy)Blendmap, (zw)Material 3 * Stage
	float4 TexMaterial01_02	    : TEXCOORD3;	// (xy)Material (3 * Stage) + 1, (zw)Material 3 * Stage +2
	float3 WorldUp              : TEXCOORD4;
	
 #if defined(ENABLE_SHADOWMAPPING)
    float4 LightVPS              : TEXCOORD5;    // Schatten
 
    #ifdef USE_PS_LIGHTING
	    float3 LightDirTS        : TEXCOORD6;
    
        #ifdef ENABLE_SPECULAR_REFLECTIONS
	        float3 CamDirTS      : TEXCOORD7;
        #endif

//      #ifdef ENABLE_PARALLAX
//	    float2 vParallaxOffsetTS : TEXCOORD8;
//      #endif
    #endif  // USE_PS_LIGHTING
 
 #else // ENABLE_SHADOWMAPPING
 
    #ifdef USE_PS_LIGHTING
	    float3 LightDirTS        : TEXCOORD5;
    
        #ifdef ENABLE_SPECULAR_REFLECTIONS
	        float3 CamDirTS      : TEXCOORD6;
        #endif

//      #ifdef ENABLE_PARALLAX
//	    float2 vParallaxOffsetTS : TEXCOORD7;
//      #endif
    #endif // USE_PS_LIGHTING
    
 #endif    // ENABLE_SHADOWMAPPING
};

// ================================================================================================
#define RS_Z_ALPHA_SOLID_SET\
	ShadeMode = Gouraud;\
	CullMode = CW;\
	ZEnable = true;\
	ZWriteEnable = true;\
	AlphaTestEnable = false;\
	AlphaBlendEnable = false;\
	BlendOp = Add;\
	SrcBlend = SrcAlpha;\
	DestBlend = InvSrcAlpha;

#define RS_Z_ALPHA_BLENDED_SET\
	ZEnable = true;\
	ZWriteEnable = true;\
	AlphaBlendEnable = true;\
	BlendOp = Add;\
	DestBlend = One;\
	SrcBlend = One;

#define RS_Z_BLENDED_SNOW\
	ZEnable = true;\
	ZWriteEnable = true;\
	AlphaBlendEnable = true;\
	DestBlend = InvSrcAlpha;\
	SrcBlend = SrcAlpha;

#define RS_Z_BLENDED_NORMALS\
	ZEnable = true;\
	ZWriteEnable = true;\
	AlphaBlendEnable = true;\
	DestBlend = Zero;\
	SrcBlend = DestColor;
	
#define RS_Z_BLENDED_CAUSTICS\
	ZEnable = true;\
	ZWriteEnable = true;\
	AlphaBlendEnable = true;\
	ColorOp[0] = SelectArg1;\
	ColorArg1[0] = Texture;\
	DestBlend = One;\
	SrcBlend = One;

#define RS_Z_BLENDED_DEEP_SEA\
	ZEnable = true;\
	ZWriteEnable = true;\
	AlphaBlendEnable = true;\
	DestBlend = InvSrcAlpha;\
	SrcBlend = SrcAlpha;
	
#define RS_Z_BLENDED_FOG\
	ZEnable = true;\
	ZWriteEnable = true;\
	AlphaBlendEnable = true;\
	DestBlend = InvSrcAlpha;\
	SrcBlend = SrcAlpha;
	
#define SETUP_TEXTURE_STAGES\
	TexCoordIndex[0] = 0;\
	TextureTransformFlags[0] = Disable;\
	TexCoordIndex[1] = 1;\
	TextureTransformFlags[1] = Disable;\
	TexCoordIndex[2] = 2;\
	TextureTransformFlags[2] = Disable;\
	TexCoordIndex[3] = 3;\
	TextureTransformFlags[3] = Disable;

// ================================================================================================
// blend maps
// ================================================================================================
DECL_BLEND_SAMPLER(BlendMap00, TextureBlendMap00, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_BLEND_SAMPLER(BlendMap01, TextureBlendMap01, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_BLEND_SAMPLER(BlendMap02, TextureBlendMap02, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_BLEND_SAMPLER(BlendMap03, TextureBlendMap03, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_BLEND_SAMPLER(BlendMap04, TextureBlendMap04, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_BLEND_SAMPLER(BlendMap05, TextureBlendMap05, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_BLEND_SAMPLER(BlendMap06, TextureBlendMap06, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_BLEND_SAMPLER(BlendMap07, TextureBlendMap07, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_BLEND_SAMPLER(BlendMap08, TextureBlendMap08, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_BLEND_SAMPLER(BlendMap09, TextureBlendMap09, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)

DECL_BLEND_SAMPLER(BlendMapSnow, TextureBlendSnow, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)

// ================================================================================================
// texture maps
// ================================================================================================
#include <GroundTile_Samplers.hlsl>
/* by jow entfernt: in GroundTile_Samplers.hlsl verschoben, da durch das define DECL_SAMPLER probleme mit Point Filtering auftreten
DECL_SAMPLER(Surface00, TextureSurface00, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface01, TextureSurface01, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface02, TextureSurface02, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface03, TextureSurface03, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface04, TextureSurface04, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface05, TextureSurface05, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface06, TextureSurface06, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface07, TextureSurface07, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface08, TextureSurface08, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface09, TextureSurface09, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface10, TextureSurface10, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface11, TextureSurface11, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface12, TextureSurface12, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface13, TextureSurface13, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface14, TextureSurface14, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface15, TextureSurface15, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface16, TextureSurface16, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface17, TextureSurface17, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface18, TextureSurface18, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface19, TextureSurface19, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface20, TextureSurface20, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface21, TextureSurface21, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface22, TextureSurface22, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface23, TextureSurface23, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface24, TextureSurface24, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface25, TextureSurface25, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface26, TextureSurface26, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface27, TextureSurface27, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface28, TextureSurface28, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Surface29, TextureSurface29, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)

DECL_SAMPLER(Snow, TextureSnow, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
DECL_SAMPLER(Caust, TextureCaust, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
*/

// ================================================================================================
#ifdef USE_SURFACE_NORMALTEXTURES	
    DECL_SAMPLER(GlbNormal, GlbTextureNormal, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(SnowNormal, SnowTextureNormal, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface00, NormTexSurface00, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface01, NormTexSurface01, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface02, NormTexSurface02, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface03, NormTexSurface03, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface04, NormTexSurface04, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface05, NormTexSurface05, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface06, NormTexSurface06, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface07, NormTexSurface07, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface08, NormTexSurface08, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface09, NormTexSurface09, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface10, NormTexSurface10, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface11, NormTexSurface11, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface12, NormTexSurface12, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface13, NormTexSurface13, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface14, NormTexSurface14, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface15, NormTexSurface15, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface16, NormTexSurface16, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface17, NormTexSurface17, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface18, NormTexSurface18, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface19, NormTexSurface19, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface20, NormTexSurface20, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface21, NormTexSurface21, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface22, NormTexSurface22, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface23, NormTexSurface23, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface24, NormTexSurface24, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface25, NormTexSurface25, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface26, NormTexSurface26, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface27, NormTexSurface27, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface28, NormTexSurface28, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    DECL_SAMPLER(NormSurface29, NormTexSurface29, MIN_FILTERTYPE, MAG_FILTERTYPE, MIP_FILTERTYPE, MAX_ANISOTROPY)
    
#endif // USE_SURFACE_NORMALTEXTURES

// ================================================================================================
#ifdef ENABLE_SPECULAR_REFLECTIONS

    #define DECL_GLOSS_SAMPLER(S, T)\
    texture2D T;\
    sampler2D S = sampler_state\
    {\
    	Texture = (T);\
    	AddressU = Wrap;\
    	AddressV = Wrap;\
      	MagFilter = linear;\
      	MinFilter = linear;\
      	MipFilter = linear;\
      	MaxAnisotropy = 1;\
    };

    // glossiness 1x1 texture for each tile
    DECL_GLOSS_SAMPLER(GlossTile00, GlossTexTile00)
    DECL_GLOSS_SAMPLER(GlossTile01, GlossTexTile01)
    DECL_GLOSS_SAMPLER(GlossTile02, GlossTexTile02)
    DECL_GLOSS_SAMPLER(GlossTile03, GlossTexTile03)
    DECL_GLOSS_SAMPLER(GlossTile04, GlossTexTile04)
    DECL_GLOSS_SAMPLER(GlossTile05, GlossTexTile05)
    DECL_GLOSS_SAMPLER(GlossTile06, GlossTexTile06)
    DECL_GLOSS_SAMPLER(GlossTile07, GlossTexTile07)
    DECL_GLOSS_SAMPLER(GlossTile08, GlossTexTile08)
    DECL_GLOSS_SAMPLER(GlossTile09, GlossTexTile09)
#endif // ENABLE_SPECULAR_REFLECTIONS

#ifdef USE_SURFACE_NORMALTEXTURES	

    // ================================================================================================
    float4 GetNormalData( sampler2D p_fNormalTexture, float2 p_vTexCoords)
    {
        // return tex2D( p_fNormalTexture, p_vTexCoords).xyzw;
        // -> for Objectspace-Normals use this:
        float4 vNormal = tex2D( p_fNormalTexture, p_vTexCoords).xyzw;
        vNormal.xy = vNormal.xy * 2 - 1;
        vNormal.z = vNormal.z - 0.5;
        vNormal.xyz = normalize(vNormal.xyz);
        return vNormal;
        
        //return normalize(float3(vNormal.x * 2, vNormal.y *2, vNormal.z));
    }

    // ================================================================================================
    void ComputeNormalMapSet( out float4 vDiffuse, out float4 vSpecular, PSIn p_iPSIn, float3 p_vWeights,
							    sampler2D p_NormalTex0,
							    sampler2D p_NormalTex1,
							    sampler2D p_NormalTex2, 
							    float3 vSpecLevels, float3 vGlossiness )
    // ================================================================================================
    // with Specular reflections...
    // ================================================================================================
  #ifdef ENABLE_SPECULAR_REFLECTIONS
    {
        // normalize Vertex -> Cam(Eye) Vector
		// float3 vCamDir = normalize(p_iPSIn.CamDirTS); (mal weglassen)
		
        // die 3 Normalen werden hier auch ber das Blending verrechnet - ist
        // zwar nicht 100% richtig, weil Farben != Vektoren, aber erstmal so
		
    	// ----------------------
		// read out normals an blend them
		float3 fTexNorm  = GetNormalData(p_NormalTex0, p_iPSIn.TexBlend_Material00.zw).xyz * p_vWeights.x;
		fTexNorm       += GetNormalData(p_NormalTex1, p_iPSIn.TexMaterial01_02.xy).xyz * p_vWeights.y;
		fTexNorm       += GetNormalData(p_NormalTex2, p_iPSIn.TexMaterial01_02.zw).xyz * p_vWeights.z;
        fTexNorm        = normalize(fTexNorm);
        
    	// ----------------------
		// compute diffuse lighting
    	//vDiffuse.xyz = SUN_LIGHT_DIFFUSE * clamp(dot(fTexNorm, -p_iPSIn.LightDirTS), 0, 1);
    	vDiffuse.xyz = SUN_LIGHT_DIFFUSE * saturate(dot(fTexNorm, -p_iPSIn.LightDirTS));
        vDiffuse.w = 1.0;
        
        // ----------------------
		// compute specular lighting
        float fSpecLevel  = dot(p_vWeights, vSpecLevels);       // specular level   / pixel   -> compine them
        float fGlossiness = 100.f*dot(p_vWeights, vGlossiness); // glossiness level / texture -> compine them
        
        // 3ds max version using float-vectors
        float3 fHalf = normalize(p_iPSIn.CamDirTS - p_iPSIn.LightDirTS);
        //vSpecular.xyz = saturate(fSpecLevel * pow( clamp(dot(fHalf, fTexNorm),0,1), fGlossiness));
        vSpecular.xyz = saturate(fSpecLevel * pow( saturate(dot(fHalf, fTexNorm)), fGlossiness));
        vSpecular.xyz *= SUN_LIGHT_DIFFUSE;
        vSpecular.w = 1.0;
    }
    
  #else  // ENABLE_SPECULAR_REFLECTIONS
    // ================================================================================================
    // no Specular reflections...
    // ================================================================================================
    {
		// ----------------------
		// no specular
		vSpecular = float4(0,0,0,1);
  
		// ----------------------
		// read out normals an blend them
		float3 fTexNorm  = GetNormalData(p_NormalTex0, p_iPSIn.TexBlend_Material00.zw).xyz * p_vWeights.x;
		fTexNorm       += GetNormalData(p_NormalTex1, p_iPSIn.TexMaterial01_02.xy).xyz * p_vWeights.y;
		fTexNorm       += GetNormalData(p_NormalTex2, p_iPSIn.TexMaterial01_02.zw).xyz * p_vWeights.z;
        // fTexNorm    += float3(0,0,1); -> muss rein, wenn es KEINE Normal gibt :<
        fTexNorm = normalize(fTexNorm);
        
        // ----------------------
		// compute diffuse lighting
    	//vDiffuse.xyz = SUN_LIGHT_DIFFUSE * clamp(dot(fTexNorm, -p_iPSIn.LightDirTS), 0, 1);
    	vDiffuse.xyz = SUN_LIGHT_DIFFUSE * saturate(dot(fTexNorm, -p_iPSIn.LightDirTS));
        vDiffuse.w = 1.0;
    }
  #endif // ENABLE_SPECULAR_REFLECTIONS

 #ifdef ENABLE_PARALLAX
	// ================================================================================================
    void ComputeParallaxOffset(inout float2 TexCoords, sampler2D HeightTexture, float3 vViewTS)
	{
	    static const float Scale =  0.03; //0.012;
		static const float Bias  = -0.02; //-0.01;

		float fCurrentHeight = tex2D(HeightTexture, TexCoords).w;
		float fHeight        = fCurrentHeight * Scale + Bias;
		// fHeight             /= vViewTS.z;
		TexCoords           += vViewTS.xy * fHeight;
	}
 #endif // ENABLE_PARALLAX
  	
    // ================================================================================================
    PS_Out PSBlendMaterialsSolidSet(PSIn i_PSIn, sampler2D BlendMap00, sampler2D Surface00, sampler2D Surface01, sampler2D Surface02,
                                    sampler2D NormSurface00, sampler2D NormSurface01, sampler2D NormSurface02, float3 Glossiness, uniform float DepthFactor = 0.0f)
    {
		PS_Out output = (PS_Out)0;

	    float4 Weights = tex2D( BlendMap00, i_PSIn.TexBlend_Material00.xy );

 #ifdef ENABLE_PARALLAX
		ComputeParallaxOffset(i_PSIn.TexBlend_Material00.zw, NormSurface00, i_PSIn.CamDirTS );
		ComputeParallaxOffset(i_PSIn.TexMaterial01_02.xy, NormSurface01, i_PSIn.CamDirTS );
		ComputeParallaxOffset(i_PSIn.TexMaterial01_02.zw, NormSurface02, i_PSIn.CamDirTS );
 #endif // ENABLE_PARALLAX

	    float4 ColorMaterial00 = tex2D( Surface00, i_PSIn.TexBlend_Material00.zw );
	    float4 ColorMaterial01 = tex2D( Surface01, i_PSIn.TexMaterial01_02.xy );
	    float4 ColorMaterial02 = tex2D( Surface02, i_PSIn.TexMaterial01_02.zw );
    	
	    float4 Color = (	ColorMaterial00 * Weights.x +
						ColorMaterial01 * Weights.y +
						ColorMaterial02 * Weights.z ) * i_PSIn.Color;
        
        // calculate Lighting with Texturecolor
        float4 OutColor;
        
 #if defined(ENABLE_SHADOWMAPPING) && !defined(VARIANCE_SHADOW_MAPPING)
        // by mad: um schatten erweitert
        float fCurrentDepth = GetDepthInLightSpace(i_PSIn.LightVPS);
        float fDepth = GetProjDepthValue(i_PSIn.LightVPS, ShadowMap);
 #endif
 
        OutColor.xyz = SUN_LIGHT_AMBIENT * Color.xyz;
        OutColor.w   = Color.w;
        
 #if defined(VARIANCE_SHADOW_MAPPING)        
        // calculate Diffuse and Specular Lightings...
        float3 vSpecLevels = float3(ColorMaterial00.w, ColorMaterial01.w, ColorMaterial02.w);
    	
        float4 vDiffuse, vSpecular;
        ComputeNormalMapSet(vDiffuse, vSpecular, i_PSIn, Weights.xyz, 
                            NormSurface00, NormSurface01, NormSurface02,
                            vSpecLevels, Glossiness);
        // by mad: um schatten erweitert
        float fCurrentDepth = GetDepthInLightSpace(i_PSIn.LightVPS);
        float fLit = GetObjectInShadowFactor(fCurrentDepth, i_PSIn.LightVPS, ShadowMap);
        
        // 3ds max methode:
        OutColor.xyz += fLit*(vDiffuse * Color.xyz + vSpecular);
 
 #else // !VARIANCE_SHADOW_MAPPING
 
 #  if defined(ENABLE_SHADOWMAPPING)
        if (fDepth < fCurrentDepth)
        {    
 #  endif
            // calculate Diffuse and Specular Lightings...
		    float3 vSpecLevels = float3(ColorMaterial00.w, ColorMaterial01.w, ColorMaterial02.w);
        	
		    float4 vDiffuse, vSpecular;
		    ComputeNormalMapSet(vDiffuse, vSpecular, i_PSIn, Weights.xyz, 
		                        NormSurface00, NormSurface01, NormSurface02,
		                        vSpecLevels, Glossiness);
            // 3ds max methode:
            OutColor.xyz += vDiffuse * Color.xyz + vSpecular;
 #  if defined(ENABLE_SHADOWMAPPING)
        }
 #  endif  
 #endif // VARIANCE_SHADOW_MAPPING
        /*if (fDepth < fCurrentDepth && fCurrentDepth <= 1)
        {
            OutColor.xyz = SUN_LIGHT_AMBIENT * Color.xyz;
        }
        else
        {
		    // calculate Diffuse and Specular Lightings...
		    float3 vSpecLevels = float3(ColorMaterial00.w, ColorMaterial01.w, ColorMaterial02.w);
    		
		    float4 vDiffuse, vSpecular;
		    ComputeNormalMapSet(vDiffuse, vSpecular, i_PSIn, Weights.xyz, 
		                        NormSurface00, NormSurface01, NormSurface02,
		                        vSpecLevels, Glossiness);
    		                    
            
            // 3ds max methode:
            OutColor.xyz = SUN_LIGHT_AMBIENT * Color.xyz + vDiffuse * Color.xyz + vSpecular;
		    // OutColor.xyz = SUN_LIGHT_AMBIENT * Color.xyz + fFactor* vDiffuse * Color.xyz + fFactor * vSpecular;
            
            // old methode:
            //OutColor.xyz = min(1, SUN_LIGHT_AMBIENT + vDiffuse) * Color.xyz + vSpecular.xyz;
        }*/
		output.Color = OutColor;

#if defined(USE_DEPTH_MAP_MRT)
		output.Depth.x = (1.0f - i_PSIn.PosVS.w) * DepthFactor;		// Projected depth
		output.Depth.y = i_PSIn.PosVS.z * DepthFactor;			// Linear view space position
		output.Depth.a = 1.0f;						// Do not blend any depth values!
#endif

        return output;
    }

    // ================================================================================================
    void UpdateLighting(VSIn i_VSIn, inout VSOut o_VSOut)
    {
	    float3 LightDirOS = normalize( mul( SUN_LIGHT_DIRECTION_WS, matInverseWorld ) );

        // by mad: transform light vector from object to tangent space
        //o_VSOut.Color = clamp(lerp( GROUND_BLEND_COLOR_MIN, GROUND_BLEND_COLOR_MAX, i_VSIn.TexBlendNCWeight.z ),0,1);
        //o_VSOut.Color = saturate(lerp( GROUND_BLEND_COLOR_MIN, GROUND_BLEND_COLOR_MAX, i_VSIn.TexBlendNCWeight.z ));
		o_VSOut.Color = saturate(lerp( GROUND_BLEND_COLOR_MIN, GROUND_BLEND_COLOR_MAX, i_VSIn.TexBlendWeight.r ));
        o_VSOut.Color.w = 1.0f;
    	// o_VSOut.Color = 1;
        
        // -> neue Methode...
        float3 ny = normalize(cross(i_VSIn.Normal, float3(1,0,0)));
        float3 nx = normalize(cross(i_VSIn.Normal, float3(0,1,0)));
    	
        o_VSOut.LightDirTS.x = dot(nx, LightDirOS);
        o_VSOut.LightDirTS.y = dot(ny, LightDirOS);
        o_VSOut.LightDirTS.z = dot(i_VSIn.Normal, LightDirOS);
        o_VSOut.LightDirTS   = normalize(o_VSOut.LightDirTS);

        o_VSOut.WorldUp.x = dot(nx, float3(0,0,1));
        o_VSOut.WorldUp.y = dot(ny, float3(0,0,1));
        o_VSOut.WorldUp.z = dot(i_VSIn.Normal, float3(0,0,1));
        o_VSOut.WorldUp   = normalize(o_VSOut.WorldUp);
        
 #ifdef ENABLE_SPECULAR_REFLECTIONS       
        float4 CamDirVecOS = mul(float4(CAMERA_POS_WS.xyz,1.0), matInverseWorld);	// transform cam-pos in objects space
        CamDirVecOS.xyz /= CamDirVecOS.w;                                   // important: use 4x4 matrix, because we modify a position!
        CamDirVecOS.xyz -= i_VSIn.Pos.xyz;						            // and get direction from current pos
		
		o_VSOut.CamDirTS.x = dot(nx, CamDirVecOS.xyz);
        o_VSOut.CamDirTS.y = dot(ny, CamDirVecOS.xyz);
        o_VSOut.CamDirTS.z = dot(i_VSIn.Normal, CamDirVecOS.xyz);

        
    #ifdef ENABLE_PARALLAX_SPECIAL   // unused     
		// Compute initial parallax displacement direction:
		float2 vParallaxDirection = normalize( o_VSOut.CamDirTS.xy );
	       
		// The length of this vector determines the furthest amount of displacement:
		float fLength         = length( o_VSOut.CamDirTS );
		float fParallaxLength = sqrt( fLength * fLength - o_VSOut.CamDirTS.z * o_VSOut.CamDirTS.z ) / o_VSOut.CamDirTS.z; 
	       
		// Compute the actual reverse parallax displacement vector:
		Out.vParallaxOffsetTS = vParallaxDirection * fParallaxLength;
	       
		// Need to scale the amount of displacement to account for different height ranges
		// in height maps. This is controlled by an artist-editable parameter:
		Out.vParallaxOffsetTS *= 1; //g_fHeightMapScale;        
	#endif // ENABLE_PARALLAX

        o_VSOut.CamDirTS   = normalize(o_VSOut.CamDirTS);
 #endif	
    }

#else  // USE_SURFACE_NORMALTEXTURES

    // ================================================================================================
    PS_Out PSBlendMaterialsSolidSet(PSIn i_PSIn, sampler2D BlendMap00, sampler2D Surface00, sampler2D Surface01, sampler2D Surface02, uniform float DepthFactor = 0.0f)
    {
		PS_Out output = (PS_Out)0;

	    float4 Weights = tex2D( BlendMap00, i_PSIn.TexBlend_Material00.xy );
	    float4 ColorMaterial00 = tex2D( Surface00, i_PSIn.TexBlend_Material00.zw );
	    float4 ColorMaterial01 = tex2D( Surface01, i_PSIn.TexMaterial01_02.xy );
	    float4 ColorMaterial02 = tex2D( Surface02, i_PSIn.TexMaterial01_02.zw );
    	
	    float4 Color = (	ColorMaterial00 * Weights.x +
						    ColorMaterial01 * Weights.y +
						    ColorMaterial02 * Weights.z ) * i_PSIn.Color;		
		
 #if defined(ENABLE_SHADOWMAPPING)
        // by jow: added shadow factor
        float fCurrentDepth = GetDepthInLightSpace(i_PSIn.LightVPS);
		float fShadow = GetObjectInShadowFactor(fCurrentDepth, i_PSIn.LightVPS, ShadowMap);

		Color.xyz *= fShadow;
		//Color.xyz = float3(fShadow, fShadow, fShadow);
 #endif
		
		output.Color = Color;

#if defined(USE_DEPTH_MAP_MRT)
		output.Depth.x = (1.0f - i_PSIn.PosVS.w) * DepthFactor;	// Projected Depth
		output.Depth.y = i_PSIn.PosVS.z * DepthFactor;		// Linear view space depth
		output.Depth.a = 1.0f;
#endif

        return output;
    }

    // ================================================================================================
    void UpdateLighting(VSIn i_VSIn, inout VSOut o_VSOut)
    {
	    float3 LightDirOS = normalize( mul( SUN_LIGHT_DIRECTION_WS, matInverseWorld ) );
	    float3 A = SUN_LIGHT_AMBIENT;
	    float3 D = SUN_LIGHT_DIFFUSE;

        //o_VSOut.WorldUp.x = dot(nx, float3(0,0,1));
        //o_VSOut.WorldUp.y = dot(ny, float3(0,0,1));
        //o_VSOut.WorldUp.z = dot(i_VSIn.Normal, float3(0,0,1));
        o_VSOut.WorldUp   = normalize(i_VSIn.Normal);

	    o_VSOut.Color.xyz = EnlightenAD( i_VSIn.Pos, normalize(i_VSIn.Normal), LightDirOS, A, D );
	    //o_VSOut.Color *= lerp( GROUND_BLEND_COLOR_MIN, GROUND_BLEND_COLOR_MAX, i_VSIn.TexBlendNCWeight.z );
		o_VSOut.Color *= lerp( GROUND_BLEND_COLOR_MIN, GROUND_BLEND_COLOR_MAX, i_VSIn.TexBlendWeight.r );
	    o_VSOut.Color.w = 1.0f;
    }

#endif // USE_SURFACE_NORMALTEXTURES

// ================================================================================================
#ifdef USE_SURFACE_NORMALTEXTURES
    #define PSBlendMatSet(i_PSIn, BlendMap, Surface0, Surface1, Surface2,\
                              NormSurface0, NormSurface1, NormSurface2, Glossiness, DepthFactor)\
    PSBlendMaterialsSolidSet(i_PSIn, BlendMap, Surface0, Surface1, Surface2,\
                              NormSurface0, NormSurface1, NormSurface2, Glossiness, DepthFactor)
#else
    #define PSBlendMatSet(i_PSIn, BlendMap, Surface0, Surface1, Surface2,\
                              NormSurface0, NormSurface1, NormSurface2, Glossiness, DepthFactor)\
    PSBlendMaterialsSolidSet(i_PSIn, BlendMap, Surface0, Surface1, Surface2, DepthFactor)
#endif

// ================================================================================================
PS_Out PSBlendMaterialsSolidSet( PSIn i_PSIn, uniform float DepthFactor = 1.0f )
{
    //return PSBlendMatSet(i_PSIn, BlendMap00, Surface00, Surface01, Surface02,  NormSurface00, NormSurface01, NormSurface02, SURFACE_GLOSSINESS00);
    return PSBlendMatSet(i_PSIn, BlendMap00, Surface00, Surface01, Surface02,  NormSurface00, NormSurface01, NormSurface02, tex2D(GlossTile00, float2(0,0)).xyz, DepthFactor);
}

// ================================================================================================
PS_Out PSBlendMaterialsBlendSet01( PSIn i_PSIn, uniform float DepthFactor = 0.0f )
{
	return PSBlendMatSet(i_PSIn, BlendMap01, Surface03, Surface04, Surface05, NormSurface03, NormSurface04, NormSurface05, tex2D(GlossTile01, float2(0,0)).xyz, DepthFactor);
}

// ================================================================================================
PS_Out PSBlendMaterialsBlendSet02( PSIn i_PSIn, uniform float DepthFactor = 0.0f )
{
	return PSBlendMatSet(i_PSIn, BlendMap02, Surface06, Surface07, Surface08, NormSurface06, NormSurface07, NormSurface08, tex2D(GlossTile02, float2(0,0)).xyz, DepthFactor);
}

// ================================================================================================
PS_Out PSBlendMaterialsBlendSet03( PSIn i_PSIn, uniform float DepthFactor = 0.0f )
{
	return PSBlendMatSet(i_PSIn, BlendMap03, Surface09, Surface10, Surface11, NormSurface09, NormSurface10, NormSurface11, tex2D(GlossTile03, float2(0,0)).xyz, DepthFactor);
}

// ================================================================================================
PS_Out PSBlendMaterialsBlendSet04( PSIn i_PSIn, uniform float DepthFactor = 0.0f )
{
    return PSBlendMatSet(i_PSIn, BlendMap04, Surface12, Surface13, Surface14, NormSurface12, NormSurface13, NormSurface14, tex2D(GlossTile04, float2(0,0)).xyz, DepthFactor);
}

// ================================================================================================
PS_Out PSBlendMaterialsBlendSet05( PSIn i_PSIn, uniform float DepthFactor = 0.0f )
{
	return PSBlendMatSet(i_PSIn, BlendMap05, Surface15, Surface16, Surface17, NormSurface15, NormSurface16, NormSurface17, tex2D(GlossTile05, float2(0,0)).xyz, DepthFactor);
}

// ================================================================================================
PS_Out PSBlendMaterialsBlendSet06( PSIn i_PSIn, uniform float DepthFactor = 0.0f )
{
	return PSBlendMatSet(i_PSIn, BlendMap06, Surface18, Surface19, Surface20, NormSurface18, NormSurface19, NormSurface20, tex2D(GlossTile06, float2(0,0)).xyz, DepthFactor);
}

// ================================================================================================
PS_Out PSBlendMaterialsBlendSet07( PSIn i_PSIn, uniform float DepthFactor = 0.0f )
{
	return PSBlendMatSet(i_PSIn, BlendMap07, Surface21, Surface22, Surface23, NormSurface21, NormSurface22, NormSurface23, tex2D(GlossTile07, float2(0,0)).xyz, DepthFactor);
}

// ================================================================================================
PS_Out PSBlendMaterialsBlendSet08( PSIn i_PSIn, uniform float DepthFactor = 0.0f )
{
	return PSBlendMatSet(i_PSIn, BlendMap08, Surface24, Surface25, Surface26, NormSurface24, NormSurface25, NormSurface26, tex2D(GlossTile08, float2(0,0)).xyz, DepthFactor);
}

// ================================================================================================
PS_Out PSBlendMaterialsBlendSet09( PSIn i_PSIn, uniform float DepthFactor = 0.0f )
{
	return PSBlendMatSet(i_PSIn, BlendMap09, Surface27, Surface28, Surface29, NormSurface27, NormSurface28, NormSurface29, tex2D(GlossTile09, float2(0,0)).xyz, DepthFactor);
}

#ifdef USE_BLENDED_NORMALS

    // ================================================================================================
    PS_Out PSBlendNormals( PSIn i_PSIn )
    {
		PS_Out output = (PS_Out)0;

	    float3 fTexNorm = tex2D( GlbNormal, i_PSIn.TexMaterial00 ).xyz;
	    //fTexNorm = fTexNorm * 2.0 - 1.0; // -> for Objectspace-Normals 
	    fTexNorm = normalize(fTexNorm);
    	
	    float4 Color;
	    Color.xyz = EnlightenAD( float3(0,0,0), fTexNorm, i_PSIn.LightDirTS, SUN_LIGHT_AMBIENT, SUN_LIGHT_DIFFUSE);
	    Color.w = 1.0;
	    Color.xyz *= i_PSIn.Color.xyz;

		output.Color = Color;

#if defined(USE_DEPTH_MAP_MRT)
		output.Depth.x = 1.0f - i_PSIn.PosVS.w;		// Projected depth
		output.Depth.y = i_PSIn.PosVS.z;			// Linear view space position
		output.Depth.a = 1.0f;						// Do not blend depth values!
#endif

	    return output;
    }
    
#else // USE_BLENDED_NORMALS
    
    // ================================================================================================
    PS_Out PSBlendNormals( PSIn i_PSIn )
    {
		PS_Out output = (PS_Out)0;

		output.Color = float4(1, 1, 1, 1);

#if defined(USE_DEPTH_MAP_MRT)
		output.Depth.x = 1.0f - i_PSIn.PosVS.w;		// Projected depth
		output.Depth.y = i_PSIn.PosVS.z;			// Linear view space position
		output.Depth.a = 1.0f;						// Do not blend depth values!
#endif

        return output;
    }
    
#endif	// USE_BLENDED_NORMALS

// ================================================================================================
PS_Out PSBlendSnow( PSIn i_PSIn )
{
	PS_Out output = (PS_Out)0;

#ifdef USE_BLENDED_SNOW

	float4 Weight = tex2D( BlendMapSnow, i_PSIn.TexBlend_Material00.xy );
	float4 Color = tex2D( Snow, i_PSIn.TexBlend_Material00.zw );
	float fArea  = Color.a;
	Color.a = 1.0;
	
    float3 norm = float3(0,0,1);
    
  #ifdef USE_SURFACE_NORMALTEXTURES
    float4 normTex = tex2D( SnowNormal, i_PSIn.TexBlend_Material00.zw );
    float fSpec = normTex.a;
	norm = normTex.xyz;
    norm = norm * 2 - 1;
    norm = normalize(norm);
  #endif // USE_SURFACE_NORMALTEXTURES
    
    float4 diffuseLight = i_PSIn.Color;
    float4 specularLight = float4(0,0,0,1);

  #ifdef USE_PS_LIGHTING
    diffuseLight.xyz *= 0.51 * saturate(dot(norm, -i_PSIn.LightDirTS));
    
    #ifdef ENABLE_SPECULAR_REFLECTIONS
      const float cfSnowGlossiness = 30.0;
      const float cfSnowSpecLevel  = 0.6;
    
      float3 fHalf = normalize(i_PSIn.CamDirTS - i_PSIn.LightDirTS);
      //specularLight.xyz += saturate(fSpec * cfSnowSpecLevel * pow( clamp(dot(norm,fHalf),0,1), cfSnowGlossiness));
      specularLight.xyz += saturate(fSpec * cfSnowSpecLevel * pow( saturate(dot(norm,fHalf)), cfSnowGlossiness));
    #endif // ENABLE_SPECULAR_REFLECTIONS
  #else    // USE_PS_LIGHTING
    diffuseLight.xyz = 0.39 * saturate(diffuseLight);
  #endif   // USE_PS_LIGHTING
    
    diffuseLight.xyz  *= SUN_LIGHT_DIFFUSE;
    specularLight.xyz *= SUN_LIGHT_DIFFUSE;

    
  #if defined(ENABLE_SHADOWMAPPING)
    float fCurrentDepth = GetDepthInLightSpace(i_PSIn.LightVPS);
    float fShadow = GetObjectInShadowFactor(fCurrentDepth, i_PSIn.LightVPS, ShadowMap);
    
    // Color *= saturate(0.5 + fShadow);
    Color.xyz = ((float3(0.50,0.51,0.55) + diffuseLight * fShadow)*Color);
    //Color.xyz   = saturate(max(SUN_LIGHT_AMBIENT, diffuseLight * fShadow)*Color);
    //Color.xyz   = max(SUN_LIGHT_AMBIENT, diffuseLight * fShadow)*Color;
    //Color.xyz = saturate((SUN_LIGHT_AMBIENT + diffuseLight * fShadow)*Color);
    //Color.xyz = saturate(SUN_LIGHT_AMBIENT+diffuseLight)*Color;
    Color.xyz += fShadow*specularLight;
    Color.a   *= i_PSIn.Color.a;
  #else  // ENABLE_SHADOWMAPPING
    
    Color.xyz = ((float3(0.50,0.51,0.55) + diffuseLight)*Color);
    //Color.xyz  = saturate(max(SUN_LIGHT_AMBIENT, diffuseLight)*Color);
    Color.xyz += specularLight;
    // Color   = saturate(Color * (diffuseLight + SUN_LIGHT_AMBIENT));
    Color.a   *= i_PSIn.Color.a;
  #endif // ENABLE_SHADOWMAPPING
	
    float fDotUp = dot(norm, i_PSIn.WorldUp);
    
	// calc Snow-Area with Weight...
	float factorAlpha = 0;
    Weight.a *= saturate(1.5*pow(fDotUp, 1.5)+0.1);
    //return float4(Weight.a,0,0,1);
	//if (Weight.a > 0)
	//    factorAlpha = clamp((Weight.a - (1 - fArea)) / Weight.a, 0, 1);
	
	float fWaterLevel = saturate((i_PSIn.DeepSeaWeightFog.z - WaterHeight + 5)*0.1);
	Weight.a *= fWaterLevel;
	
	//factorAlpha = (0.25+Weight.a*0.75)*clamp((Weight.a - (1 - fArea)) / 0.18,0,1);
	factorAlpha = (0.25+Weight.a*0.75)*saturate((Weight.a - (1 - fArea)) / 0.18);
	//factorAlpha = clamp((Weight.a - (1 - fArea)) / 0.18,0,1);

	//return float4(factorAlpha, 0,0,1);
	
	Color.a = min(1.0f, factorAlpha);
	// Color.a = Weight.a * min( 1.0f, Color.a + Weight.a);
	output.Color = Color;
#else  // USE_BLENDED_SNOW
	output.Color = float4(1, 1, 1, 0);
#endif // USE_BLENDED_SNOW

	return output;
}

// ================================================================================================
float4 GridColor(float2 p_vTexCoord, float4 p_vColor)
{
#define STEPS 100.0 //0.29
#define BORDER 3 // 0.004
    
	// by mak erweitert um GRID_TYPE
//    float fScale   = (GRID_TYPE == 3) ? 0.5 : 1.0;
    float fScale   = (GRID_TYPE >= 3) ? 0.5 : 1.0;
	// by jow Umstellung des Terrain Offsets
    // float2 fOffset = (float2)((GRID_TYPE == 3) ? 50.f : 0.0);   //Nur mit USE_TERRAIN_CELL_OFFSET
	float2 fOffset = float2(0.0f, 0.0f);
    float2 texCoord = (fScale * p_vTexCoord.xy + fOffset) % STEPS;
    float  factor = 0.0;
    if (texCoord.x <= BORDER || texCoord.x >= STEPS - BORDER ||
        texCoord.y <= BORDER || texCoord.y >= STEPS - BORDER)
    {
        float f1 = max((BORDER - texCoord.y)/BORDER, (BORDER - texCoord.x)/BORDER);
        float f2 = max((texCoord.y - (STEPS-BORDER))/BORDER, (texCoord.x - (STEPS-BORDER))/BORDER);
        factor   = max(f1,f2);
        //factor = 1.0;
    }
    
    if (GRID_TYPE == 2)
    {
        #define SUBSTEPS  (0.5*STEPS)
        #define SUBBORDER (0.5*BORDER)
        
        float2 texCoord = (p_vTexCoord.xy) % SUBSTEPS;
        float  subFactor = 0.0;
        if (texCoord.x <= SUBBORDER || texCoord.x >= SUBSTEPS - SUBBORDER ||
            texCoord.y <= SUBBORDER || texCoord.y >= SUBSTEPS - SUBBORDER)
        {
            float f1 = max((SUBBORDER - texCoord.y)/SUBBORDER, (SUBBORDER - texCoord.x)/SUBBORDER);
            float f2 = max((texCoord.y - (SUBSTEPS-SUBBORDER))/SUBBORDER, (texCoord.x - (SUBSTEPS-SUBBORDER))/SUBBORDER);
            subFactor   = max(f1,f2);
            //factor = 1.0;
        }
        
        factor = max(factor, 0.5*subFactor);
    }
    
    float4 color = p_vColor;
    color.a = factor;
    return color;
}
