// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2007 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: ank
// date: 18.08.2006

#include "Skinning.h"

const float4 PickColor          : GLOBAL;
const bool	g_EnableAlphaTest	: GLOBAL = true;

#define PickRenderStates\
    FillMode            = solid;\
	ShadeMode           = flat;\
	CullMode            = cw;\
	ZEnable             = true;\
	ZWriteEnable        = true;\
	AlphaRef            = 5;\
	AlphaFunc           = greater;\
	AlphaBlendEnable    = false;\
	Lighting            = false;\
	SpecularEnable      = false;\
	FogEnable           = false;\
	ColorOp[0]          = SelectArg1;\
	ColorArg1[0]        = Current;\
	AlphaOp[0]          = Disable;\
	AlphaArg1[0]        = Current;\
	AddressU[0]         = Clamp;\
	AddressV[0]         = Clamp;\
	MinFilter[0]        = Point;\
    MagFilter[0]        = Point;\
    MipFilter[0]        = Point;\
    MaxAnisotropy[0]    = 1

// ================================================================================================
struct VS_OutPicking
{
	float4 Position : POSITION;
	float4 Color : COLOR;
	float2 TexCoords : TEXCOORD0;

#if defined(USE_COLOR_PICKER_MRT)
	float4 PositionVS : TEXCOORD1;
#endif
};

// ================================================================================================
struct PS_OutPicking
{
	float4 Color : COLOR0;

#if defined(USE_COLOR_PICKER_MRT)
	float4 Depth : COLOR1;	// x = linear view space z; y, z, w unused
#endif
};

// ================================================================================================
// Vertex Shaders for color picking
// ================================================================================================
VS_OutPicking VS_PickingNoTex(VS_OutNormal input)
{
	VS_OutPicking output = (VS_OutPicking)0;
	output.Position = mul(float4(input.Pos, 1.0f), matWorldViewProjection);
	output.Color = PickColor;

#if defined(USE_COLOR_PICKER_MRT)
	output.PositionVS = mul(float4(input.Pos, 1.0f), matWorldView);
#endif
    
    return output;
}

// ================================================================================================
VS_OutPicking VS_PickingTex(VS_OutNormalTex input)
{
	VS_OutNormal vsOutNormal;
	vsOutNormal.Pos = input.Pos;
	vsOutNormal.Normal = input.Normal;

	VS_OutPicking output = VS_PickingNoTex(vsOutNormal);

	output.TexCoords = input.Tex;

    return output;
}

// ================================================================================================
VS_OutPicking VS_PickingSkinned(VS_InSkinning i, uniform int NumBones)
{
	// Pos is in WS
    float3 Pos = SkinningPosOnly(i.Pos, i.BlendWeights, i.BlendIndices, NumBones);

	VS_OutPicking output = (VS_OutPicking)0;
	output.Position = mul(float4(Pos, 1.0f), matViewProj);
	output.Color = PickColor;
	output.TexCoords = i.Tex;

#if defined(USE_COLOR_PICKER_MRT)
	output.PositionVS = mul(float4(Pos, 1.0f), matView);
#endif

    return output;
}

// ================================================================================================
// Pixel Shaders for color picking
// ================================================================================================
PS_OutPicking PS_PickingStandard(VS_OutPicking input)
{
	PS_OutPicking output = (PS_OutPicking)0;

	output.Color = input.Color;

#if defined(USE_COLOR_PICKER_MRT)
	output.Depth.x = input.PositionVS.z;
#endif

	return output;
}

// ================================================================================================
PS_OutPicking PS_PickingAlphaTexture(VS_OutPicking input)
{
	PS_OutPicking output = PS_PickingStandard(input);

	float alpha = tex2D(SamplerBase, input.TexCoords).a;
	output.Color.a = alpha;

	return output;
}

// ================================================================================================
PS_OutPicking PS_PickingAlphaTextureSecondBlend(VS_OutPicking input)
{
	PS_OutPicking output = PS_PickingAlphaTexture(input);

	float alpha = tex2D(Sampler4, input.TexCoords).a;
	output.Color.a = ((alpha + 0.001f) >= g_fSecondColorDecay) ? output.Color.a : 0.0f;

	return output;
}

// ================================================================================================
// Techniques for color picking
// ================================================================================================
technique UnskinnedPicking_NoTex
<
	string Description = "Picking shader (no texture)";
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	            
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_PickingNoTex();
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_PickingStandard();
    }
}

// ================================================================================================
technique UnskinnedPicking
<
	string Description = "Picking shader";
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        PickRenderStates;
	    //AlphaTestEnable = true;
		AlphaTestEnable = <g_EnableAlphaTest>;

        VertexShader = compile SHAD_VS_MINVER_1_1 VS_PickingTex();
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_PickingAlphaTexture();
    }
}

// ================================================================================================
technique UnskinnedPicking_Special
<
	string Description = "Picking shader special";
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        PickRenderStates;
	    //AlphaTestEnable = true;
		AlphaTestEnable = <g_EnableAlphaTest>;

        VertexShader = compile SHAD_VS_MINVER_1_1 VS_PickingTex();
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_PickingAlphaTextureSecondBlend();
    }
}

// ================================================================================================
technique UnskinnedPickingNBT
<
	string Description = "Unskinned picking shader with NBT-data";
	string NBTMethod = "ATI";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	            
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_PickingTex();
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_PickingStandard();
    }
}

// ================================================================================================
technique UnskinnedPickingNBT_Special
<
	string Description = "Unskinned picking shader with NBT-data";
	string NBTMethod = "ATI";
>
{
    pass p0
    {	        
        PickRenderStates;
        //AlphaTestEnable = true;
		AlphaTestEnable = <g_EnableAlphaTest>;
	            
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_PickingTex();
        PixelShader  = compile SHAD_PS_MINVER_2_0 PS_PickingAlphaTextureSecondBlend();
    }
}

// ================================================================================================
technique SkinnedPicking
<
	string Description = "Skinning shader for pick operations";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	    
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_PickingSkinned(4);
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_PickingStandard();
    }
}

// ================================================================================================
technique SkinnedPicking_4Tex
<
	string Description = "Skinning shader for pick operations (4 textures)";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	    
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_PickingSkinned(4);
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_PickingStandard();
    }
}

// ================================================================================================
technique SkinnedPickingNBT
<
	string Description = "Skinning shader for pick operations with NBT-data";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "ATI";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	            
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_PickingSkinned(4);
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_PickingStandard();
    }
}

// ================================================================================================
technique SkinnedPickingNBT_Special
<
	string Description = "Skinning shader for pick operations with NBT-data";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "ATI";
>
{
    pass p0
    {	        
        PickRenderStates;
        //AlphaTestEnable = true;
		AlphaTestEnable = <g_EnableAlphaTest>;
	            
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_PickingSkinned(4);
        PixelShader  = compile SHAD_PS_MINVER_2_0 PS_PickingAlphaTextureSecondBlend();
    }
}

// ================================================================================================
technique PickGroundTile
<
	string Description = "Ground tile picking shader";
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	            
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_PickingNoTex();
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_PickingStandard();
    }
}

// ================================================================================================
technique PickGroundTileDeepSea
<
	string Description = "Ground tile DeepSea picking shader";
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	            
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_PickingNoTex();
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_PickingStandard();
    }
}


/* by jow entfernt und durch die obigen shader ersetzt. 
// ================================================================================================
struct VS_PosCol
{
    float4 Pos  : POSITION;
    float4 Col  : COLOR;
};

// ================================================================================================
// Hier wird eine dummy Berechnung durchgefhrt, damit alle Daten benutzt und nicht vom Compiler
// gelscht werden.
// ================================================================================================
VS_PosCol VS_SkinnedPicking(VS_InSkinning i, uniform int NumBones)
{
    float3 Pos = SkinningPosOnly(i.Pos, i.BlendWeights, i.BlendIndices, NumBones);

    VS_PosCol o;
    o.Pos = mul(float4(Pos, 1.0f), matViewProj);    // Pos is in world space
    o.Col = PickColor;
    o.Col.a = i.Normal.x + i.Tex.x;                 // Alpha-Komponente wird eh nicht beachtet // todo: evtl. schon noch beachten

    return o;
}

// ================================================================================================
technique SkinnedPicking
<
	string Description = "Skinning shader for pick operations";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	    
        VertexShader = compile vs_1_1 VS_SkinnedPicking(4);
        PixelShader = 0;
    }
}





// ================================================================================================
// Hier wird eine dummy Berechnung durchgefhrt, damit alle Daten benutzt und nicht vom Compiler
// gelscht werden.
// ================================================================================================
VS_PosCol VS_SkinnedPicking_4Tex(VS_InSkinning_4Tex i, uniform int NumBones)
{
    float3 Pos = SkinningPosOnly(i.Pos, i.BlendWeights, i.BlendIndices, NumBones);

    VS_PosCol o;
    o.Pos = mul(float4(Pos, 1.0f), matViewProj);    // Pos is in world space
    o.Col = PickColor;
    o.Col.a = i.Normal.x + min(0, i.Tex.x + i.Tex1.x + i.Tex2.x + i.Tex3.x + 100);    // Alpha-Komponente wird eh nicht beachtet // todo: evtl. schon noch beachten

    return o;
}

// ================================================================================================
technique SkinnedPicking_4Tex
<
	string Description = "Skinning shader for pick operations (4 textures)";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	    
        VertexShader = compile vs_1_1 VS_SkinnedPicking_4Tex(4);
        PixelShader = 0;
    }
}




// ================================================================================================
// Hier wird eine dummy Berechnung durchgefhrt, damit alle Daten benutzt und nicht vom Compiler
// gelscht werden.
// ================================================================================================
// VS_PosCol VS_SkinnedPickingNBT(VS_InSkinnedNBT i, uniform int NumBones)
VS_OutColTex VS_SkinnedPickingNBT(VS_InSkinnedNBT i, uniform int NumBones)
{
    float3 Pos = SkinningNBTPosOnly(i, NumBones);

    //VS_PosCol o;
    VS_OutColTex o = (VS_OutColTex)0;
    o.Pos = mul(float4(Pos, 1.0f), matViewProj);                    // Pos is in world space
    o.Tex = i.Tex0;
    o.Amb = PickColor;  // from o.Col
    o.Amb.a = i.Normal.x + i.Binormal.x + i.Tangent.x + i.Tex0.x + i.Tex1.x + i.Tex2.x + i.Tex3.x;    // Alpha-Komponente wird eh nicht beachtet
 
    return o;
}

// ================================================================================================
technique SkinnedPickingNBT
<
	string Description = "Skinning shader for pick operations with NBT-data";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "ATI";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	            
        VertexShader = compile vs_1_1 VS_SkinnedPickingNBT(4);
        PixelShader = 0;
    }
}

// ================================================================================================
technique SkinnedPickingNBT_Special
<
	string Description = "Skinning shader for pick operations with NBT-data";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "ATI";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaRef        = 5;
        AlphaTestEnable = true;
	            
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_SkinnedPickingNBT(4);
        PixelShader  = compile SHAD_PS_MINVER_2_0 PS_ColorAndTextureAlpha(true);
    }
}


// ================================================================================================
// Hier wird eine dummy Berechnung durchgefhrt, damit alle Daten benutzt und nicht vom Compiler
// gelscht werden.
// ================================================================================================
VS_OutColTex VS_UnskinnedPicking(float3 Pos : POSITION,
                                 float3 Normal : NORMAL,
                                 float2 Tex0   : TEXCOORD0,
                                 float2 Tex1   : TEXCOORD1,
                                 float2 Tex2   : TEXCOORD2,
                                 float2 Tex3   : TEXCOORD3)
{
    VS_OutColTex o;
    o.Pos = mul(float4(Pos, 1.0f), matWorldViewProjection);
    o.Amb = PickColor;
    o.Amb.a = Normal.x + Tex1.x + Tex2.x + Tex3.x; // Alpha-Komponente wird im PS komplett berschrieben
    o.Diff = float4(0,0,0,0);
    o.Tex = Tex0;
#if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = float4(0,0,0,0);
#endif    
    return o;
}

// ================================================================================================
VS_PosCol VS_UnskinnedPicking_NoTex(float3 Pos : POSITION, float3 Normal : NORMAL)
{
    VS_PosCol o;
    o.Pos = mul(float4(Pos, 1.0f), matWorldViewProjection);
    o.Col = PickColor;
    o.Col.a += min(0, Normal.x+100);
    
    return o;
}


// ================================================================================================
technique UnskinnedPicking
<
	string Description = "Picking shader";
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        //PickRenderStates;
        
        FillMode            = solid;
	    ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = true;
	    ZWriteEnable        = true;
	    AlphaRef            = 5;
	    AlphaTestEnable     = true;
	    AlphaFunc           = greater;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    SpecularEnable      = false;
	    FogEnable           = false;

        VertexShader = compile SHAD_VS_MINVER_1_1 VS_UnskinnedPicking();
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_ColorAndTextureAlpha();
    }
}

// ================================================================================================
technique UnskinnedPicking_Special
<
	string Description = "Picking shader special";
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        //PickRenderStates;
        
        FillMode            = solid;
	    ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = true;
	    ZWriteEnable        = true;
	    AlphaRef            = 5;
	    AlphaTestEnable     = true;
	    AlphaFunc           = greater;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    SpecularEnable      = false;
	    FogEnable           = false;

        VertexShader = compile SHAD_VS_MINVER_1_1 VS_UnskinnedPicking();
        PixelShader = compile SHAD_PS_MINVER_2_0  PS_ColorAndTextureAlpha(true);
    }
}

// ================================================================================================
technique UnskinnedPicking_NoTex
<
	string Description = "Picking shader (no texture)";
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	            
        VertexShader = compile vs_1_1 VS_UnskinnedPicking_NoTex();
        PixelShader = 0;
    }
}


// ================================================================================================
// Hier wird eine dummy Berechnung durchgefhrt, damit alle Daten benutzt und nicht vom Compiler
// gelscht werden.
// ================================================================================================
VS_PosCol VS_UnskinnedPickingNBT(VS_InNBT i)
{
    VS_PosCol o;
    o.Pos = mul(i.Pos, matWorldViewProjection);
    o.Col = PickColor;
    o.Col.a = i.Normal.x + i.Binormal.x + i.Tangent.x + i.Tex0.x + i.Tex1.x + i.Tex2.x + i.Tex3.x;  // Alpha-Komponente wird eh nicht beachtet

    return o;
}

// ================================================================================================
technique UnskinnedPickingNBT
<
	string Description = "Unskinned picking shader with NBT-data";
	string NBTMethod = "ATI";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	            
        VertexShader = compile vs_1_1 VS_UnskinnedPickingNBT();
        PixelShader = 0;
    }
}

// ================================================================================================
VS_OutColTex VS_UnskinnedPickingNBT_Special(VS_InNBT i)
{
    VS_OutColTex o = (VS_OutColTex)0;
    o.Pos = mul(i.Pos, matWorldViewProjection);
    o.Amb = PickColor;
    o.Tex = i.Tex0;
    o.Amb.a = i.Normal.x + i.Binormal.x + i.Tangent.x + i.Tex0.x + i.Tex1.x + i.Tex2.x + i.Tex3.x;  // Alpha-Komponente wird eh nicht beachtet

    return o;
}

// ================================================================================================
technique UnskinnedPickingNBT_Special
<
	string Description = "Unskinned picking shader with NBT-data";
	string NBTMethod = "ATI";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaRef        = 5;
        AlphaTestEnable = true;
	            
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_UnskinnedPickingNBT_Special();
        PixelShader  = compile SHAD_PS_MINVER_2_0 PS_ColorAndTextureAlpha(true);
    }
}




// ================================================================================================
struct VS_InGround
{
   float3 Pos					: POSITION;
   float3 Normal				: NORMAL;
   float4 TexBlendNCWeight		: COLOR0;
   float2 TexSurface			: TEXCOORD0;
};


// ================================================================================================
// Hier wird eine dummy Berechnung durchgefhrt, damit alle Daten benutzt und nicht vom Compiler
// gelscht werden.
// ================================================================================================
VS_PosCol VS_PickGroundTile(VS_InGround i)
{
    VS_PosCol o;
    o.Pos = mul(float4(i.Pos, 1), matWorldViewProjection);
    o.Col = PickColor;
    o.Col.a = i.Normal.x + i.TexBlendNCWeight + i.TexSurface.x; // Alpha-Komponente wird eh nicht beachtet

    return o;
}

// ================================================================================================
technique PickGroundTile
<
	string Description = "Ground tile picking shader";
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	            
        VertexShader = compile vs_1_1 VS_PickGroundTile();
        PixelShader = 0;
    }
}




// ================================================================================================
struct VS_InGroundDeepSea
{
   float3 Pos					: POSITION;
   float3 Normal				: NORMAL;
   float4 TexBlendNCWeight		: COLOR0;
   float2 TexSurface			: TEXCOORD0;
   float2 WaterHeight			: TEXCOORD1;
};

// ================================================================================================
// Hier wird eine dummy Berechnung durchgefhrt, damit alle Daten benutzt und nicht vom Compiler
// gelscht werden.
// ================================================================================================
VS_PosCol VS_PickGroundTileDeepSea(VS_InGroundDeepSea i)
{
    VS_PosCol o;
    o.Pos = mul(float4(i.Pos, 1), matWorldViewProjection);
    o.Col = PickColor;
    o.Col.a = i.Normal.x + i.TexBlendNCWeight + i.TexSurface.x + i.WaterHeight.x; // Alpha-Komponente wird eh nicht beachtet

    return o;
}

// ================================================================================================
technique PickGroundTileDeepSea
<
	string Description = "Ground tile DeepSea picking shader";
	string NBTMethod = "none";
>
{
    pass p0
    {	        
        PickRenderStates;
        AlphaTestEnable = false;
	            
        VertexShader = compile vs_1_1 VS_PickGroundTileDeepSea();
        PixelShader = 0;
    }
}
*/
