// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2007 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: ank
// date: 30.03.2006

#include "GlobalFX.h"
#include "shadowmap.hlsl"

// ================================================================================================
const float4x4 matWVP              : WORLDVIEWPROJECTION;

// const float4x4 matLightViewProj : GLOBAL;
//const float4x4 matBaseView_V	   : GLOBAL;
const float4x4 matBaseView_P	   : GLOBAL;
//const float4x4 matBaseView_PInv	   : GLOBAL;
const float4x4 matBaseView_WVPInv  : GLOBAL; // WICHTIG! -> Matrix der Hauptkamera...
const float4x4 matBaseView_WVP     : GLOBAL;

const float g_fContrast	    : GLOBAL = 1;
const float g_fBrightness	: GLOBAL = 4;
const float g_fLuminance    : GLOBAL = 9;
const float g_fThreshold    : GLOBAL = 4;
const float g_fOffset       : GLOBAL = 0.4;

const float g_fWeightAdjust : GLOBAL = 1.0f;
const int g_iNumBlurPixels : GLOBAL = 13;
static const int g_iNumPixels = 13;
const float g_afPixelOffset[g_iNumPixels] : GLOBAL;
const float g_afPixelWeight[g_iNumPixels] : GLOBAL;

const float g_noiseScale : GLOBAL;

// ================================================================================================
texture BaseMap<string NTM = "base";>;
texture ShaderMap0<int NTMIndex = 0;>;
texture ShaderMap1<int NTMIndex = 1;>;  // u.a. fr VolumeLight

// ================================================================================================
sampler2D Sampler0 = sampler_state
{
	Texture = (BaseMap);
	AddressU = Clamp;
	AddressV = Clamp;
    MinFilter = Linear;
	MagFilter = Linear;
  	MipFilter = Point;
  	MaxAnisotropy = 1;
};

sampler2D Sampler1 = sampler_state // Special-Textur (nur Pointsampling)
{
	Texture = (ShaderMap0);
    AddressU = BORDER;
    AddressV = BORDER;
//    MinFilter = 1;
//    MagFilter = 1;
    MinFilter = Linear;
    MagFilter = Linear;
    MipFilter = 1;
    MaxAnisotropy = 1;
    BorderColor = float4(0.0f, 0.0f, 0.0f, 0.0f);
};

sampler2D Sampler2 = sampler_state
{
	Texture = (ShaderMap1);
	AddressU = Clamp;
	AddressV = Clamp;
    MinFilter = Point;  
	MagFilter = Point;
  	MipFilter = Point;
  	MaxAnisotropy = 1;
};

sampler2D SamplerSSAODepth = sampler_state
{
	Texture = (BaseMap);
	AddressU = Clamp;
	AddressV = Clamp;
    MinFilter = Linear;  
	MagFilter = Linear;
  	MipFilter = Point;
  	MaxAnisotropy = 1;
};

sampler2D SamplerSSAONoise = sampler_state
{
	Texture = (ShaderMap0);
	AddressU = Wrap;
	AddressV = Wrap;
    MinFilter = Point;  
	MagFilter = Point;
  	MipFilter = Point;
  	MaxAnisotropy = 1;
};

// ================================================================================================
// Vertexshader
// ================================================================================================
struct VS_ScreenQuadOut
{
    float4 vPos : POSITION;
    float2 vTex : TEXCOORD0;
};

// ================================================================================================
VS_ScreenQuadOut VS_ScreenQuad(float4 inPos: POSITION, float2 inTex: TEXCOORD)
{
    VS_ScreenQuadOut o = (VS_ScreenQuadOut)0;
    
    o.vPos = mul(inPos, matWVP);
    o.vTex = inTex;
    return o;
}

// ================================================================================================
// Pixelshader
// ================================================================================================
float4 PS_BlurX(float2 vTex : TEXCOORD0) : COLOR
{		
	float4 col = 0;

	for (int i=0; i<g_iNumBlurPixels; i++)
	{
		col += g_afPixelWeight[i] * g_fWeightAdjust * tex2D(Sampler0, vTex + float2(0, g_afPixelOffset[i]));
	}
 
	return abs(col);
}

// ================================================================================================
float4 PS_BlurY(float2 vTex : TEXCOORD0) : COLOR
{		
	float4 col = 0;
	
	for (int i=0; i<g_iNumBlurPixels; i++)
	{
		col += g_afPixelWeight[i] * g_fWeightAdjust* tex2D(Sampler0, vTex + float2(g_afPixelOffset[i], 0));
	}
 
	return abs(col);
}

// ================================================================================================
float4 PS_Bloom(float2 vTex : TEXCOORD0) : COLOR
{
	// Textur auslesen
	float4 col = tex2D(Sampler0, vTex);

	col *= g_fBrightness;    // Helligkeit hochschrauben
	
	// col = (1-(1-x)^contrast)^contrast
	return pow(float4(1, 1, 1, 1) - pow(float4(1, 1, 1, 1) - col, g_fContrast), 1/g_fContrast);
}

// ================================================================================================
float4 PS_Simple(float2 vTex : TEXCOORD0) : COLOR
{
    return tex2D(Sampler0, vTex);
}

static const float3 LUMINANCE_VECTOR = float3(0.2125f, 0.7154f, 0.0721f);

// ================================================================================================
// Desc: Perform a high-pass filter on the source texture
float4 PS_BrightPassFilter(float2 vTex : TEXCOORD0) : COLOR
{
    float3 col = dot(tex2D(Sampler0, vTex), LUMINANCE_VECTOR);
    col -= g_fOffset;
    col *= g_fLuminance;
    col -= g_fThreshold;
    col = max(col, 0);
    //col /= (1 + col);   // BrightPassOffset
    return float4(col, 1);
}

// ================================================================================================
float4 PS_Monochrome(float2 vTex : TEXCOORD0) : COLOR
{
    return dot((float3)tex2D(Sampler0, vTex), LUMINANCE_VECTOR);
}

static const float2 g_afPixelOffset4xDownFilter[16] =
{
	{ -0.002, -0.002 },
	{ -0.002, -0.001 },
	{ -0.002,  0.001 },
	{ -0.002,  0.002 },
	{ -0.001, -0.002 },
	{ -0.001, -0.001 },
	{ -0.001,  0.001 },
	{ -0.001,  0.002 },
	{  0.001, -0.002 },
	{  0.001, -0.001 },
	{  0.001,  0.001 },
	{  0.001,  0.002 },
	{  0.002, -0.002 },
	{  0.002, -0.001 },
	{  0.002,  0.001 },
	{  0.002,  0.002 }
};

// ================================================================================================
float4 PS_DownFilter4x(float2 vTex : TEXCOORD0) : COLOR
{		
	float4 col = 0;

    for (int i=0; i<16; i++)
    {
        col += tex2D(Sampler0, vTex + g_afPixelOffset4xDownFilter[i]);
    }

    return col / 16;
}

// ================================================================================================
// technique
// ================================================================================================
technique DownFilter4x
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif		
        PixelShader		= compile SHAD_PS_MINVER_2_0 PS_DownFilter4x();
	}
}

// ================================================================================================
technique Simple
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif		
        PixelShader		= compile SHAD_PS_MINVER_2_0 PS_Simple();
	}
}

// ================================================================================================
technique BrightPassFilter
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif		
        PixelShader		= compile SHAD_PS_MINVER_2_0 PS_BrightPassFilter();
	}
}

// ================================================================================================
technique Monochrome
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif		
        PixelShader		= compile SHAD_PS_MINVER_2_0 PS_Monochrome();
	}
}

// ================================================================================================
technique BlurX
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif
		PixelShader		= compile SHAD_PS_MINVER_2_0 PS_BlurX();
	}
}

// ================================================================================================
technique BlurXAdd
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = true;
		BlendOp				= Add;
        SrcBlend            = One;
	    DestBlend           = One;
        Lighting            = false;
	    
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif
		PixelShader		= compile SHAD_PS_MINVER_2_0 PS_BlurX();
	}
}

// ================================================================================================
technique BlurY
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif
		PixelShader		= compile SHAD_PS_MINVER_2_0 PS_BlurY();
	}
}


// ================================================================================================
technique BlurYAdd
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = true;
		BlendOp				= Add;
        SrcBlend            = One;
	    DestBlend           = One;
	    Lighting            = false;
	    
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif
		PixelShader		= compile SHAD_PS_MINVER_2_0 PS_BlurY();
	}
}

// ================================================================================================
technique Bloom
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = true;
		BlendOp				= Add;
	    SrcBlend            = One;
	    DestBlend           = One;
	    Lighting            = false;
	    
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif
		PixelShader		= compile SHAD_PS_MINVER_2_0 PS_Bloom();
	}
}

// ================================================================================================
// Volume-Light Berechnungen hier:
// ================================================================================================
const float  g_fNear               : GLOBAL = 1;
const float3 g_SceneCameraPosition : GLOBAL;
const float3 g_SceneCameraDirection: GLOBAL;
const float3 g_LightForward        : GLOBAL;    // Licht-Vec (TODO)

#define MAX_STEPS  300  // orginal waren: 2000 <wtf>; atm 200 max, compiling dauert sonst JAHRE!
#define STEP_SCALE (1.0/(float)MAX_STEPS)

const float  g_MaxTraceDistance : GLOBAL = 2000; // 10000;
const float  g_InvSamplingRate  : GLOBAL = 1.5; //5; 

// ================================================================================================
// Pixelshaders
// ================================================================================================
float4 CalculateClipPosition(float2 screenCoords, float sceneDepth)
{
    float4 clipPos;
    clipPos.x =  2.0 * screenCoords.x - 1.0;
    clipPos.y = -2.0 * screenCoords.y + 1.0;
    clipPos.z = sceneDepth;
    clipPos.w = 1;
    
    return clipPos;
}

// ================================================================================================
void RecreateWorldPosition(out float4 p_PositionWS, in float2 p_ScreenCoords, in float p_ScreenDepth)
{
    float4 clipPosition = CalculateClipPosition(p_ScreenCoords, p_ScreenDepth);

    p_PositionWS = mul(clipPosition, matBaseView_WVPInv);
	p_PositionWS *= 1.0/p_PositionWS.w;
}

// ================================================================================================
// Calculate the view space position from screen coordinates and view space depth
// ================================================================================================
void RecreatePositionInViewSpace(out float3 positionVS, in float2 screenCoords, in float depthVS)
{
	float4 clipPosition = CalculateClipPosition(screenCoords, depthVS);

	positionVS.x = depthVS * clipPosition.x / matBaseView_P._m00;
	positionVS.y = depthVS * clipPosition.y / matBaseView_P._m11;
	positionVS.z = depthVS;
}

// ================================================================================================
/*
float __GetShadowFactorFromPosition(float2 vTex)
{
    float4 positionWS;
    float depthScene   = 1 - tex2D(Sampler0, vTex).x;
    RecreateWorldPosition(positionWS, vTex, depthScene);
    
    // convert into texture coords
    float4 shadowUV = mul( float4(positionWS.xyz, 1.0), matLightViewProj);
	shadowUV /= shadowUV.w;
	shadowUV.x = ( shadowUV.x + 1.0 ) * 0.5;  
	shadowUV.y = ( 1.0 - shadowUV.y ) * 0.5;
	   
    float sceneShadowDepth = 1 - tex2D(Sampler1, shadowUV.xy).x;
    return (sceneShadowDepth + 0.0006 >= shadowUV.z);                  // 0 == schatten; 1 == keiner
}
*/

// ================================================================================================
float GetShadowFactorFromPosition(float2 vTex)
{
    vTex = clamp(vTex,0.001,0.999);
    return 1 - tex2D(Sampler1, vTex.xy).x;
}

// ================================================================================================
#define NUM_FILTERSIZE 5
static const float2 g_filterTexCoord[NUM_FILTERSIZE] = 
{
    float2( 0,-1.6),
    float2( 0,-0.8),
    float2( 0, 0),
    float2( 0, 0.8),
    float2( 0, 1.6),
};
static const float g_filterStrength[NUM_FILTERSIZE] = 
{
     1,
     2,
     0,
    -2,
    -1
};
static const float g_fPixelOffset = 1.0/512.0;

// ================================================================================================
static const float2 g_filterDTexCoord[6] = 
{
    float2( 0,-3),
    float2( 0,-2),
    float2( 0,-1),
    float2( 0, 1),
    float2( 0, 2),
    float2( 0, 3),
};

static const float g_filterDStrength[6] = 
{
     0.5,
     1.2,
     2.7,
     2.7,
     1.2,
     0.5
};

// ================================================================================================
// bestimmt wo innerhalb der Welt starke Kanten sind (zB. Lcken),
// in denen es zur Volume-Bildung kommen kann
// ================================================================================================
float4 PS_ComputeDepthDiscontinuities(float2 vTex : TEXCOORD0) : COLOR
{
    float4 col  = float4(0,0,0,1);

#define METHODE_1
#if defined(METHODE_1)
 
    float baseDepth = GetShadowFactorFromPosition(vTex);
    float2 biasFactor = float2(0,0);
    
    for(int i = 0; i < 6; ++i)
    {
        float2 offset = g_filterDTexCoord[i] / 512.0;
        float depth = GetShadowFactorFromPosition(vTex + offset);
        float f = saturate((depth - baseDepth)/ 0.4);
        biasFactor.x += g_filterDStrength[i] * f;
    }

    for(int i = 0; i < 6; ++i)
    {
        float2 offset = g_filterDTexCoord[i] / 512.0;
        float depth = GetShadowFactorFromPosition(vTex + offset.yx);
        float f = saturate((depth - baseDepth)/ 0.4);
        biasFactor.y += g_filterDStrength[i] * f;
    }
    col.x = max(biasFactor.x, biasFactor.y) * 0.5;
    return col;
#else    
    float4 col0 = float4(0,0,0,1);
    float4 col1 = float4(0,0,0,1);
    
    for(int i = 0; i < NUM_FILTERSIZE; ++i)
    {
        float2 offset = g_filterTexCoord[i] * g_fPixelOffset;
        float factor  = GetShadowFactorFromPosition(vTex + offset);
        col0.x += factor * g_filterStrength[i];
    }

    for(int i = 0; i < NUM_FILTERSIZE; ++i)
    {
        float2 offset = g_filterTexCoord[i].yx * g_fPixelOffset;
        float factor  = GetShadowFactorFromPosition(vTex + offset);
        col0.x += factor * g_filterStrength[i];
    }
    
    for(int i = 0; i < NUM_FILTERSIZE; ++i)
    {
        float2 offset = -g_filterTexCoord[i] * g_fPixelOffset;
        float factor  = GetShadowFactorFromPosition(vTex + offset);
        col1.x += factor * g_filterStrength[i];
    }

    for(int i = 0; i < NUM_FILTERSIZE; ++i)
    {
        float2 offset = -g_filterTexCoord[i].yx * g_fPixelOffset;
        float factor  = GetShadowFactorFromPosition(vTex + offset);
        col1.x += factor * g_filterStrength[i];
    }
    col = abs(col0) + abs(col1);
    return col;
    //return col * 0.5;
#endif    
}

// ================================================================================================
// Testfunktion
// ================================================================================================
float4 PS_VolumeLight__(float2 vTex : TEXCOORD0) : COLOR
{
    //-------------
	// Sampler0 = DepthMap (Tiefentextur aus aktuellen View)
    // Sampler1 = ShadowMap(Tiefentextur vom Licht)
    //-------------
	float sceneDepth = 1 - tex2D(Sampler0, vTex).x;
    float4 col = float4(0,0,0,1);
	
	//-------------
	// reconstruct ClipSpace Position
	float4 clipPos = CalculateClipPosition(vTex, sceneDepth);
	
	//-------------
	// generate WorldSpace - Position
	float4 posWS = mul(clipPos, matBaseView_WVPInv);
	posWS *= 1.0/posWS.w;
     
	//-------------
	// float2 help = pow(posWS.xy*0.00025, 2);
    // return float4(0,help,1);
    // return float4(posWS.xyz/4000, 1);
	//-------------
	
	float4 shadowUV;
	float3 coordinates;
	
	shadowUV = mul( float4(posWS.xyz, 1.0), matLightViewProj);
	coordinates = shadowUV.xyz / shadowUV.w;
	coordinates.x = ( coordinates.x + 1.0 ) * 0.5;  // convert into texture coords
	coordinates.y = ( 1.0 - coordinates.y ) * 0.5;
	
	float sampleFine = 0;
	float depth = 1 - tex2D(Sampler1, coordinates.xy).x + 0.005;
    
    sampleFine = (depth + 0.09 - coordinates.z) / 0.1;
    sampleFine = 1 - saturate(sampleFine);
    // sampleFine = (depth >= coordinates.z);  // 0 == schatten; 1 == keiner
    // sampleFine = (depth < coordinates.z);  // 1 == schatten; 0 == keiner
    // sampleFine = 1 - saturate(depth - coordinates.z);
    
    return sampleFine;
}

// ================================================================================================
float4 PS_VolumeLight(float2 vTex : TEXCOORD0) : COLOR
{
    //-------------
	// Sampler0 = DepthMap (Tiefentextur aus aktuellen View)
    // Sampler1 = ShadowMap(Tiefentextur vom Licht)
    //-------------
#ifdef VARIANCE_SHADOW_MAPPING  // variance shadow mapping here
	float sceneDepth = tex2D(Sampler0, vTex).x;
#else
	float sceneDepth = 1 - tex2D(Sampler0, vTex).x;
#endif
	//-------------
    // return float4(pow(sceneDepth,100), 0,0,1);
	//-------------
    
    float4 col = float4(0,0,0,1);
	
	//-------------
	// reconstruct ClipSpace Position
	//-------------
	float4 clipPos = CalculateClipPosition(vTex, sceneDepth);
	//-------------
	//return float4(1 - clipPos.z, clipPos.xy,1);
	//-------------
	
	//-------------
	// generate WorldSpace - Position
	//-------------
	float4 posWS = mul(clipPos, matBaseView_WVPInv);
	posWS *= 1.0/posWS.w;
	// posWS /= posWS.w;
     
	//-------------
	// float2 help = pow(posWS.xy*0.00025, 2);
    // return float4(0,help,1);
    // return float4(posWS.xyz/4000, 1);
	//-------------
	
	// vector from cur. Pos to Camera
	// and divide it by #iterations to sample ...
	float3 vecForward = posWS - g_SceneCameraPosition.xyz;
	
#if defined(SHADERMODEL3) && !defined(DISABLE_VOLUMELIGHT)
	vecForward = normalize(vecForward);
	
	// calculate max. tracing distance
	float traceDistance = dot( posWS.xyz - ( g_SceneCameraPosition.xyz + vecForward * g_fNear ), vecForward );
	traceDistance = clamp( traceDistance, 0.0, g_MaxTraceDistance ); // Far trace distance
	
	// calculate startposition for sampling (@nearplane)
	// posWS.xyz = g_SceneCameraPosition.xyz + vecForward * g_fNear;
	float3 startPos = g_SceneCameraPosition.xyz + vecForward * g_fNear;
    
    {
        // ANGLE OPTIMIZATION    
        // compute how much we face the light/have same direction
        float dotViewLight = dot( vecForward, g_LightForward );
        // scale up the forward-vector, if we are have a narrow angle
        // (90 => 1, 0 => e^1 = 2.71... )
    	vecForward *= exp( dotViewLight * dotViewLight );
    }
    
    // now ajust the length by our inverse samplingrate
	vecForward *= g_InvSamplingRate * 2.0;
	int stepsNum = clamp( traceDistance/length(vecForward), 0, MAX_STEPS);
	
	//-------------
	// debug:
	// return float4( (float)stepsNum/(float)MAX_STEPS, 0,0,1);
	//-------------
	
	float jitter = 0.9; // maybe noise hinzufgen... (kleiner offset auf den Startwert)
	float step  = length(vecForward);
	float scale = step * STEP_SCALE;     // Set base brightness factor (over all steps)
	
	float4 shadowUV;
	float3 coordinates;
	
	//-------------
	// Calculate coordinate delta ( coordinate step in ligh space )
	// first calculate 2 positions and transform them into lightspace,
	// after that, generate diverence vector => our stepvector in lightspace!
	//
	// REVERSE:
	// float3 curPosition = posWS.xyz - vecForward * jitter;  // debug (endposition@object)
	//
	// first position:
	//-------------
	float3 curPosition = startPos.xyz + vecForward * jitter;
	shadowUV = mul( float4(curPosition, 1.0), matLightViewProj);
	coordinates = shadowUV.xyz / shadowUV.w;
	coordinates.x = ( coordinates.x + 1.0 ) * 0.5;  // convert into texture coords
	coordinates.y = ( 1.0 - coordinates.y ) * 0.5;
	
	//-------------
	// debug:
	// return float4(coordinates,1);
    //
	// float depth = 1-tex2D(Sampler1, coordinates.xy).x;
	// return float4(depth, coordinates.z,0,1);
	// if (depth >= coordinates.z)
	//    return float4(0.9,0.9,0.9,1);
    //else
    //    return float4(0.5,0.5,0.5,1);
	// 
	// REVERSE:
	// curPosition = curPosition  - vecForward; // debug
	//-------------
	
	// second position:
	curPosition = startPos.xyz + vecForward * ( 1.0 + jitter );
	shadowUV = mul( float4(curPosition, 1.0), matLightViewProj );
	float3 coordinateEnd = shadowUV.xyz / shadowUV.w;
	coordinateEnd.x = ( coordinateEnd.x + 1.0 ) * 0.5;
	coordinateEnd.y = ( 1.0 - coordinateEnd.y ) * 0.5;
	
	float3 coordinateDelta = coordinateEnd - coordinates;
	
	// float2 vecForwardProjection;
	// vecForwardProjection.x = dot( g_LightRight, vecForward );
	// vecForwardProjection.y = dot( g_LightUp, vecForward );
    
    float light = 0.0;      // result
	float sampleFine = 0;
	
	// loop through all samplingpositions...
	// for the compile (u suck) - to check, that the loop will finish:
	stepsNum = stepsNum % (MAX_STEPS+1);
    
    for( int i = 0; i < stepsNum; i++ )
    {
        float depth = 1 - tex2D(Sampler1, coordinates.xy).x;
        
        // alte Variante:
        sampleFine = (depth + 0.09 - coordinates.z) /0.1;
        sampleFine = 1 - saturate(sampleFine);
        
        float density = 0.01;
        density += tex2D(Sampler2, coordinates.xy).x;
        
        sampleFine *= density;

        light += scale * sampleFine;
        
        // calculate next position
        coordinates += coordinateDelta;
    }
    
    // -----------
	// float depth;
	// depth = 1-tex2D(Sampler1, coordinates.xy).x;
	// return float4(depth,0,0,1);
	// -----------
	
	return float4(light, light, light, 1);
	
#else
	vecForward *= 0.05;
	
	/*/-------------
	float3 vVec = float3(0,0,0);
	for(int i=0; i<20; ++i)
	{
	    float4 localLightVPS = PerformTransformInLightSpace(posWS + float4(vVec,0));
	    float fCurrentDepth = GetDepthInLightSpace(localLightVPS);
        
        col.xyz += GetObjectInShadowFactor(fCurrentDepth, localLightVPS, Sampler1);
        vVec += vecForward;
	}
	col.xyz *= 0.05;
	*/
	col.xyz = float3(0,0,0);
#endif
	
	return col;
}

// ================================================================================================
float4 PS_AddVolumeLight2Scene(float2 vTex : TEXCOORD0) : COLOR
{
	// debug-test
	//return tex2D(Sampler0, vTex);
	
	// Textur auslesen (2x2 geblurred)
	float4 col = float4(0,0,0,0);
	
	
	//float  f = 1.0/512.0;
	float  f = 1.0/700.0;
    col += tex2D(Sampler0, vTex + float2(f,0));
	col += tex2D(Sampler0, vTex + float2(0,f));
	col += tex2D(Sampler0, vTex + float2(-f,0));
	col += tex2D(Sampler0, vTex + float2(0,-f));
    // col *= 0.25 * 0.25; // .> new version
	col *= (4 - col) * 0.25;
    
	// Version as ADDITIV light (maybe add light/particle-color here)
	col.xyz = saturate(col - 0.2)*0.2;
	col.w   = 1;
	
	// Version as FOG:
	// col = float4(1,1,1, pow(col.x,1));
	return col;
}

// ================================================================================================
// Depth of Field global Parameters:
// ================================================================================================
const float g_fDOFfocusAreaHalf : GLOBAL = 500;
const float g_fDOFnearFading    : GLOBAL = 1000;
const float g_fDOFfarFading     : GLOBAL = 2000;
const float g_fDOFmaxBlur       : GLOBAL = 1;
const float g_fDOFfocusDistance : GLOBAL = 1000;
const bool  g_bAutoFocus        : GLOBAL = true;

// ================================================================================================
float4 PS_DofCombine(float2 Tex : TEXCOORD0) : COLOR
{
    float3  colorBlurred = tex2D(Sampler0, Tex);

#ifdef VARIANCE_SHADOW_MAPPING  // variance shadow mapping here
    float depthScene = tex2D(Sampler1, Tex).x;
#else
    float depthScene = 1 - tex2D(Sampler1, Tex).x;
#endif
    float4 col = float4(0,0,0,0);
    
    float4 positionWS;
    float fFocusDistance = g_fDOFfocusDistance;
    
    // if autofocus, get focal distance from image center
    if (g_bAutoFocus)
    {
        float focalDepthScene = 1 - tex2D(Sampler1, float2(0.5, 0.5)).x;
        RecreateWorldPosition(positionWS, float2(0.5, 0.5), focalDepthScene);
        
        fFocusDistance = length(positionWS.xyz - g_SceneCameraPosition.xyz);
    }
    
    // calculate distance from image depth
    RecreateWorldPosition(positionWS, Tex, depthScene);
    
    float distance = length(positionWS.xyz - g_SceneCameraPosition.xyz);
    distance = distance - fFocusDistance;
    
    float fadeDistance = g_fDOFfarFading;
	// by jow entfernt: im God-View wird kein Near bereich bentigt
    /*if (distance < 0) // near-bereich
    {
        fadeDistance = g_fDOFnearFading;
        distance = abs(distance);
    }*/
    
    distance = distance - g_fDOFfocusAreaHalf;
    distance = distance/fadeDistance * g_fDOFmaxBlur;
    distance = min(saturate(distance), g_fDOFmaxBlur);

    // debug
    //col = float4(distance, distance, distance, 1);
    
    // linear:
    col = float4(colorBlurred, distance);
    
    // exp:
    // col = float4(colorBlurred, distance*distance);
    
    // debug:
    // if (Tex.x > 0.5 )
    //    col = float4(pow(depthScene,2),0,0, 1);

    return col;
}

// ================================================================================================
// technique
// ================================================================================================
technique ComputeDepthDiscontinuities
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    Lighting            = false;

#if defined(SHADERMODEL3)
	    VertexShader	= compile vs_3_0 VS_ScreenQuad();
		PixelShader		= compile ps_3_0 PS_ComputeDepthDiscontinuities();
#else
	    VertexShader	= null;
		PixelShader		= compile ps_2_a PS_ComputeDepthDiscontinuities();
#endif
	}
}

const float4    LightDir : GLOBAL = {0.0f, 0.0f, -1.0f, 1.0f};
const float4    LightPos : GLOBAL = {0.0f, 0.0f, 1000.0f, 1.0f};

// ================================================================================================
// ================================================================================================
float4 PS_V1_nVolumeLight(float2 vTex : TEXCOORD0) : COLOR
{
    //-------------
	// Sampler0 = DepthMap (Tiefentextur aus aktuellen View)
    // Sampler1 = ShadowMap(Tiefentextur vom Licht)
    //-------------
    float sceneDepth = 1 - tex2D(Sampler0, vTex).x;
	//-------------
    // return float4(pow(sceneDepth,100), 0,0,1);
	//-------------
    
    float4 col = float4(0,0,0,1);
	
	//-------------
	// reconstruct ClipSpace Position
	//-------------
	float4 clipPos = CalculateClipPosition(vTex, sceneDepth);
	//-------------
	//return float4(1 - clipPos.z, clipPos.xy,1);
	//-------------
	
	//-------------
	// generate WorldSpace - Position
	//-------------
	float4 posWS = mul(clipPos, matBaseView_WVPInv);
	posWS *= 1.0/posWS.w;

    float fLightFactor = (sceneDepth < 1.0)?0.0:1.0;
    col.xyz = fLightFactor * float3(1,1,1);
    
    // float3 ScreenSpaceLightDir = mul(matBaseView_WVP, float4(LightDir.xyz, 0)).xyz;
    // ScreenSpaceLightDir = normalize(ScreenSpaceLightDir);
    
    float4 vSunPosition;
    vSunPosition.xyz = g_SceneCameraPosition - 100000 * g_LightForward;
    vSunPosition.w = 1.0;
    
    vSunPosition = mul(vSunPosition, matBaseView_WVP);
    vSunPosition.xy /= vSunPosition.w;
    vSunPosition = vSunPosition * 0.5 + 0.5;
    vSunPosition.y = 1 - vSunPosition.y;
    
    float2 ScreenSpaceLightDir = vSunPosition.xy - vTex.xy;
    float fMaxDistance = length(ScreenSpaceLightDir);
    ScreenSpaceLightDir = ScreenSpaceLightDir/fMaxDistance; // normalize
    
    col = 0.0;

    #define MAX_IT 100
    const int nMax = MAX_IT%155;
    
    float fFactorIncrease = fMaxDistance/nMax;
    float fFactor = 0;
    
    // if (vTex.x > 0.125 && vTex.x < 0.975)
    {
        float  fCombineFactor = (sceneDepth >= 1.0)?1.0:(1-sceneDepth);
        float3 ColLight = float3(1,1,0.9); //tex2D(Sampler2, vSunPosition).xyz;
        
        int n = 0;
	    while (fFactor < fMaxDistance && n < nMax)
	    {
	        
	        float2 vCurTex = vTex + fFactor*ScreenSpaceLightDir;
	        float fDepth = 1 - tex2D(Sampler0, vCurTex).x;
	        // float f = ((fDepth<1.0)?0:fDepth);
	        float f = ((fDepth<1.0)?0:1);
	        col.xyz += f; // pow(1.1, f);
	        
	        fFactor += fFactorIncrease;
	        ++n;
	    }
		
		col.xyz *= ColLight;
		col.xyz /= (float)n;
	    col.xyz *= fCombineFactor;
	}
	//else
	//{
	//	float f = 1 - tex2D(Sampler0, vTex).x;
	//	col.xyz += ((f<1.0)?0:1);
	//}
	
    return col;
}

// ================================================================================================
technique VolumeLight_Calculate
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    Lighting            = false;

#if defined(SHADERMODEL3)
	    VertexShader	= compile vs_3_0 VS_ScreenQuad();
		PixelShader		= compile ps_3_0 PS_VolumeLight();
#else
	    VertexShader	= null;
		PixelShader		= compile ps_2_a PS_VolumeLight();
#endif
	}
}

// ================================================================================================
technique VolumeLight_Scene
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    Lighting            = false;
	    AlphaTestEnable     = false;
	    // AlphaBlendEnable    = false;
	    AlphaBlendEnable    = true;
		BlendOp				= Add;
	    
	    //ADDITIV: 
	    SrcBlend          = One;  // do nothing!
	    DestBlend         = One;  //Zero; // One;
	    //BLENDING
	    //SrcBlend            = SrcAlpha;
	    //DestBlend           = InvSrcAlpha;
	    
#if defined(SHADERMODEL3) // && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile vs_3_0 VS_ScreenQuad();
		PixelShader		= compile SHAD_PS_MINVER_3_0 PS_AddVolumeLight2Scene();
#else
	    VertexShader	= null;
		PixelShader		= compile SHAD_PS_MINVER_2_0 PS_AddVolumeLight2Scene();
#endif
	}
}

// ================================================================================================
technique DepthOfField
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    Lighting            = false;
	    AlphaTestEnable     = false;
	    //AlphaBlendEnable    = false; // -> direkt visible...
	    AlphaBlendEnable    = true;
		BlendOp				= Add;
	    
	    //BLENDING
	    SrcBlend            = SrcAlpha;
	    DestBlend           = InvSrcAlpha;
	    //SrcBlend          = One;
	    //DestBlend         = Zero;
	    
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif
		PixelShader		= compile SHAD_PS_MINVER_2_0 PS_DofCombine();
	}
}

// ================================================================================================
// ================================================================================================
float4 PS_GodRay_Mask(float2 vTex : TEXCOORD0) : COLOR
{
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
    float sceneDepth = tex2D(Sampler0, vTex).x;
#else
    float sceneDepth = 1 - tex2D(Sampler0, vTex).x;
#endif // ...

    float4 col = float4(0,0,0,0.5);
	col.xyz = tex2D(Sampler1, vTex) * ((sceneDepth>= 1.0)?1:0);
	// if (vTex.x < 0.25)
	//    col.xyz = 1 - sceneDepth; //((sceneDepth >= 1.0)?1:0);
    //else
    //    col.xyz = tex2D(Sampler1, vTex);
    
	//col.xyz = pow(0.7, vTex.x);
	//col.xyz = tex2D(Sampler1, vTex).w;
	return col;
}

#define NUM_SAMPLES 180
const float fDensity  : GLOBAL =  0.95;  //1.0
const float fWeight   : GLOBAL =  0.85;  //0.99
const float fDecay    : GLOBAL =  0.95;  //0.98
const float fExposure : GLOBAL =  0.05;  //1.0

// ================================================================================================
float4 PS_GodRay_Streaks(float2 vTex : TEXCOORD0) : COLOR
{
    float2 texcoord = vTex;

    // mit view direction dot product...
    float3 view  = normalize(g_SceneCameraDirection);
    float3 light = normalize(-g_LightForward);
    float  dotLight = clamp(dot(view, light),0,1);
    
    float4 vSunPosition;
    vSunPosition.xyz = g_SceneCameraPosition - 100000 * g_LightForward;
    vSunPosition.w = 1.0;
    
    vSunPosition = mul(vSunPosition, matBaseView_WVP);
    vSunPosition.xy /= vSunPosition.w;

    vSunPosition = clamp(vSunPosition, -1.0, 1.0);
    vSunPosition = vSunPosition * 0.5 + 0.5;
    vSunPosition.y = 1 - vSunPosition.y;
    
    // return float4(vSunPosition.x, vSunPosition.y, 0, 1);
    
    float2 delta = (vSunPosition.xy-texcoord);
    delta *= fDensity / NUM_SAMPLES;
       
//    delta = min(delta, float2( 0.005, 0.005));
//    delta = max(delta, float2(-0.005,-0.005));
    
    float  illumDecay = 1.0;
    float3 color = tex2D(Sampler0, texcoord);
//    return float4(color,1);
    
    for (int i=0; i<NUM_SAMPLES; ++i)
    {
        texcoord += delta;
        if (texcoord.x >= 0 && texcoord.x <= 1.0 &&
            texcoord.y >= 0 && texcoord.y <= 1.0)
        {
            float3 sample = tex2D(Sampler0, texcoord);
            sample *= illumDecay * fWeight;
            color.xyz += sample;
            illumDecay *= fDecay;
        }
        else
            illumDecay = 0;
    }
    
    float4 col;
    col.xyz = tex2D(Sampler0, texcoord);
    col.w   = 1.0;
  
// fr debugging:    
//    if (vTex.x < 0.5)
//        return float4(color, 1.0); // 0; //col;
//    else
        return float4(dotLight * color * fExposure, 0.20);
}

// ================================================================================================
technique GodRays_Mask
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    Lighting            = false;

#if defined(SHADERMODEL3)
	    VertexShader	= compile vs_3_0 VS_ScreenQuad();
		PixelShader		= compile ps_3_0 PS_GodRay_Mask();
#else
	    VertexShader	= null;
		PixelShader		= compile ps_2_a PS_GodRay_Mask();
#endif
	}
}

// ================================================================================================
technique GodRays_Streaks
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    Lighting            = false;
	    AlphaTestEnable     = false;
	    // AlphaBlendEnable    = false;
	    AlphaBlendEnable    = true;
		BlendOp				= Add;
	    
	    //ADDITIV: 
	    SrcBlend          = One; // do nothing!
	    DestBlend         = One; //Zero; // One;
	    //BLENDING
	    //SrcBlend            = SrcAlpha;
	    //DestBlend           = InvSrcAlpha;
	    
#if defined(SHADERMODEL3) // && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile vs_3_0 VS_ScreenQuad();
		PixelShader		= compile SHAD_PS_MINVER_3_0 PS_GodRay_Streaks();
#else
	    VertexShader	= null;
		PixelShader		= null;
#endif
	}
}

// ================================================================================================
// ================================================================================================
sampler2D ShadowSampler0 = sampler_state
{
	Texture = (BaseMap);
	AddressU = Clamp;
	AddressV = Clamp;
    MinFilter = Point;
	MagFilter = Point;
  	MipFilter = Point;
  	MaxAnisotropy = 1;
};

// ================================================================================================
// Convert depth value to VSM values
// ================================================================================================
float4 ConvertShadowDepthToVSM(float depth)
{
	float2 exponents   = g_EVSMExponents;
	return  float4(depth, depth * depth, 0, 1);
}

// ================================================================================================
// Convert depth value to EVSM values
// ================================================================================================
float4 ConvertShadowDepthToEVSM(float depth)
{
	float2 exponents   = g_EVSMExponents;
	float2 warpedDepth = WarpDepth(depth, exponents);
	return float4(warpedDepth.xy, warpedDepth.xy * warpedDepth.xy) * 0.5 + 0.5;
}

// ================================================================================================
// added "min-filter", so that we don't need to render terrain but still the shadows do not shrink
// into none-existance
// ================================================================================================
float4 PS_ShadowMapFilterH(float2 vTex : TEXCOORD0) : COLOR
{		
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
    float4 col = 0;
	
#   if defined(USE_EVSM)
    float4 source = ConvertShadowDepthToEVSM(tex2D(ShadowSampler0, vTex));
#   else   // !USE_EVSM
    float4 source = ConvertShadowDepthToVSM(tex2D(ShadowSampler0, vTex));
#   endif  // USE_EVSM

	for (int i=0; i<g_iNumPixels; i++)
	{
        //col += g_afPixelWeight[i] * tex2D(ShadowSampler0, vTex + float2(0, g_afPixelOffset[i]));
#   if defined(USE_EVSM)
        col += g_afPixelWeight[i] * ConvertShadowDepthToEVSM(tex2D(ShadowSampler0, vTex + float2(0, g_afPixelOffset[i])));
#   else   // !USE_EVSM
        col += g_afPixelWeight[i] * ConvertShadowDepthToVSM(tex2D(ShadowSampler0, vTex + float2(0, g_afPixelOffset[i])));
#   endif  // USE_EVSM
	}

    if (source.x < col.x)
        return source;
    else
	    return col;
#else
    return ConvertShadowDepthToVSM(tex2D(ShadowSampler0, vTex));
#endif
}

// ================================================================================================
float4 PS_ShadowMapFilterV(float2 vTex : TEXCOORD0) : COLOR
{		
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	float4 col = 0;

    float4 source = tex2D(ShadowSampler0, vTex);

	for (int i=0; i<g_iNumPixels; i++)
	{
		col += g_afPixelWeight[i] * tex2D(ShadowSampler0, vTex + float2(g_afPixelOffset[i], 0));
	}

    if (source.x < col.x)
	    return source;
    else
	    return col;
#else
    return tex2D(ShadowSampler0, vTex);
#endif
}

// ================================================================================================
technique ShadowMapGeneration_FilterH
{
    pass
    {
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    Lighting            = false;

#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif
		PixelShader	 = compile SHAD_PS_MINVER_2_0 PS_ShadowMapFilterH();
    }
}

// ================================================================================================
technique ShadowMapGeneration_FilterV
{
    pass
    {
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    Lighting            = false;

#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif
		PixelShader	 = compile SHAD_PS_MINVER_2_0 PS_ShadowMapFilterV();
    }
}

// ================================================================================================
// SSAO tweakable settings
// ================================================================================================
const float fSSAOIntensity		 : GLOBAL = 15.0f;
const float fSSAORadius			 : GLOBAL = 30.0f;
const int iSSAOIterations		 : GLOBAL = 4;
const float2 fSSAONoiseScale	 : GLOBAL;		// Kein Parameter fr die Artists
const float2 fSSAODepthMapStep	 : GLOBAL;		// Kein Parameter fr die Artists

// ================================================================================================
float doAmbientOcclusion(in float2 tex, in float2 texOffset, in float3 positionView, in float3 normal)
{
	// Calculate the view space position of the sample using view sapce depth
	float sampleDepth = tex2D(SamplerSSAODepth, tex + texOffset).y;
	float3 samplePosView;
	RecreatePositionInViewSpace(samplePosView, tex + texOffset, sampleDepth);

	// Calculate and return the ambient occlusion contribution of this sample
	float3 diff = samplePosView - positionView;
	float3 direction = normalize(diff);
	float distance = length(diff);
	
	// Reject rays that are close to parallel to the surface
	// 1 = not rejected, 0 = rejected
	float rejectRay = 1.0f;
	if(dot(normal, direction) < 0.5f)
		rejectRay = 0.0f;
	
	return rejectRay * max(0.0, dot(normal, direction)) * (1.0f / (1.0f + distance));
}

// ================================================================================================
float4 PS_ScreenSpaceAmbientOcclusion(float2 tex : TEXCOORD0) : COLOR
{
	float3 random = normalize(tex2D(SamplerSSAONoise, tex * fSSAONoiseScale).xyz * 2.0f - 1.0f);
	
	float depth = tex2D(SamplerSSAODepth, tex).y;
	float3 positionView;
	RecreatePositionInViewSpace(positionView, tex, depth);

	// Calculate two more positions from view space to recalculate the surface normal in view space
	float2 offsetX = float2(fSSAODepthMapStep.x, 0.0f);
	float2 offsetY = float2(0.0f, fSSAODepthMapStep.y);

	float flipNormal = 1.0f;

	// Calculate the positions from depth
	float depthX = 0.0f;
	float depthY = 0.0f;

	// Get two samples in each direction and use the samples with the nearest view space depth releative to positionView
	float depthX1 = tex2D(SamplerSSAODepth, tex + offsetX).y;
	float depthX2 = tex2D(SamplerSSAODepth, tex - offsetX).y;
	float depthY1 = tex2D(SamplerSSAODepth, tex + offsetY).y;
	float depthY2 = tex2D(SamplerSSAODepth, tex - offsetY).y;

	if(abs(depthX1 - depth) < abs(depthX2 - depth))
	{
		depthX = depthX1;
	}
	else
	{
		depthX = depthX2;
		offsetX = -offsetX;
		flipNormal = -1.0f;
	}

	if(abs(depthY1 - depth) < abs(depthY2 - depth))
	{
		depthY = depthY1;
	}
	else
	{
		depthY = depthY2;
		offsetY = -offsetY;
		flipNormal = -flipNormal;
	}

	float3 positionX;
	float3 positionY;
	RecreatePositionInViewSpace(positionX, tex + offsetX, depthX);
	RecreatePositionInViewSpace(positionY, tex + offsetY, depthY);

	// With the three positions perform the calculation of the surface normal
	float3 p1 = positionX.xyz - positionView.xyz;
	float3 p2 = positionY.xyz - positionView.xyz;

	float3 normal = cross(p1, p2);
	normal *= flipNormal;
	normal = normalize(normal);
	
	// ============
	// Do SSAO calculations
	// ============

	//Adjust radius to get a constant size (near objects need a larger radius because they appear larger in screen space)
	float radius = fSSAORadius / positionView.z;

	// 4 samples per iteration, so 16 samples max
	const int maxIterations = 4;
	const float2 vec[maxIterations] = {float2(1.0f, 0.0f), float2(-1.0f, 0.0f), float2(0.0f, 1.0f), float2(0.0f, -1.0f)};

	float ao = 0.0f;
	for(int i = 0; i < iSSAOIterations; ++i)
	{
		float2 coord1 = reflect(vec[i], random) * radius;
		float2 coord2 = float2(coord1.x * 0.707 - coord1.y * 0.707, coord1.x * 0.707 + coord1.y * 0.707);

		// Take 4 samples per iteration
		ao += doAmbientOcclusion(tex, coord1 * 0.25, positionView.xyz, normal);
		ao += doAmbientOcclusion(tex, coord2 * 0.5, positionView.xyz, normal);
		ao += doAmbientOcclusion(tex, coord1 * 0.75, positionView.xyz, normal);
		ao += doAmbientOcclusion(tex, coord2, positionView.xyz, normal);
	}
	
	ao = ao * (1.0f / (iSSAOIterations * 4.0f));
	ao = saturate(1.0f - (ao * fSSAOIntensity));
	
	float4 color = float4(ao, ao, ao, ao);

	// Test Visualisierung
	//color.xyz = normal;

	return color;
}

// ================================================================================================
technique ScreenSpaceAmbientOcclusion
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    Lighting            = false;
		AlphaTestEnable     = false;
	    AlphaBlendEnable    = false;
	    
	    //BLENDING
	    SrcBlend          = Zero;
	    DestBlend         = SrcAlpha;
	    
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif
		PixelShader		= compile SHAD_PS_MINVER_2_0 PS_ScreenSpaceAmbientOcclusion();
	}
}

// ================================================================================================
float4 PS_ScreenSpaceAmbientOcclusionApply(float2 vTex : TEXCOORD0) : COLOR
{
	// Die Post Process Rendered Texture hat kein Alpha. Deswegen den AO-Wert aus dem roten Kanal kopieren.
	float4 ssao = tex2D(Sampler0, vTex);
	ssao.a = ssao.r;
    return ssao;
}

// ================================================================================================
technique ScreenSpaceAmbientOcclusionApply
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    Lighting            = false;
		AlphaBlendEnable    = true;
		BlendOp				= Add;
		//AlphaBlendEnable    = false; // To visualize the SSAO results directly

		//BLENDING
	    SrcBlend          = Zero;
	    DestBlend         = SrcAlpha;
	    
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif		
        PixelShader		= compile SHAD_PS_MINVER_2_0 PS_ScreenSpaceAmbientOcclusionApply();
	}
}

// ================================================================================================
// FXAA tweakable settings
// ================================================================================================
const float2 g_InverseScreenSize	: GLOBAL;		// Kein Parameter fr die Artists
const float g_FXAASpanMax			: GLOBAL;	//= 8.0f
const float g_FXAAReduceMin			: GLOBAL;	//= 1.0f / 128.0f
const float g_FXAAReduceMul			: GLOBAL;	//= 1.0f / 8.0f

// ================================================================================================
float CalculateLuma(float3 color)
{
	static const float3 luma = float3(0.299f, 0.587f, 0.114f);

	return sqrt(dot(color, luma));
}

// ================================================================================================
float CalculateLumaFromTexture(float2 uv)
{
	return CalculateLuma(tex2D(Sampler0, uv).rgb);
}

// ================================================================================================
// Offset in pixels, will be converted to uv space using inverseScreenSize
float CalculateLumaFromTexture(float2 uv, float2 offset)
{
	return CalculateLumaFromTexture(uv + offset * g_InverseScreenSize);
}

// ================================================================================================
float4 PS_FXAA(float2 inUVCoord : TEXCOORD0) : COLOR
{
	float lumaM = CalculateLumaFromTexture(inUVCoord);
	float lumaTL = CalculateLumaFromTexture(inUVCoord, float2(-1.0f, -1.0f));
	float lumaTR = CalculateLumaFromTexture(inUVCoord, float2(1.0f, -1.0f));
	float lumaBL = CalculateLumaFromTexture(inUVCoord, float2(-1.0f, 1.0f));
	float lumaBR = CalculateLumaFromTexture(inUVCoord, float2(1.0f, 1.0f));

	float2 dir = float2(
		-((lumaTL + lumaTR) - (lumaBL + lumaBR)),
		((lumaTL + lumaBL) - (lumaTR + lumaBR))
	);

	float dirReduce = max((lumaTL + lumaTR + lumaBL + lumaBR) * (g_FXAAReduceMul * 0.25f), g_FXAAReduceMin);
	float inverseDirAdjust = 1.0f / (min(abs(dir.x), abs(dir.y)) + dirReduce);

	dir = min(float2(g_FXAASpanMax, g_FXAASpanMax), max(float2(-g_FXAASpanMax, -g_FXAASpanMax), dir * inverseDirAdjust));

	dir.x *= step(1.0f, abs(dir.x));
	dir.y *= step(1.0f, abs(dir.y));

	dir *= g_InverseScreenSize;

	float3 result1 = 0.5f * (tex2D(Sampler0, inUVCoord + dir * float2(1.0f / 3.0f - 0.5f, 1.0f / 3.0f - 0.5f)).rgb + tex2D(Sampler0, inUVCoord + dir * float2(2.0f / 3.0f - 0.5f, 2.0f / 3.0f - 0.5f)).rgb);
	float3 result2 = result1 * 0.5f + 0.25f * (tex2D(Sampler0, inUVCoord + dir * float2(-0.5f, -0.5f)).rgb + tex2D(Sampler0, inUVCoord + dir * float2(0.5f, 0.5f)).rgb);
	
	float lumaMin = min(lumaM, min(min(lumaTR, lumaTL), min(lumaBR, lumaBL)));
	float lumaMax = max(lumaM, max(max(lumaTR, lumaTL), max(lumaBR, lumaBL)));
	float lumaResult2 = CalculateLuma(result2);

	float4 outputColor;

	if(lumaResult2 < lumaMin || lumaResult2 > lumaMax)
		outputColor = float4(result1, 1.0f);
	else
		outputColor = float4(result2, 1.0f);
	
	return outputColor;
}

// ================================================================================================
technique FXAA
{
	pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    AlphaTestEnable     = false;
	    Lighting            = false;
		AlphaBlendEnable    = false;
	    
#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader	= compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader	= null;
#endif		
        PixelShader		= compile SHAD_PS_MINVER_2_0 PS_FXAA();
	}
}
