// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2018 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: jow
// date: 26.10.2018

#ifndef SHADERFRAGMENTS_HLSL
#define SHADERFRAGMENTS_HLSL

#include "GlobalFX.h"
#include <GlobalParameters.hlsl>

// ================================================================================================
#define SET_FILTER(idx, minf, magf, mipf, ani)\
    MinFilter[idx]      = (minf);\
    MagFilter[idx]      = (magf);\
    MipFilter[idx]      = (mipf);\
    MaxAnisotropy[idx]  = (ani);

// ================================================================================================
#define DECL_SAMPLER(s, t, minf, magf, mipf, ani)\
texture2D t;\
sampler2D s = sampler_state\
{\
	Texture = (t);\
	AddressU = Wrap;\
	AddressV = Wrap;\
    MinFilter = (minf);\
	MagFilter = (magf);\
  	MipFilter = (mipf);\
  	MaxAnisotropy = (ani);\
};

// ================================================================================================
#define DECL_BLEND_SAMPLER(s, t, minf, magf, mipf, ani)\
texture2D t;\
sampler2D s = sampler_state\
{\
	Texture = (t);\
	AddressU = Clamp;\
	AddressV = Clamp;\
    MinFilter = (minf);\
	MagFilter = (magf);\
  	MipFilter = (mipf);\
  	MaxAnisotropy = (ani);\
};

// ================================================================================================
#define DECL_FULL_SAMPLER(s, t, u, v, minf, magf, mipf, ani)\
texture2D t;\
sampler2D s = sampler_state\
{\
	Texture = (t);\
	AddressU = u;\
	AddressV = v;\
    MinFilter = (minf);\
	MagFilter = (magf);\
  	MipFilter = (mipf);\
  	MaxAnisotropy = (ani);\
};

// ================================================================================================
#define DECL_BLEND_MAP( T, S, MAGF, MINF, MIPF )\
texture2D T;\
sampler2D S = sampler_state\
{\
	Texture = (T);\
\
	MagFilter = (MAGF);\
  	MinFilter = (MINF);\
  	MipFilter = (MIPF);\
};

// ================================================================================================
#define DECL_SURFACE_MAP( T, S, MAGF, MINF, MIPF )\
texture2D T;\
sampler2D S = sampler_state\
{\
	Texture = (T);\
	AddressU = Wrap;\
	AddressV = Wrap;\
  	MagFilter = (MAGF);\
  	MinFilter = (MINF);\
  	MipFilter = (MIPF);\
};

// ================================================================================================
struct VS_PosColUV0
{
    float4 vPosition    : POSITION;
    float4 vColor       : COLOR;
    float2 vTex         : TEXCOORD0;
};

// ================================================================================================
struct VS_PosColUV0LightVPS
{
    float4 vPosition    : POSITION;
    float4 vColor       : COLOR;
    float2 vTex         : TEXCOORD0;

#if defined(ENABLE_SHADOWMAPPING)
    float4 LightVPS     : TEXCOORD1;
#endif
};

// ================================================================================================
struct VS_PosUV0
{
    float4 vPosition    : POSITION;
    float2 vTex         : TEXCOORD0;
};

// ================================================================================================
struct PS_ColUV0
{	
	float4 vColor  : COLOR;
	float2 vTex    : TEXCOORD0;
};

// ================================================================================================
struct LightData
{
	float3 Diffuse;	
	float3 Specular;
	float3 ReflectVS;
};

// ================================================================================================
struct PS_Out
{
	float4 Color : COLOR0;

#if defined(USE_DEPTH_MAP_MRT)
	float4 Depth : COLOR1;	// Depth.x = projected depth, Depth.y = linear depth in viewspace
#endif
};

// ================================================================================================
float3 TranslationFromMatrixTranspose( float4x4 i_M )
{
	return i_M._m03_m13_m23;
}

// ================================================================================================
float3 EnlightenAD( float3 i_PosOS, float3 i_NormOS, float3 i_LightDirOS, float3 i_Ambient, float3 i_Diffuse )
{
	//float3 C = clamp(i_Ambient + i_Diffuse * max(0, dot(i_NormOS, -i_LightDirOS)), float3(0,0,0), float3(1,1,1)); 
	float3 C = i_Ambient + i_Diffuse * max(0, dot(i_NormOS, -i_LightDirOS)); 
	return C;
}

// ================================================================================================
LightData EnlightenADS( float3 i_A, float3 i_D, float3 i_S, float3 i_LDirVS, float3 i_PosVS, float3 i_NormVS, float i_SpecPow )
{
	LightData LD;

	// Compute reflection vector (view space):
	LD.ReflectVS = normalize( 2 * dot( i_NormVS, i_LDirVS ) * i_NormVS - i_LDirVS );         

	// Compute view vector (view space):
	float3 NormPos = -normalize( i_PosVS );                

	// Compute diffuse and ambient contribution: 
	LD.Diffuse = i_A + i_D * max( 0, dot( i_NormVS, i_LDirVS ) );

	// Compute specular term:
	float DotRctNPos = dot( LD.ReflectVS, NormPos );
	LD.Specular = i_S * pow( max(0, DotRctNPos ), i_SpecPow); 	

	return LD;
}

// ================================================================================================
float2 ComputeWaterData( float2 i_LastQuantity, float2 i_CurQuantity, float i_NormOffset, float i_QFactor )
{
	float2 AbsLevelNFluid = lerp( i_LastQuantity, i_CurQuantity, i_NormOffset );
	AbsLevelNFluid *= float2( i_QFactor, i_QFactor );
		
	return AbsLevelNFluid;
}

// ================================================================================================
float2 ComputeWindAnimationDisplacement(float3 posWorldSpace, float2 direction)
{
	float2 displacement = float2(0.0f, 0.0f);

	if(g_HasVertexColor == 1)
	{
		float maxDisplacementPerpendicular = g_WindMaxDisplacement * 0.5f;
		float animationSpeedPerpendicular = g_WindAnimationSpeed / 2.5f;
		float waveLengthPerpendicular = g_WindWaveLength * 3.0f;
		float heightFactorPerpendicular = g_WindHeightFactor * 2.0f;

		float2 windDir = normalize(g_WindDirection);

		// Wind animation
		float2 windDirPerpendicular = float2(windDir.y, -windDir.x);
		float dist = length(cross(posWorldSpace, float3(windDirPerpendicular, 0.0f)));
		float distPerpendicular = length(cross(posWorldSpace, float3(windDir, 0.0f)));

		float windFactor = sin((Time + (dist / g_WindWaveLength) + (posWorldSpace.z * g_WindHeightFactor)) * g_WindAnimationSpeed);
		float windFactorPerpendicular = sin((Time + (distPerpendicular / waveLengthPerpendicular) + (posWorldSpace.z * heightFactorPerpendicular)) *animationSpeedPerpendicular);

		float2 displacementFactor = (windDir * windFactor * g_WindMaxDisplacement)
								  + (windDirPerpendicular.xy * windFactorPerpendicular * maxDisplacementPerpendicular);

		displacement = direction * displacementFactor * g_VertexAnimationScale;
	}

	return displacement;
}

#endif
