// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2008 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: ank
// date: 08.05.2006

#include "Skinning.h"

// ================================================================================================
// Standard skinning stuff (vertex lighting)
// ================================================================================================

// ================================================================================================
VS_OutColSpec4Tex VS_SkinningVertexLightingTest(VS_InSkinning i, uniform int NumBones)
{
    VS_OutNormal os = Skinning(i.Pos, i.BlendWeights, i.BlendIndices, i.Normal, NumBones);

    VS_OutColSpec4Tex o = (VS_OutColSpec4Tex)0;
    o.Pos = mul(float4(os.Pos, 1.0f), matViewProj); // position liegt noch im world space vor
    // o.Col = CalcAmbient() + CalcDiffuse(os.Normal, LightDir) + MaterialEmissive;
    // o.Col *= ColorBase;
    o.Amb = CalcAmbient() + MaterialEmissive;
    o.Amb *= ColorBase;
    o.Diff = CalcDiffuse(os.Normal, LightDir);
    o.Diff *= ColorBase;
    
    o.Spec = CalcSpecular(os.Normal, LightDir, normalize(os.Pos - CamPos));
    // o.Tex = i.Tex;
    // o.Tex1 = i.Tex;
    // o.Tex2 = i.Tex;
    // o.Tex3 = i.Tex;
    o.Tex01 = i.Tex.xyxy;
    o.Tex23 = i.Tex.xyxy;
    
#if defined(ENABLE_SHADOWMAPPING)
    // by mad: fr echtzeitschatten angepasst
    o.LightVPS = PerformTransformInLightSpace(float4(os.Pos, 1.0f));
#endif
    
    return o;
}

// ================================================================================================
VS_OutColSpec4TexWaterFog VS_SkinningVertexLighting(VS_InSkinning i, uniform int NumBones)
{
    VS_OutNormal os = Skinning(i.Pos, i.BlendWeights, i.BlendIndices, i.Normal, NumBones);

    VS_OutColSpec4TexWaterFog o = (VS_OutColSpec4TexWaterFog)0;
    o.Pos = mul(float4(os.Pos, 1.0f), matViewProj); // position liegt noch im world space vor
    // o.Col = CalcAmbient() + CalcDiffuse(os.Normal, LightDir) + MaterialEmissive;
    // o.Col *= ColorBase;
    o.Amb = CalcAmbient() + MaterialEmissive;
    //o.Amb *= ColorBase;
    o.Diff = CalcDiffuse(os.Normal, LightDir);
    //o.Diff *= ColorBase;
    
    o.Spec = CalcSpecular(os.Normal, LightDir, normalize(os.Pos - CamPos));
    // o.Tex = i.Tex;
    // o.Tex1 = i.Tex;
    // o.Tex2 = i.Tex;
    // o.Tex3 = i.Tex;
    o.Tex01 = i.Tex.xyxy;
    o.Tex23 = i.Tex.xyxy;
	
	float waterFog = CalculateWaterFogFX(os.Pos.z);
	o.NormalAndWaterFog   = float4(waterFog, waterFog, waterFog, waterFog);
    
#if defined(ENABLE_SHADOWMAPPING)
    // by mad: fr echtzeitschatten angepasst
    //o.LightVPS = PerformTransformInLightSpace(float4(os.Pos, 1.0f));
	o.LightVPS = float4(os.Pos, 1.0f);
#endif
    
#if defined(USE_DEPTH_MAP_MRT)
	o.PosVS = mul(float4(os.Pos, 1.0f), matView);
	o.PosVS.w = o.Pos.z / o.Pos.w;
#endif

    return o;
}

// ================================================================================================
technique LitSkinning
<
	string Description = "Skinning shader with vertex lighting";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "none";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_SkinningVertexLighting(4);
        //PixelShader = compile   SHAD_PS_MINVER_1_1  PS_VertexLighting();
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_VertexLighting_3Tex(false, false, true);
    }
}

technique LitSkinningUnderWater
<
	string Description = "Skinning shader with vertex lighting under water";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "none";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_SkinningVertexLighting(4);
        //PixelShader = compile   SHAD_PS_MINVER_1_1  PS_VertexLighting();
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_VertexLighting_3Tex(false, true, true);
    }
}


// ================================================================================================
technique LitSkinning_Special
<
	string Description = "Skinning shader with vertex lighting and special color layer";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "none";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_SkinningVertexLighting(4);
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_VertexLighting_3Tex(true, false, true);
    }
}

technique LitSkinningUnderWater_Special
<
	string Description = "Skinning shader with vertex lighting and special color layer";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "none";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_SkinningVertexLighting(4);
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_VertexLighting_3Tex(true, true, true);
    }
}


// ================================================================================================
// Skinning with 3 additional layers (vertex lighting)
// ================================================================================================

// ================================================================================================
// Skinning shader with 4 textures and 1 texture set
// ================================================================================================
VS_OutColSpec4Tex VS_SkinningVertexLighting_4Tex_1Set(VS_InSkinning i, uniform int NumBones)
{
    VS_OutNormal os = Skinning(i.Pos, i.BlendWeights, i.BlendIndices, i.Normal, NumBones);

    VS_OutColSpec4Tex o = (VS_OutColSpec4Tex)0;
    o.Pos = mul(float4(os.Pos, 1.0f), matViewProj); // position liegt noch im world space vor
    //o.Col = CalcAmbient() + CalcDiffuse(os.Normal, LightDir) + MaterialEmissive;
    //o.Col *= ColorBase;
    o.Amb = CalcAmbient() + MaterialEmissive;
    o.Amb *= ColorBase;
    o.Diff = CalcDiffuse(os.Normal, LightDir);
    o.Diff *= ColorBase;
    
    o.Spec = CalcSpecular(os.Normal, LightDir, normalize(os.Pos - CamPos));
    //o.Tex = i.Tex;
    //o.Tex1 = i.Tex;
    //o.Tex2 = i.Tex;
    //o.Tex3 = i.Tex;
    o.Tex01 = i.Tex.xyxy;
    o.Tex23 = i.Tex.xyxy;
    
#if defined(ENABLE_SHADOWMAPPING)
    // by mad: fr echtzeitschatten angepasst
    o.LightVPS = PerformTransformInLightSpace(float4(os.Pos, 1.0f));
#endif
   
    return o;
}

// ================================================================================================
// Skinning shader with 4 textures and 4 texture sets
// ================================================================================================
VS_OutColSpec4Tex VS_SkinningVertexLighting_4Tex_4Sets(VS_InSkinning_4Tex i, uniform int NumBones)
{
    VS_OutNormal os = Skinning(i.Pos, i.BlendWeights, i.BlendIndices, i.Normal, NumBones);

    VS_OutColSpec4Tex o = (VS_OutColSpec4Tex)0;
    o.Pos = mul(float4(os.Pos, 1.0f), matViewProj); // position liegt noch im world space vor
    //o.Col = CalcAmbient() + CalcDiffuse(os.Normal, LightDir) + MaterialEmissive;
    //o.Col *= ColorBase;
    o.Amb = CalcAmbient() + MaterialEmissive;
    o.Amb *= ColorBase;
    o.Diff = CalcDiffuse(os.Normal, LightDir);
    o.Diff *= ColorBase;
    
    o.Spec = CalcSpecular(os.Normal, LightDir, normalize(os.Pos - CamPos));
    //o.Tex = i.Tex;
    //o.Tex1 = i.Tex1;
    //o.Tex2 = i.Tex2;
    //o.Tex3 = i.Tex3;
    o.Tex01.yx = i.Tex;
    o.Tex01.zw = i.Tex1;
    o.Tex23.yx = i.Tex2;
    o.Tex23.zw = i.Tex3;
    
#if defined(ENABLE_SHADOWMAPPING)
    // by mad: fr echtzeitschatten angepasst
    o.LightVPS = PerformTransformInLightSpace(float4(os.Pos, 1.0f));
#endif
   
    return o;
}

// ================================================================================================
// Sampler0 ist fr Normal oder Fur gedacht
// Sampler1 fr FurDensity
// ================================================================================================
PS_Out PS_SkinningVertexLighting_4Tex(VS_OutColSpec4Tex p_psIn)
{
    float4 col = tex2D(SamplerBase, p_psIn.Tex01.xy);
    float4 col2 = tex2D(Sampler2, p_psIn.Tex01.zw);
    float4 col3 = tex2D(Sampler3, p_psIn.Tex23.xy);
    float4 col4 = tex2D(Sampler4, p_psIn.Tex23.zw);
    
    col *= ColorBase;
    col2 *= ColorLayer0;
    col3 *= ColorLayer1;
    col4 *= ColorLayer2;

    col = ModulateSrcAlphaInvSrcAlpha(col2, col);
    col = ModulateSrcAlphaInvSrcAlpha(col3, col);
    col = ModulateSrcAlphaInvSrcAlpha(col4, col);
    
#if defined(ENABLE_SHADOWMAPPING)
    // by mad: um schatten erweitert
    float fCurrentDepth = GetDepthInLightSpace(p_psIn.LightVPS);
    float fShadow = GetObjectInShadowFactor(fCurrentDepth, p_psIn.LightVPS, ShadowMap);

    // col *= p_psIn.Col;
    col *= clamp(p_psIn.Amb + fShadow*p_psIn.Diff,0,1);
    col += fShadow*p_psIn.Spec;
#else
    col *= clamp(p_psIn.Amb + p_psIn.Diff,0,1);
    col += p_psIn.Spec;
#endif    

	PS_Out output = (PS_Out)0;

	output.Color = float4LerpXYZ(col, WATERGROUND_COLOR, AnimalDepth);

    return output;
}

// ================================================================================================
technique LitSkinning_4Tex_1Set
<
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "none";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_SkinningVertexLighting_4Tex_1Set(4);
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_SkinningVertexLighting_4Tex();
    }
}

// ================================================================================================
technique LitSkinning_4Tex_4Sets
<
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "none";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_SkinningVertexLighting_4Tex_4Sets(4);
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_SkinningVertexLighting_4Tex();
    }
}



// ================================================================================================
// Hier wird eine dummy Berechnung durchgefhrt, damit die Normale benutzt und nicht von GB
// gelscht wird.
// ================================================================================================
VS_OutColTex VS_SkinningShadow(VS_InSkinning i, uniform int NumBones)
{
    float3 Pos = SkinningPosOnly(i.Pos, i.BlendWeights, i.BlendIndices, NumBones);

    VS_OutColTex o;
    o.Pos = mul(float4(Pos, 1.0f), matViewProj);  // Pos is in world space
    o.Amb = float4(1, 1, 1, saturate(i.Normal.x + 100)); // a ergibt auch 1
    o.Diff = float4(0,0,0,0);
    o.Tex = i.Tex;
    
#if defined(ENABLE_SHADOWMAPPING)
    // by mad: fr echtzeitschatten angepasst
    o.LightVPS = PerformTransformInLightSpace(float4(Pos, 1.0f));
#endif
   
    return o;
}

// ================================================================================================
VS_OutColTex VS_SkinningShadow_4Tex_4Sets(VS_InSkinning_4Tex i, uniform int NumBones)
{
    float3 Pos = SkinningPosOnly(i.Pos, i.BlendWeights, i.BlendIndices, NumBones);

    VS_OutColTex o;
    o.Pos = mul(float4(Pos, 1.0f), matViewProj);  // Pos is in world space
    o.Amb = float4(1, 1, 1, saturate(i.Normal.x + i.Tex1.x + i.Tex2.x + i.Tex3.x + 100)); // a ergibt auch 1
    o.Diff = float4(0,0,0,0);
    o.Tex = i.Tex;
    
#if defined(ENABLE_SHADOWMAPPING)
    // by mad: fr echtzeitschatten angepasst
    o.LightVPS = PerformTransformInLightSpace(float4(Pos, 1.0f));
#endif
   
    return o;
}

// ================================================================================================
technique UnlitSkinning
<
	string Description = "Skinning shader without lighting";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "none";
>
{
    pass p0
    {
        FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = true;
	    ZWriteEnable        = true;
	    AlphaTestEnable     = true;
	    //AlphaTestEnable     = false;
	    AlphaRef            = 128;
	    AlphaFunc           = greater;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    SpecularEnable      = false;
	    	    
	    /*ColorOp[0]          = SelectArg1;
		ColorArg1[0]        = Current;
		AlphaOp[0]          = Disable;
		
	    AddressU[0]         = Clamp;
	    AddressV[0]         = Clamp;
	    MinFilter[0]        = Point;
        MagFilter[0]        = Point;
        MipFilter[0]        = Point;
        MaxAnisotropy[0]    = 1;*/
	            
        //VertexShader = compile vs_1_1 VS_SkinningShadow(4);
        //PixelShader = 0;
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_SkinningShadow(4);
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_ColorAndTextureAlpha();
    }
}

// ================================================================================================
technique UnlitSkinning_4Tex
<
	string Description = "Skinning shader without lighting";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "none";
>
{
    pass p0
    {
        FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = true;
	    ZWriteEnable        = true;
	    AlphaTestEnable     = true;
	    //AlphaTestEnable     = false;
	    AlphaRef            = 128;
	    AlphaFunc           = greater;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    SpecularEnable      = false;
	    	    
	    ColorOp[0]          = SelectArg1;
		ColorArg1[0]        = Current;
		AlphaOp[0]          = Modulate;
		AlphaArg1[0]        = Current;
		AlphaArg2[0]        = Texture;
		ColorOp[1]          = Disable;
		AlphaOp[1]          = Disable;
		
	    AddressU[0]         = Clamp;
	    AddressV[0]         = Clamp;
	    MinFilter[0]        = Point;
        MagFilter[0]        = Point;
        MipFilter[0]        = Point;
        MaxAnisotropy[0]    = 1;//*/
	            
        VertexShader = compile vs_1_1 VS_SkinningShadow_4Tex_4Sets(4);
        PixelShader = 0;
        
        //VertexShader = compile SHAD_VS_MINVER_1_1 VS_SkinningShadow_4Tex_4Sets(4);
        //PixelShader = compile SHAD_PS_MINVER_1_1 PS_ColorAndTextureAlpha();
    }
}
