// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2008 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: ank
// date: 08.05.2006

#include "GlobalFX.h"
#include "Skinning.h"

// ================================================================================================
struct VS_OutColSpec2Tex
{
    float4 Pos  : POSITION;
    float4 Amb  : COLOR;
    float4 Diff : COLOR1;
    float4 Spec : TEXCOORD0;
    float2 Tex0 : TEXCOORD1;
    float2 Tex1 : TEXCOORD2;

#if defined(ENABLE_SHADOWMAPPING)
    float4 LightVPS : TEXCOORD3;
#endif

#if defined(SHADERMODEL3)
    float Fog : FOG;
#endif
};

// ================================================================================================
VS_OutColSpec2Tex VS_SkinningVertexLightingCaustics(VS_InSkinning i, uniform int NumBones)
{
    VS_OutNormal os = Skinning(i.Pos, i.BlendWeights, i.BlendIndices, i.Normal, NumBones);
 
    VS_OutColSpec2Tex o;
    o.Pos = mul(float4(os.Pos, 1), matViewProj); // position liegt noch im world space vor
    o.Amb  = CalcAmbient() + MaterialEmissive;
    o.Amb  *= ColorBase;
    o.Diff = CalcDiffuse(os.Normal, LightDir);
    o.Diff *= ColorBase;
    
    o.Spec = CalcSpecular(os.Normal, LightDir, normalize(os.Pos - CamPos));
    o.Spec.w = CalculateCausticStrength(os.Normal);   // use w for caustics visibility
    o.Tex0 = i.Tex;
    o.Tex1 = i.Tex;
    
#if defined(ENABLE_SHADOWMAPPING)
    // by mad: fr echtzeitschatten angepasst
    o.LightVPS = PerformTransformInLightSpace(float4(os.Pos, 1.0f));
#endif

#if defined(SHADERMODEL3)
    o.Fog = 1.0f - (o.Pos.z/o.Pos.w);
#endif
    return o;
}

// ================================================================================================
PS_Out PS_VertexLightingCaustics(VS_OutColSpec2Tex p_psIn)
{	
#if defined(ENABLE_SHADOWMAPPING)
    // by mad: um schatten erweitert
    float fCurrentDepth = GetDepthInLightSpace(p_psIn.LightVPS);
    float fShadow = GetObjectInShadowFactor(fCurrentDepth, p_psIn.LightVPS, ShadowMap);

	float4 colTex = tex2D(SamplerBase, p_psIn.Tex0);
    colTex = colTex * (p_psIn.Amb + fShadow * p_psIn.Diff);
    
    if (fShadow > 0)
    {
        colTex.rgb += p_psIn.Spec.rgb;
    
        //float4 colCaust = tex2D(Sampler1, p_psIn.Tex0);
        //colCaust *= p_psIn.Spec.w * min(1, AnimalDepth * 6);
        //colTex.rgb += colCaust.rgb * CaustEnhanceFac;
        colTex.rgb += GetCausticColor(AnimalDepth, p_psIn.Tex0, p_psIn.Spec.w);
    }
#else
	float4 colTex = tex2D(SamplerBase, p_psIn.Tex0);
    colTex     *= (p_psIn.Amb + p_psIn.Diff);
    colTex.rgb += p_psIn.Spec.rgb;
    
    //float4 colCaust = tex2D(Sampler1, p_psIn.Tex0);
    //colCaust *= p_psIn.Spec.w * min(1, AnimalDepth * 6);
    //colTex.rgb += colCaust.rgb * CaustEnhanceFac;
    colTex.rgb += GetCausticColor(AnimalDepth, p_psIn.Tex0, p_psIn.Spec.w);
#endif    

	PS_Out output = (PS_Out)0;
	output.Color = float4LerpXYZ(colTex, WATERGROUND_COLOR, AnimalDepth);

    return output;
}

// ================================================================================================
// Skinning mit Vertex-Lighting und Caustics
// ================================================================================================
technique LitSkinningCaustics
<
	string Description = "Skinning shader with vertex lighting and caustics";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "none";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_SkinningVertexLightingCaustics(4);
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_VertexLightingCaustics();
    }
}


// ================================================================================================
VS_OutTSD VS_SkinningNBT(VS_InSkinnedNBT i, uniform int NumBones)
{
    VS_OutSkinnedNBT os = SkinningNBT(i, NumBones);
 
    VS_OutTSD o = (VS_OutTSD)0;  
    o.Pos = mul(float4(os.Pos, 1.0f), matViewProj);  // os.Pos is in world space
    o.Col = CalcAmbient() + MaterialEmissive;
    o.Col *= ColorBase;
    
    o.Spec.w = CalculateCausticStrength(os.Normal);   // use w for caustics visibility
   
    o.Tex01.xy = i.Tex0.xy;
    if (numLayer > 1)
        o.Tex01.zw = i.Tex1.xy;
    else
        o.Tex01.zw = o.Tex01.xy;
        
    if (numLayer > 2)
        o.Tex23.xy = i.Tex2.xy;
    else
        o.Tex23.xy = o.Tex01.xy;
        
    if (numLayer > 3)
        o.Tex23.zw = i.Tex3.xy;
    else
        o.Tex23.zw = o.Tex01.xy;

    float3x3 matTangent;
    matTangent[0] = os.Tangent;
    matTangent[1] = os.Binormal;
    matTangent[2] = os.Normal;
    
    float3 ViewDir = normalize(os.Pos - CamPos);    // world space view direction
    o.TSLightDir = mul(matTangent, LightDir);       // world space light dir to tangent space light dir
    o.TSViewDir  = mul(matTangent, ViewDir);         // world space view dir to tangent space view dir
    
    o.WorldUpAndWater.xyz = mul(matTangent, float3(0,0,1));
    o.WorldUpAndWater.w   = CalculateWaterFogFX(os.Pos.z);
    
#if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(float4(os.Pos, 1.0f));
#endif
    
    return o;
}

// ================================================================================================
PS_Out PS_TSDLightingColorLerp(VS_OutTSD p_psIn)
{    
    //float4 col = PS_TSDLighting(p_psIn);
	PS_Out output = (PS_Out)0;
	output.Color = PS_TSDLighting_3Tex(p_psIn, false, false, true).Color;

    // by mad -> is done internal
    //col.rgb += GetCausticColor(p_psIn.WorldUpAndWater.w, p_psIn.Tex01.xy, p_psIn.Spec.w);
    //return float4LerpXYZ(col, WATERGROUND_COLOR, p_psIn.WorldUpAndWater.w);
    // old version:
    // return float4LerpXYZ(col, WATERGROUND_COLOR, AnimalDepth);
    return output;
}

// ================================================================================================
// Skinning with normal mapping
// ================================================================================================
technique LitSkinningNBT
<
	string Description = "Skinning shader with normal mapping";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "ATI";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	    AlphaRef        = 128;
	        
        VertexShader = compile  SHAD_VS_MINVER_2_0  VS_SkinningNBT(4);
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_TSDLightingColorLerp();
    }
}
// ================================================================================================
// Hier wird eine dummy Berechnung durchgefhrt, damit die Normalen benutzt und nicht vom Compiler
// gelscht werden.
// ================================================================================================
VS_OutColSpec4Tex VS_SkinningNBTLoD(VS_InSkinnedNBT i, uniform int NumBones)
{
    VS_OutNormal os = Skinning(i.Pos, i.BlendWeights, i.BlendIndices, i.Normal, NumBones);
 
    VS_OutColSpec4Tex o = (VS_OutColSpec4Tex)0;
    o.Pos = mul(float4(os.Pos, 1.0f), matViewProj);  // os.Pos is in world space
    //o.Col = CalcAmbient() + CalcDiffuse(os.Normal, LightDir) + MaterialEmissive;
    //o.Col *= ColorBase;
    //o.Col.a *= saturate(i.Binormal.x + i.Tangent.x + 10); // x1
    
    o.Amb = CalcAmbient() + MaterialEmissive;
    o.Amb *= ColorBase;
    o.Amb.a *= saturate(i.Binormal.x + i.Tangent.x + 10); // x1
    o.Diff = CalcDiffuse(os.Normal, LightDir);
    o.Diff *= ColorBase;
    
    o.Spec = CalcSpecular(os.Normal, LightDir, normalize(os.Pos - CamPos));
    o.Spec.w = CalculateCausticStrength(os.Normal);   // use w for caustics visibility
    //o.Tex = i.Tex;
    //o.Tex1 = i.Tex;
    //o.Tex2 = i.Tex;
    //o.Tex3 = i.Tex;
    o.Tex01.xy = i.Tex0.xy;
    o.Tex01.zw = i.Tex1.xy;
    o.Tex23.xy = i.Tex2.xy;
    o.Tex23.zw = i.Tex3.xy;
    
#if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(float4(os.Pos, 1.0f));
#endif
    
    return o;
}

// ================================================================================================
PS_Out PS_VertexLighting_DeepSeaColor(VS_OutColSpec4Tex p_psIn)
{
	PS_Out output = (PS_Out)0;

    float4 col = PS_VertexLighting(p_psIn).Color;

	output.Color = float4LerpXYZ(col, WATERGROUND_COLOR, AnimalDepth);
    return output;
}

// ================================================================================================
technique LitSkinningNBTLoD
<
	string Description = "Skinning shader with(out) normal mapping (LoD-version)";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "ATI";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	    AlphaRef        = 128;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_SkinningNBTLoD(4);
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_VertexLighting_DeepSeaColor();
    }
}





// ================================================================================================
// Hier wird eine dummy Berechnung durchgefhrt, damit die Normalen benutzt und nicht vom Compiler
// gelscht werden.
// ================================================================================================
VS_OutColTex VS_NBTShadow(VS_InSkinnedNBT i, uniform int NumBones)
{
    float3 Pos = SkinningNBTPosOnly(i, NumBones);
 
    VS_OutColTex o;       
    o.Pos = mul(float4(Pos, 1.0f), matViewProj);  // Pos is in world space
    o.Amb = float4(1, 1, 1, saturate(i.Normal.x + i.Binormal.x + i.Tangent.x + 10 + i.Tex1.x + i.Tex2.x + i.Tex3.x)); // a ergibt auch 1
    o.Diff = float4(0,0,0,0);
    o.Tex = i.Tex0;
    
#if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(float4(Pos, 1.0f));
#endif
    
    return o;
}

// ================================================================================================
technique UnlitSkinningNBT
<
	string Description = "Skinning shader without lighting but normals in calculation";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "ATI";
>
{
    pass p0
    {
        FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = true;
	    ZWriteEnable        = true;
	    AlphaTestEnable     = true;
	    //AlphaTestEnable     = false;
	    AlphaRef            = 128;
	    AlphaFunc           = greater;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    SpecularEnable      = false;
	    
	    /*ColorOp[0]          = SelectArg1;
		ColorArg1[0]        = Current;
		AlphaOp[0]          = SelectArg1;
		AlphaArg1[0]        = Texture;
	    
	    AddressU[0]         = Clamp;
	    AddressV[0]         = Clamp;
	    MinFilter[0]        = Point;
        MagFilter[0]        = Point;
        MipFilter[0]        = Point;
        MaxAnisotropy[0]    = 1;*/
	        
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_NBTShadow(4);
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_ColorAndTextureAlpha();
        //VertexShader = compile vs_1_1 VS_NBTShadow(4);
        //PixelShader = 0;
        
    }
}

// ================================================================================================
PS_Out PS_TSDLightingColorLerp_Special(VS_OutTSD p_psIn)
{
	PS_Out output = (PS_Out)0;

    // don't use internal caustic/waterfog calculations, shader version to limited...
    float4 col = PS_TSDLighting_3Tex(p_psIn, true, false, false).Color;

	output.Color = float4LerpXYZ(col, WATERGROUND_COLOR, saturate(p_psIn.WorldUpAndWater.w));
    return output;
}

// ================================================================================================
// Skinning with normal mapping und special textur blending...
// ================================================================================================
technique LitSkinningNBT_Special
<
	string Description = "Skinning shader with normal mapping and special decaying";
	int BonesPerPartition = MAX_BONES;
	string NBTMethod = "ATI";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	    AlphaRef        = 128;
	        
        VertexShader = compile  SHAD_VS_MINVER_2_0  VS_SkinningNBT(4);
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_TSDLightingColorLerp_Special();
    }
}