// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2008 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: jow
// date: 29.08.2018

#include "Skinning.h"

sampler2D SamplerSkirt = sampler_state
{
    Texture = (BaseMap);
    AddressU = Wrap;
    AddressV = Wrap;
    MinFilter = Point;
    MagFilter = Point;
    MipFilter = Linear;
    MaxAnisotropy = (MAX_ANISOTROPY);
};

const float2	heightLimits	 : GLOBAL;

struct VS_SkirtIn
{
	float4 position : POSITION;
	float3 normal	: NORMAL;
	float2 texUV	: TEXCOORD0;
};

struct VS_SkirtOut
{
	float4 position				: POSITION;
	float4 ambient				: COLOR;
	float4 diffuse				: COLOR1;
	float2 texUV				: TEXCOORD0;	// x = texU; y = worldposition.z
#if defined(USE_DEPTH_MAP_MRT)
	float4 positionVSAndDepth	: TEXCOORD1;	// x, y, z = View Space Position, w = projected depth
#endif
};

VS_SkirtOut VS_Skirt(VS_SkirtIn vsIn)
{
	VS_SkirtOut vsOut = (VS_SkirtOut)0;

	float4 positionWorld = mul(vsIn.position, matWorld);
	float3 normalWorld = normalize(mul(vsIn.normal, (float3x3)matWorld));

	vsOut.position = mul(vsIn.position, matWorldViewProjection);
	vsOut.ambient = CalcAmbient();
	vsOut.diffuse = CalcDiffuse(normalWorld, LightDir);

	vsOut.texUV = float2(vsIn.texUV.x, positionWorld.z);

#if defined(USE_DEPTH_MAP_MRT)
	vsOut.positionVSAndDepth = mul(vsIn.position, matWorldView);
	vsOut.positionVSAndDepth.w = vsOut.position.z / vsOut.position.w;
#endif

	return vsOut;
}

PS_Out PS_Skirt(VS_SkirtOut psIn)
{
	PS_Out psOut = (PS_Out)0;

	float heightDistance = heightLimits.y - heightLimits.x;
	float height = psIn.texUV.y - heightLimits.x;

	float texV = 1.0f - saturate(height / heightDistance);

	float2 uv = float2(psIn.texUV.x, texV);

	float4 color = tex2D(SamplerSkirt, uv);

	color.rgb *= saturate(psIn.ambient + psIn.diffuse);

	color.a = 1.0f;
	psOut.Color = color;

#if defined(USE_DEPTH_MAP_MRT)
	psOut.Depth.x = 1.0f - psIn.positionVSAndDepth.w;
	psOut.Depth.y = psIn.positionVSAndDepth.z;
	psOut.Depth.a = 1.0f;
#endif

	return psOut;
}

// ================================================================================================
technique SkirtStandard
<
	string Description = "Standard shader for the skirt (map border)";
	bool UsesNIRenderState = true;
	string NBTMethod = "none";
>
{
    pass p0
    { 
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
		ZWriteEnable	= true;
	    
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_Skirt();
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_Skirt();
    }
}