// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2008 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: ank
// date: 02.08.2006

#include "Skinning.h"

#define ShadowRenderStates\
    FillMode            = solid;\
	ShadeMode           = flat;\
	CullMode            = cw;\
	ZEnable             = true;\
	ZWriteEnable        = true;\
	AlphaTestEnable     = true;\
	AlphaBlendEnable    = false;\
	Lighting            = false;\
	SpecularEnable      = false;\
	FogEnable           = false/*;\
	ColorOp[0]          = SelectArg1;\
	ColorArg1[0]        = Current;\
	AlphaOp[0]          = SelectArg1;\
	AlphaArg1[0]        = Texture;\
	AddressU[0]         = Clamp;\
	AddressV[0]         = Clamp;\
	MinFilter[0]        = Point;\
    MagFilter[0]        = Point;\
    MipFilter[0]        = Point;\
    MaxAnisotropy[0]    = 1*/

// ================================================================================================
VS_OutColSpec VS_VertexLighting_NoTex(float4 Pos : POSITION, float3 Normal : NORMAL, float3 Color : COLOR, uniform bool windAnimation = true)
{
    float4 PosWorld = mul(Pos, matWorld);
    float3 NormalWorld = normalize(mul(Normal, (float3x3)matWorld));

	// wind animation
	if(windAnimation)
	{
		float2 direction = 1.0f - Color.xy;
		PosWorld.xy += ComputeWindAnimationDisplacement(PosWorld.xyz, direction);
	}

	// output
    VS_OutColSpec o;
    o.Pos   = mul(PosWorld, matViewProj);
    o.Amb   = CalcAmbient() + MaterialEmissive;
    //o.Amb   *= ColorBase;		// by jow: in den Pixel Shader verschoben
    o.Diff  = CalcDiffuse(NormalWorld, LightDir);
    //o.Diff  *= ColorBase;		// by jow: in den Pixel Shader verschoben
    o.Spec   = CalcSpecular(NormalWorld, LightDir, normalize(float3(PosWorld.xyz) - float3(CamPos.xyz)));
    o.Spec.w = CalculateCausticStrength(NormalWorld);   // use w for caustics visibility

#if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(PosWorld);
#endif
    
    return o;
}

// ================================================================================================
VS_OutColSpec4TexWaterFog VS_VertexLighting(float4 Pos    : POSITION, 
                                    float3 Normal : NORMAL,
                                    float2 Tex0   : TEXCOORD0,
                                    float2 Tex1   : TEXCOORD1,
                                    float2 Tex2   : TEXCOORD2,
                                    float2 Tex3   : TEXCOORD3,
									float3 Color  : COLOR,	// Color used for wind direction
									uniform bool windAnimation = true)
{
    VS_OutColSpec oTmp = VS_VertexLighting_NoTex(Pos, Normal, Color, windAnimation);
    VS_OutColSpec4TexWaterFog o = (VS_OutColSpec4TexWaterFog)0;
    o.Pos   = oTmp.Pos;
    o.Amb   = oTmp.Amb;
    o.Diff   = oTmp.Diff;
    o.Spec   = oTmp.Spec;
        
    o.Tex01.xy  = mul(float4(Tex0.xy, 1, 1), matTex);   // Texturtransformation
    
    if (numLayer > 1)
        o.Tex01.zw = mul(float4(Tex1.xy, 1, 1), matTex); // Texturtransformation
    else
        o.Tex01.zw = o.Tex01.xy;
        
    if (numLayer > 2)
        o.Tex23.xy = mul(float4(Tex2.xy, 1, 1), matTex); // Texturtransformation
    else
        o.Tex23.xy = o.Tex01.xy;
    
    if (numLayer > 3)
        o.Tex23.zw = mul(float4(Tex3.xy, 1, 1), matTex); // Texturtransformation
    else
        o.Tex23.zw = o.Tex01.xy;
	
	float4 PosWorld = mul(Pos, matWorld);
	float waterFog = CalculateWaterFogFX(PosWorld.z);
	float3 worldNormal = normalize(mul(Normal, (float3x3)matWorld));
	o.NormalAndWaterFog   = float4(worldNormal, waterFog);
	
#if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = oTmp.LightVPS;
#endif

#if defined(USE_DEPTH_MAP_MRT)
	o.PosVS = mul(Pos, matWorldView);
	o.PosVS.w = o.Pos.z / o.Pos.w;
#endif
    
    return o;
}

// ================================================================================================
technique Standard
<
	string Description = "Standard shader for unskinned objects";
	bool UsesNIRenderState = true;
	string NBTMethod = "none";
>
{
    pass p0
    { 
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	    
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_VertexLighting();
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_VertexLighting_3Tex(false, true); // Underwater hier auch aktiviert, sonst gibt es Probleme mit Zunen, wenn diese Teils im Wasser und Teils Ausserhalb des Wassers sind.
    }
}

// ================================================================================================
technique StandardUnderWater
<
	string Description = "Standard shader for unskinned objects under water";
	bool UsesNIRenderState = true;
	string NBTMethod = "none";
>
{
    pass p0
    { 
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	    
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_VertexLighting();
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_VertexLighting_3Tex(false, true);
    }
}

// ================================================================================================
technique Standard_NoTex
<
	string Description = "Standard shader for unskinned objects";
	bool UsesNIRenderState = true;
	string NBTMethod = "none";
>
{
    pass p0
    { 
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	    
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_VertexLighting_NoTex();
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_VertexLighting_NoTex();
    }
}



// ================================================================================================
VS_OutCol VS_StandardShadow_NoTex(float4 Pos : POSITION, float3 Normal : NORMAL)
{
    VS_OutCol o;
    o.Pos = mul(Pos, matWorldViewProjection);
    o.Amb  = float4(1,1,1, min(1, Normal.x+100));
    o.Diff = float4(1,1,1,1);

#if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace   (Pos);
#endif
    
    return o;
}

// ================================================================================================
VS_OutColTex VS_StandardShadow(float4 Pos : POSITION, float3 Normal : NORMAL, float2 Tex : TEXCOORD0)
{
    VS_OutColTex o;
    o.Pos = mul(Pos, matWorldViewProjection);
    o.Amb  = float4(1,1,1, min(1, Normal.x+100));
    o.Diff = float4(1,1,1,1);
    o.Tex = Tex;

#if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(Pos);
#endif
    
    return o;
}

// ================================================================================================
technique StandardShadow
{
    pass p0
    { 
        ShadowRenderStates;
	    
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_StandardShadow();
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_ColorAndTextureAlpha();
        //VertexShader = compile vs_1_1 VS_StandardShadow();
        //PixelShader = 0;
    }
}

// ================================================================================================
technique StandardShadow_NoTex
{
    pass p0
    { 
        ShadowRenderStates;
	    
        VertexShader = compile vs_1_1 VS_StandardShadow_NoTex();
        PixelShader = 0;
        //VertexShader = compile SHAD_VS_MINVER_1_1 VS_StandardShadow_NoTex();
        //PixelShader = compile SHAD_PS_MINVER_2_0 PS_VertexLighting_NoTex();
    }
}



// ================================================================================================
VS_OutTSD VS_NBT(VS_InNBT i)
{
    float4 PosWorld = mul(i.Pos, matWorld);
 
    VS_OutTSD o = (VS_OutTSD)0;   
    o.Pos = mul(PosWorld, matViewProj);
    o.Col = CalcAmbient() + MaterialEmissive;
    
    o.Spec.w = CalculateCausticStrength(i.Normal);   // use w for caustics visibility
    
    o.Tex01.xy = mul(float4(i.Tex0.xy, 1, 1), matTex); // Texturtransformation
    if (numLayer > 1)
        o.Tex01.zw = mul(float4(i.Tex1.xy, 1, 1), matTex); // Texturtransformation
    else
        o.Tex01.zw = o.Tex01.xy;
        
    if (numLayer > 2)
        o.Tex23.xy = mul(float4(i.Tex2.xy, 1, 1), matTex); // Texturtransformation
    else
        o.Tex23.xy = o.Tex01.xy;
    
    if (numLayer > 3)
        o.Tex23.zw = mul(float4(i.Tex3.xy, 1, 1), matTex); // Texturtransformation
    else
        o.Tex23.zw = o.Tex01.xy;

    float3x3 matTangent;
    matTangent[0] = normalize(mul(i.Tangent, (float3x3)matWorld));
    matTangent[1] = normalize(mul(i.Binormal, (float3x3)matWorld));
    matTangent[2] = normalize(mul(i.Normal, (float3x3)matWorld));

    o.WorldUpAndWater.xyz = mul(matTangent, float3(0,0,1));
    o.WorldUpAndWater.w   = CalculateWaterFogFX(PosWorld.z);
  
    float3 ViewDir = normalize(PosWorld - CamPos);  // world space view direction
    o.TSLightDir = mul(matTangent, LightDir);       // world space light dir to tangent space light dir
    o.TSViewDir  = mul(matTangent, ViewDir);        // world space view dir to tangent space view dir
    
    //o.TSLightDir = i.Normal * 0.5 + 0.5;
    //o.TSLightDir = i.Normal;
    //o.TSLightDir = i.Binormal;
    
    /*if (g_fNBTValue < 1)    // normales Vertexlighting fr den Fadingbereich
    {
        float4 colVertex = o.Col + CalcDiffuse(matTangent[2], LightDir);
        //colVertex += CalcSpecular(matTangent[2], LightDir, normalize(PosWorld - float3(CamPos.xyz)));
        o.Col = float4Lerp(colVertex, o.Col, g_fNBTValue);
        o.Spec = CalcSpecular(matTangent[2], LightDir, normalize(PosWorld - float3(CamPos.xyz)));
    }*/
    
#if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(PosWorld);
#endif

#if defined(USE_DEPTH_MAP_MRT)
	o.PosVS = mul(i.Pos, matWorldView);
	o.PosVS.w = o.Pos.z / o.Pos.w;
#endif

    return o;
}

// ================================================================================================
PS_Out PS_StandardNBT(VS_OutTSD p_psIn)
{
    return PS_TSDLighting_3Tex(p_psIn, false, true, true);
}

// ================================================================================================
technique StandardNBT
<
	string Description = "Shader with normal mapping";
	string NBTMethod = "ATI";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_NBT();
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_StandardNBT(); //PS_TSDLighting();
    }
}

// ================================================================================================
PS_Out PS_StandardNBT_Special(VS_OutTSD p_psIn)
{
    return PS_TSDLighting_3Tex(p_psIn, true, false, true);
}

// ================================================================================================
technique StandardNBT_Special
<
	string Description = "Shader with normal mapping";
	string NBTMethod = "ATI";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_NBT();
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_StandardNBT_Special();
    }
}

// ================================================================================================
VS_OutColSpec4Tex VS_NBTLoD(VS_InNBT i)
{
    float4 PosWorld = mul(i.Pos, matWorld);
    float4 NormalWorld = normalize(mul(i.Normal, matWorld));
 
    VS_OutColSpec4Tex o = (VS_OutColSpec4Tex)0;
    o.Pos = mul(PosWorld, matViewProj);
    //o.Col = CalcAmbient() + CalcDiffuse(NormalWorld, LightDir) + MaterialEmissive;
    //o.Col *= ColorBase;
    //o.Col.a *= saturate(i.Binormal.x + i.Tangent.x + 10); // x1
    o.Amb = CalcAmbient() + MaterialEmissive;
    o.Amb *= ColorBase;
    o.Amb.a *= saturate(i.Binormal.x + i.Tangent.x + 10); // x1
    o.Diff = CalcDiffuse(NormalWorld, LightDir);
    o.Diff *= ColorBase;
    
    o.Spec = CalcSpecular(NormalWorld, LightDir, normalize(float3(PosWorld.xyz) - float3(CamPos.xyz)));
    //o.Spec = CalcSpecularWithAlpha(NormalWorld, LightDir, normalize(float3(PosWorld.xyz) - float3(CamPos.xyz)));
    o.Spec.w = CalculateCausticStrength(NormalWorld);   // use w for caustics visibility
    
    o.Tex01.xy = mul(float4(i.Tex0.xy, 1, 1), matTex); // Texturtransformation
    //o.Tex01.zw = i.Tex;
    //o.Tex23.xy = i.Tex;
    //o.Tex23.zw = i.Tex;
    
    if (numLayer > 1)
        o.Tex01.zw = mul(float4(i.Tex1.xy, 1, 1), matTex); // Texturtransformation
    else
        o.Tex01.zw = o.Tex01.xy;
        
    if (numLayer > 2)
        o.Tex23.xy = mul(float4(i.Tex2.xy, 1, 1), matTex); // Texturtransformation
    else
        o.Tex23.xy = o.Tex01.xy;
    
    if (numLayer > 3)
        o.Tex23.zw = mul(float4(i.Tex3.xy, 1, 1), matTex); // Texturtransformation
    else
        o.Tex23.zw = o.Tex01.xy;    
    
#if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(PosWorld);
#endif

    return o;
}

// ================================================================================================
technique StandardNBTLoD
<
	string Description = "Shader with(out) normal mapping (LoD-version)";
	string NBTMethod = "ATI";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_NBTLoD();
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_VertexLighting_3Tex();
    }
}



// ================================================================================================
VS_OutColTex VS_NBTShadow(VS_InNBT i)
{    
    VS_OutColTex o;       
    o.Pos = mul(i.Pos, matWorldViewProjection);
    o.Amb = float4(1, 1, 1, saturate(i.Normal.x + i.Binormal.x + i.Tangent.x  + i.Tex1.x + i.Tex2.x + i.Tex3.x + 10)); // a ergibt auch 1
    o.Diff = float4(1, 1, 1, 1);
    o.Tex = i.Tex0;

#if defined(ENABLE_SHADOWMAPPING)
    o.LightVPS = PerformTransformInLightSpace(i.Pos);
#endif

    return o;
}

// ================================================================================================
technique StandardNBTShadow
<
	string Description = "Shadow NBT shader";
	string NBTMethod = "ATI";
>
{
    pass p0
    {
        ShadowRenderStates;
	        
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_NBTShadow();
        PixelShader = compile SHAD_PS_MINVER_2_0 PS_ColorAndTextureAlpha();
        //VertexShader = compile vs_1_1 VS_NBTShadow();
        //PixelShader = 0;
    }
}

// ================================================================================================
static const float Scale = 0.03;
static const float Bias = -0.02;

// ================================================================================================
float mad(float f1, float f2, float f3)
{
    return f1 * f2 + f3;
}
 
// ================================================================================================
float2 mad2(float f1, float2 f2, float2 f3)
{
    return f1 * f2 + f3;
}

// ================================================================================================
PS_Out PS_TSDLightingParallax(VS_OutTSD i)
{
    float fHeight = tex2D(Sampler1, i.Tex01.xy).r;
    fHeight   = fHeight * Scale + Bias;
    i.Tex01.xy = -i.TSViewDir.xy * fHeight + i.Tex01.xy;
    
//    //i.TSViewDir = normalize(i.TSViewDir);
//    float2 fHeight = tex2D(Sampler1, i.Tex);        // Hhenpixel aus der Textur auslesen und mit den Faktoren verschieben
//    fHeight = mad(fHeight, Scale, Bias);
//    i.Tex = mad2(fHeight, -i.TSViewDir.xy, i.Tex);  // Neue Texturkoordinaten mit einer Verschiebung berechnen
//    //i.Tex = mad2(fHeight * g_fNBTValue, -i.TSViewDir.xy, i.Tex);  // Neue Texturkoordinaten mit einer Verschiebung berechnen
//    
//    return PS_TSDLighting(i);
    return PS_TSDLighting_3Tex(i, false, true, true);
}

// ================================================================================================
technique StandardNBTParallax
<
	string Description = "Shader with parallax mapping";
	string NBTMethod = "ATI";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_NBT();
        PixelShader = compile   SHAD_PS_MINVER_2_0  PS_TSDLightingParallax();
    }
}

// ================================================================================================
PS_Out PS_TSDLightingParallax_Special(VS_OutTSD i)
{
    float fHeight = tex2D(Sampler1, i.Tex01.xy).r;
    fHeight       = fHeight * Scale + Bias;
    i.Tex01.xy    = -i.TSViewDir.xy * fHeight + i.Tex01.xy;
    
    return PS_TSDLighting_3Tex(i, true, false, true);
}

// ================================================================================================
technique StandardNBTParallax_Special
<
	string Description = "Shader with parallax mapping";
	string NBTMethod = "ATI";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_NBT();
        PixelShader = compile   SHAD_PS_MINVER_3_0  PS_TSDLightingParallax_Special();
    }
}

// ================================================================================================
technique StandardNBTParallaxLoD
<
	string Description = "Shader with(out) parallax mapping (LoD-version)";
	string NBTMethod = "ATI";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	        
        VertexShader = compile  SHAD_VS_MINVER_1_1  VS_NBTLoD();
        PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_VertexLighting_3Tex();
    }
}

// ================================================================================================
technique Standard_Special
<
	string Description = "Standard shader for unskinned objects";
	bool UsesNIRenderState = true;
	string NBTMethod = "none";
>
{
    pass p0
    { 
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	    
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_VertexLighting(true);
        PixelShader  = compile SHAD_PS_MINVER_2_0 PS_VertexLighting_3Tex(true);
    }
}

technique StandardUnderWater_Special
<
	string Description = "Standard shader for unskinned objects under water";
	bool UsesNIRenderState = true;
	string NBTMethod = "none";
>
{
    pass p0
    { 
        FillMode        = <GROUND_FILL_MODE>;
	    Lighting        = false;
	    SpecularEnable  = false;
	    
        VertexShader = compile SHAD_VS_MINVER_1_1 VS_VertexLighting();
        PixelShader  = compile SHAD_PS_MINVER_2_0 PS_VertexLighting_3Tex(true, true);
    }
}