// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2008 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: ank
// date: 02.04.2008

#include "GlobalFX.h"
#include "shaderfragments.hlsl"
#include "shadowmap.hlsl"

// ================================================================================================
const float3    SUN_LIGHT_DIRECTION_WS : GLOBAL;
const float4    SUN_LIGHT_AMBIENT  : GLOBAL;
const float4    SUN_LIGHT_DIFFUSE  : GLOBAL;
const float4    SUN_LIGHT_SPECULAR : GLOBAL;

const float4    EW_WATER_COLOR : GLOBAL;

const float3    CAMERA_POS_WS : GLOBAL;
const DWORD     EW_CULL_MODE: GLOBAL = 2;
const DWORD     ENVIRONMENT_WATER_FILL_MODE: GLOBAL;

const float4x4 matBump                  : GLOBAL;
const float4x4 matWorldViewWater        : GLOBAL;       // view of water camera...
const float WaterAlpha                  : GLOBAL = 0.5;
const float WaterPower                  : GLOBAL = 256;
const float WaterGloss                  : GLOBAL = 1;
const float WaterPowerVI                : GLOBAL = 2048;
const float WaterGlossVI                : GLOBAL = 0.25;

texture BumpMap   <int NTMIndex = 0;>;
texture BumpMap2  <int NTMIndex = 1;>;
texture NormalMap <int NTMIndex = 1;>;
texture NormalMap2<int NTMIndex = 2;>;

// ================================================================================================
sampler2D ReflectionSampler = sampler_state 
{
	Texture = (BaseMap);
	AddressU = Clamp; //Wrap;
	AddressV = Clamp; //Wrap;
  	MagFilter = <MAG_FILTERTYPE>;
  	MinFilter = <MIN_FILTERTYPE>;
  	MipFilter = <MIP_FILTERTYPE>;
  	MaxAnisotropy = <MAX_ANISOTROPY>;
};

// ================================================================================================
sampler2D BumpSampler = sampler_state 
{
	Texture = (BumpMap);
	AddressU = Wrap;
	AddressV = Wrap;
  	MagFilter = <MAG_FILTERTYPE>;
  	MinFilter = <MIN_FILTERTYPE>;
  	MipFilter = <MIP_FILTERTYPE>;
  	MaxAnisotropy = <MAX_ANISOTROPY>;
};

// ================================================================================================
sampler2D BumpSampler2 = sampler_state 
{
	Texture = (BumpMap2);
	AddressU = Wrap;
	AddressV = Wrap;
  	MagFilter = <MAG_FILTERTYPE>;
  	MinFilter = <MIN_FILTERTYPE>;
  	MipFilter = <MIP_FILTERTYPE>;
  	MaxAnisotropy = <MAX_ANISOTROPY>;
};

// ================================================================================================
sampler2D NormalSampler = sampler_state 
{
	Texture = (NormalMap);
	AddressU = Wrap;
	AddressV = Wrap;
  	MagFilter = <MAG_FILTERTYPE>;
  	MinFilter = <MIN_FILTERTYPE>;
  	MipFilter = <MIP_FILTERTYPE>;
  	MaxAnisotropy = <MAX_ANISOTROPY>;
};

// ================================================================================================
sampler2D NormalSampler2 = sampler_state 
{
	Texture = (NormalMap2);
	AddressU = Wrap;
	AddressV = Wrap;
  	MagFilter = <MAG_FILTERTYPE>;
  	MinFilter = <MIN_FILTERTYPE>;
  	MipFilter = <MIP_FILTERTYPE>;
  	MaxAnisotropy = <MAX_ANISOTROPY>;
};

// ================================================================================================
struct VS_OutReflect
{
    float4 Pos            : POSITION;
    float3 CameraSpacePos : TEXCOORD0;
    float2 Tex            : TEXCOORD1;
    float3 OutPos         : TEXCOORD2;
    float2 TexModified    : TEXCOORD3;
};

// ================================================================================================
VS_OutReflect VS_Reflect(float4 Pos : POSITION, float2 Tex : TEXCOORD0)
{
    VS_OutReflect o; 
    o.Pos = mul(Pos, matWorldViewProjection);               // Vertexposition im Bildraum
    o.CameraSpacePos = mul(Pos, (float4x3)matWorldView);    // Vertexposition im Kameraraum
    
    // Texturkoordinaten im Kameraraum (entspricht Matrixmultiplikation mit Matrix, wo _31=0.5 und _32=-0.5)
    float f = o.CameraSpacePos.z * 0.5;
    o.CameraSpacePos.x += f;
    o.CameraSpacePos.y -= f;
    
    o.Tex = Tex;
    o.TexModified = Tex;
    
    o.OutPos = mul(Pos, matWorld);

    return o;
}

// ================================================================================================
float4 PS_Reflect(VS_OutReflect i) : COLOR
{
    float4 colBump = tex2D(BumpSampler, i.Tex * 0.02) * 2 - 1;
    
    float2 vD;
    vD.x = colBump.x*matBump._11 + colBump.y*matBump._21;
    vD.y = colBump.x*matBump._12 + colBump.y*matBump._22;
      
    float2 texCamSpacePos = i.CameraSpacePos.xy / i.CameraSpacePos.z;  // projizierte Texturkoordinaten
    
    float3 bump;
    bump.x = clamp(texCamSpacePos.x + vD.x, 0, 1);
    bump.y = clamp(texCamSpacePos.y + vD.y, -1, 0);
    bump.z = 1;

    float4 colRefl = tex2D(ReflectionSampler, bump.xy);
    
    // Lighting
    float3 vNegViewDir = normalize(CAMERA_POS_WS - i.OutPos);
    
    i.TexModified *= 0.25;
    i.TexModified += vD;
        
    float3 colNormal = tex2D(NormalSampler, i.TexModified) * 2 - 1; // "decompress" normal

    // Calculate diffuse
    float fAngle = dot(colNormal, vNegViewDir);
    float4 colDif = SUN_LIGHT_DIFFUSE * max(0, fAngle);	
            
    // Calculate specular
	float3 vReflection = normalize(reflect(SUN_LIGHT_DIRECTION_WS.xyz, colNormal));
	float4 colSpec = pow(max(0, dot(vReflection, vNegViewDir)), WaterPower) * SUN_LIGHT_SPECULAR * WaterGloss;
	
	// Calculate "softspecular" (viewindependent)
	float3 vSoftSpecReflection = reflect(vNegViewDir.xyz, float3(0,0,1));
	vSoftSpecReflection = normalize(reflect(vSoftSpecReflection.xyz, colNormal));
	float4 colSoftSpec = pow(max(0, dot(vSoftSpecReflection, vNegViewDir)), WaterPowerVI) * SUN_LIGHT_SPECULAR * WaterGlossVI;
	colSoftSpec.a *= 0.5;

	float4 col = EW_WATER_COLOR * (SUN_LIGHT_AMBIENT + colDif);
	col *= colRefl;
	col.a = WaterAlpha;
	
    return col + colSoftSpec + colSpec;
}


// ================================================================================================
// anderer Wassershader by mad:
// ================================================================================================
struct VS_InWater
{
    float4 Pos : POSITION;
    float2 Tex : TEXCOORD0;
};

// ================================================================================================
struct VS_OutWater
{
	float4 Pos			: POSITION;
    float4 TexCoord     : TEXCOORD0;    // org. Texturkoordinaten
    float4 PosWVS       : TEXCOORD1;    // proj.  Position (WorldViewSpace)
    float3 CamDirTS     : TEXCOORD2;    // Vtx -> Cam   - Direction
    float3 LightDirTS   : TEXCOORD3;    // Vtx -> Light - Direction
    float  UWaterAFactor: TEXCOORD4;    // AlphaFactor for Water (underwater)...
    float4 LightVPS     : TEXCOORD5;    // Pos in LightViewProjSpace
#if defined(USE_DEPTH_MAP_MRT)
	float  Depth		: TEXCOORD6;
#endif
};

// ================================================================================================
// Konstanten und Variablen, die extern gesetzt werden knnen:
//
// SHADER_VERSION2 einkommentieren, wenn der neue Shader verwendet werden soll
//
// USE_REFRACTION einkommentieren, wenn Refraction-Textur vorhanden und Effekt gewnscht
// -> alternativ das Flag auch extern einfgen
//
// USE_EXTRA_SHADING bewirkt, dass zustzlich zur Reflektion/Refraction noch
// eine Beleuchtungsberechnung auf das Wasser ausgefhrt wird, wodurch
// nochmal Highlights etc hinzugefgt werden
//
// CAST_SHADOW bewirkt, das auch ein schatten auf die wasseroberflche fllt
// ================================================================================================
#define SHADER_VERSION2
#define USE_EXTRA_SHADING

#if PIXELSHADER_VERSION >= 30 && !defined(MIN_SHADER_VERSION)
#   define CAST_SHADOW
#   define USE_REFRACTION
#endif

const float4x3 matInvWorld           : INVWORLD;
const float3 WIND_AND_TIME           : GLOBAL;

const float  g_NormTLyr0Scale        : GLOBAL = 2.3;                // Texturscaling for Normalmap
const float2 g_NormTLyr0DirNSpeed    : GLOBAL = float2(0.010, 0.04);  // TexCoord-Movement per Second
const float  g_NormLyr0Factor        : GLOBAL = 0.74;               // Contributionfactor for final normal

const float  g_NormTLyr1Scale        : GLOBAL = 4.3;
const float2 g_NormTLyr1DirNSpeed    : GLOBAL = float2(0.021, 0.09);
const float  g_NormLyr1Factor        : GLOBAL = 0.5;

const float  g_ReflScale             : GLOBAL = 0.075;  // wie stark Normalen die Reflektion verzerren
const float  g_RefractScale          : GLOBAL = 0.03;   // wie stark Normalen die Refraction verzerren,
                                                        // sollte nie kleiner als Reflektion sein!
const float  g_NormStrength          : GLOBAL = 2;

// Fresnel Factoren:
// n0(air) = 1.0; n1(water) = 1.3333
// Formel: R(alpha) = R(0) + (1-R(0)) * (1-cos(alpha))^5
// wobei: R(0) = (n0-n1)^2 / (n0+n1)^2
//const float r0 = 0.02037;
//const float r1 = 0.97963;
const float r0 : GLOBAL = 0.12037;
const float r1 : GLOBAL = 0.87963;

// ================================================================================================
#if defined(USE_REFRACTION)
texture RefractionMap<int NTMIndex = 3;>;
sampler2D RefractionSampler = sampler_state 
{
	Texture = (RefractionMap);
	AddressU = Clamp;
	AddressV = Clamp; //Wrap;
  	MagFilter = <MAG_FILTERTYPE>;
  	MinFilter = <MIN_FILTERTYPE>;
  	MipFilter = <MIP_FILTERTYPE>;
  	MaxAnisotropy = <MAX_ANISOTROPY>;
};
#endif
                                                        
// ================================================================================================
VS_OutWater VS_WaterRefl(VS_InWater i)
{
    VS_OutWater o;
    
    o.Pos = mul(i.Pos, matWorldViewProjection);

#if defined(USE_DEPTH_MAP_MRT)
	o.Depth = o.Pos.z / o.Pos.w;
#endif

    o.PosWVS = mul(i.Pos, matWorldViewWater);   // Vertexposition im Kameraraum
    
    // some ajustments for using as texture coords in PS (without calculation all there!)
    o.PosWVS.xy *= 0.5;                         
    float f = o.PosWVS.z * 0.5f;    // to save some instructions in PS, multiply here with add/subtract 0.5f*z
    o.PosWVS.xy += f;               // to x and y -> in PS we divide by z, leaving +0.5/-0.5 left (e.g. conversion
                                    // from projection space [-1,1] to texture space [0,1]
    o.PosWVS.y = o.PosWVS.z - o.PosWVS.y;   // inversion of y is removed from PS to VS, cause we divide by z, its not 1-y but z-y ;)
    
    o.PosWVS.w = (CAMERA_POS_WS.z < i.Pos.z)? -1.0 : 1.0;
    
    o.TexCoord.xy = (i.Tex + g_NormTLyr0DirNSpeed.xy*WIND_AND_TIME.z)*g_NormTLyr0Scale.x;
    o.TexCoord.zw = (i.Tex + g_NormTLyr1DirNSpeed.xy*WIND_AND_TIME.z)*g_NormTLyr1Scale.x;
    
    float3 CamDirVecOS = mul(CAMERA_POS_WS.xyz, matInvWorld);   // transform cam-pos in objects space
    CamDirVecOS -= i.Pos.xyz;                                   // and get direction from current pos
	o.CamDirTS = CamDirVecOS;
	
	o.LightDirTS = mul(SUN_LIGHT_DIRECTION_WS.xyz, matInvWorld);// LightDirection from world to object space
	// water is just a plane in xy (from definition?!)
	// so OS == TS for vectors ... otherwise use this code:
	// float tmp = o.LightDirTS.y;      // swap y and z component
	// o.LightDirTS.y = o.LightDirTS.z;
	// o.LightDirTS.z = tmp;
	
	o.UWaterAFactor = (o.PosWVS.w >= 0) ? (1-WaterAlpha) : (0.5 * (1-WaterAlpha));
	
    // this is bullshit, cause we only have 4 vertices @ worldedge. so this will
    // give us no useable information
    //o.LightVPS = PerformTransformInLightSpace(i.Pos);
    o.LightVPS = i.Pos;
    return o;
}

// ================================================================================================
float2 GetNormalTexCoords(float2 inTexCoords, 
                          float Scaling, 
                          float2 waterMovement, 
                          float time)
{
    return inTexCoords*Scaling + waterMovement * Scaling * time;
}

// ================================================================================================
float2 CalcProjTexCoords(const float4 Pos)
{
	float3 vCamSpacePos = mul(Pos, (float4x3)matWorldView);
	
	// Texturkoordinaten im Kameraraum (...)
	float f = vCamSpacePos.z * 0.5;
    vCamSpacePos.x += f;
    vCamSpacePos.y -= f;
    
    vCamSpacePos.xy = vCamSpacePos.xy / vCamSpacePos.z;
    
    return vCamSpacePos.xy;
}

// ================================================================================================
float4 PS_WaterRefl(VS_OutWater i) : COLOR
{
    // zuerst einmal die Texturkoordinaten bestimmen fr unsere Wellen (Normalentextur)
    // hier erstmal nur 2 Layer (Mglicherweise besser mit 4 oder mehr Layern)
    // float2 Norm0TexCoord = GetNormalTexCoords(i.TexCoord, g_NormTLyr0Scale, g_NormTLyr0DirNSpeed, WIND_AND_TIME.z);
    // float2 Norm1TexCoord = GetNormalTexCoords(i.TexCoord, g_NormTLyr1Scale, g_NormTLyr1DirNSpeed, WIND_AND_TIME.z);
    float2 Norm0TexCoord = i.TexCoord.xy;
    float2 Norm1TexCoord = i.TexCoord.zw;
    
    // Normalendaten auslesen anhand der Texturkoordinaten
    float3 nrm0 = tex2D(NormalSampler, Norm0TexCoord);
    float3 nrm1 = tex2D(NormalSampler2, Norm1TexCoord);
    
    // entgltige Normale bestimmen ber mixen der normalen
    float3 normal = nrm0 * g_NormLyr0Factor;
    normal = nrm1 * g_NormLyr1Factor + normal;    // so geschrieben, damit compiler daraus eine 'mad' operation macht 
    
    // anschliessend die Farbinfos noch in Vektorform bringen & normalisieren:
    normal += -0.5*(g_NormLyr0Factor + g_NormLyr1Factor);
    normal.xy = normal.xy * g_NormStrength;
    // PosWVS == -1 wenn unterwasser
    normal = i.PosWVS.w * normalize(normal);
    
    // Projezierte Texturkoordinaten bestimmen
    float2 ProjTexCoord = i.PosWVS.xy/i.PosWVS.z;
    // float2 ProjTexCoord = CalcProjTexCoords(i.PosWVS);
    
    // anschliessend sowohl fr Reflektion als auch Refraction anpassen
    float2 ReflTexCoord = ProjTexCoord + normal.xy * g_ReflScale;
    
    float4 ReflColor = tex2D(ReflectionSampler, ReflTexCoord);
    //return ReflColor;
    
#if defined(USE_REFRACTION)    
    // Refraction nur, wenn wirklich gewnscht:
    float2 RefractTexCoord; // = ProjTexCoord; -> wrong
    
    //RefractTexCoord = ProjTexCoord + normal.xy * min(g_RefractScale, g_ReflScale);
    float4 WorldView  = mul(i.LightVPS, matWorldViewProjection);
    RefractTexCoord   = WorldView.xy/WorldView.z;
    RefractTexCoord   = RefractTexCoord * 0.5f + 0.5f;
    RefractTexCoord.y = 1 - RefractTexCoord.y;
    
    // make refract independent from reflectscale
    //RefractTexCoord += normal.xy * min(g_RefractScale, g_ReflScale);
    RefractTexCoord -= normal.xy * g_RefractScale;
    RefractTexCoord = clamp(RefractTexCoord, 0, 1);
    
    float4 RefractColor = tex2D(RefractionSampler, RefractTexCoord);
    //return RefractColor;
#endif
    
    // jetzt die Reflektions bzw Refraktionsfaktoren nach Fresnel berechnen:
    i.CamDirTS = normalize(i.CamDirTS);
    float cosAlpha = saturate(dot( normal, i.CamDirTS));
    
    // Formel: R(alpha) = R(0) + (1-R(0)) * (1-cos(alpha))^5
    float invCosAlpha = 1 - cosAlpha;
    invCosAlpha = pow(invCosAlpha, 5);
    float r_alpha = r0 + r1*invCosAlpha;   // Reflections-koeffizient
    
    float4 BaseColor;
    // die Mischfarbe anhand des Reflektionsfaktors bilden,
    // theoretisch kann man den Faktor noch abschwchen mit dem Alpha-Wert aus der Refraktion,
    // der sich anhand der Wassertiefe ergeben wrde!
#if defined(USE_REFRACTION)
    BaseColor.xyz = lerp(EW_WATER_COLOR * RefractColor, ReflColor, r_alpha);
#else    
    BaseColor.xyz = lerp(EW_WATER_COLOR, ReflColor, r_alpha);
#endif
    
#if defined(USE_REFRACTION)
    // Unterwasser-Infos bekommen wir von Textur,
    // deshalb kein alpha
    BaseColor.a = 1;
#else
    // ohne wird der Reflektionskoeffizient als Transparenz genutzt,
    // wodurch stark-reflektierende Bereiche undurchsichtiger sind
    //BaseColor.a = (1-WaterAlpha)*r_alpha + WaterAlpha;
    BaseColor.a = i.UWaterAFactor*r_alpha + WaterAlpha;
#endif
    
    //return float4(BaseColor.a, BaseColor.a, BaseColor.a, 1);
    
#if defined(USE_EXTRA_SHADING)    

#   if defined(CAST_SHADOW)
    // zu wenige instructions mit SM2 dafr, deshalb nur bei SM3+ aktivieren:
    // by mad: um schatten erweitert
    float4 vPosition    = PerformTransformInLightSpace(i.LightVPS);
    vPosition.xy += normal.xy * 0.5f * g_ReflScale;    // modify position based on reflection (waves)
    float fCurrentDepth = GetDepthInLightSpace(vPosition);
    float fShadow       = GetObjectInShadowFactor(fCurrentDepth, vPosition, ShadowMap);
    //float fCurrentDepth = GetDepthInLightSpace(i.LightVPS);
    //float fShadow = GetObjectInShadowFactor(fCurrentDepth, i.LightVPS, ShadowMap);
#   else  // !CAST_SHADOW
    const float fShadow = 1.f;
#   endif // CAST_SHADOW

    // zuletzt noch Diffuse- und Specular-Farben bestimmen
    // um noch eine Form von Shading zu beinhalten
    i.LightDirTS = -normalize(i.LightDirTS);
    
    float  dotLight    = dot(normal, i.LightDirTS);
    float4 colDiffuse  = fShadow * SUN_LIGHT_DIFFUSE  * max(0, dotLight) + SUN_LIGHT_AMBIENT;

    float3 floatVec    = normalize(i.CamDirTS + i.LightDirTS);
    float  facSpec     = max(dot(floatVec, normal), 0);
    facSpec            = WaterGloss * pow(facSpec, WaterPower);
    // facSpec            = pow(facSpec, WaterPower);
    // float4 colSpecular = WaterGloss * SUN_LIGHT_SPECULAR * facSpec;
    
    floatVec             = normalize(i.CamDirTS + float3(0,0,10/g_NormStrength));   // 1/10 die View-Direction reinnehmen
    float  facSpecVI     = max(dot(floatVec, normal), 0);
    facSpecVI            = WaterGlossVI * pow(facSpecVI, WaterPowerVI);
    // facSpecVI            = pow(facSpecVI, WaterPowerVI);
    // float4 colSpecularVI = WaterGlossVI * SUN_LIGHT_SPECULAR  * facSpecVI;
    
    float specMax = max(fShadow * facSpec, facSpecVI);
    
    // BaseColor.xyz = BaseColor.xyz * colDiffuse + fShadow * max(colSpecular, colSpecularVI);
    // BaseColor.xyz = BaseColor.xyz * colDiffuse + max(colSpecular, colSpecularVI);
    BaseColor.xyz = BaseColor.xyz * colDiffuse + specMax * SUN_LIGHT_SPECULAR; //max(colSpecular, colSpecularVI);

//  #if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
    // BaseColor.a   = max(BaseColor.a , max(WaterGlossVI*facSpecVI,WaterGloss*facSpec));
    BaseColor.a   = max(BaseColor.a , specMax);
//  #endif    
#endif
    
    return BaseColor;
}

// ================================================================================================
VS_OutWater VS_WaterSimple(VS_InWater i)
{
    VS_OutWater o;
    
    o.Pos = mul(i.Pos, matWorldViewProjection);

#if defined(USE_DEPTH_MAP_MRT)
	o.Depth = o.Pos.z / o.Pos.w;
#endif
    
	o.PosWVS = mul(i.Pos, matWorldView);
    o.PosWVS.w = (CAMERA_POS_WS.z < i.Pos.z)? -1.0 : 1.0;
    
    
    o.TexCoord.xy = (i.Tex + g_NormTLyr0DirNSpeed.xy*WIND_AND_TIME.z)*g_NormTLyr0Scale.x;
    o.TexCoord.zw = (i.Tex + g_NormTLyr1DirNSpeed.xy*WIND_AND_TIME.z)*g_NormTLyr1Scale.x;
    
    float3 CamDirVecOS = mul (CAMERA_POS_WS.xyz, matInvWorld);	// transform cam-pos in objects space
    CamDirVecOS -= i.Pos.xyz;							        // and get direction from current pos
	o.CamDirTS = CamDirVecOS;
	
	o.LightDirTS = mul(SUN_LIGHT_DIRECTION_WS.xyz, matInvWorld);// LightDirection from world to object space
	o.UWaterAFactor = (o.PosWVS.w >= 0) ? (1-WaterAlpha) : (0.5 * (1-WaterAlpha));
	o.LightVPS = PerformTransformInLightSpace(i.Pos);
    
    return o;
}

// ================================================================================================
PS_Out PS_WaterSimple(VS_OutWater i)
{
	PS_Out output = (PS_Out)0;

    // zuerst einmal die Texturkoordinaten bestimmen fr unsere Wellen (Normalentextur)
    // hier erstmal nur 2 Layer (Mglicherweise besser mit 4 oder mehr Layern)
    // float2 Norm0TexCoord = GetNormalTexCoords(i.TexCoord, g_NormTLyr0Scale, g_NormTLyr0DirNSpeed, WIND_AND_TIME.z);
    // float2 Norm1TexCoord = GetNormalTexCoords(i.TexCoord, g_NormTLyr1Scale, g_NormTLyr1DirNSpeed, WIND_AND_TIME.z);
    float2 Norm0TexCoord = i.TexCoord.xy;
    float2 Norm1TexCoord = i.TexCoord.zw;

    // Normalendaten auslesen anhand der Texturkoordinaten
    float3 nrm0 = tex2D(NormalSampler, Norm0TexCoord);
    float3 nrm1 = tex2D(NormalSampler2, Norm1TexCoord);
    
    // entgltige Normale bestimmen ber mixen der normalen
    float3 normal = nrm0 * g_NormLyr0Factor;
    normal = nrm1 * g_NormLyr1Factor + normal;    // so geschrieben, damit compiler daraus eine 'mad' operation macht 
    
    // anschliessend die Farbinfos noch in Vektorform bringen & normalisieren:
    normal += -0.5*(g_NormLyr0Factor + g_NormLyr1Factor);
    normal.xy = normal.xy * g_NormStrength;
    // PosWVS == -1 wenn unterwasser
    normal = i.PosWVS.w * normalize(normal);
    
    float4 ReflColor = EW_WATER_COLOR;
    //ReflColor.xyz += float3(1.25,0.25,0.25);
    ReflColor.xyz += float3(0.25,0.25,0.25);
    ReflColor = saturate(ReflColor);
    
    // jetzt die Reflektions bzw Refraktionsfaktoren nach Fresnel berechnen:
    i.CamDirTS = normalize(i.CamDirTS);
    float cosAlpha = saturate(dot( normal, i.CamDirTS));
    
    // Formel: R(alpha) = R(0) + (1-R(0)) * (1-cos(alpha))^5
    float invCosAlpha = 1 - cosAlpha;
    invCosAlpha = pow(invCosAlpha, 5);
    float r_alpha = r0 + r1*invCosAlpha;   // Reflections-koeffizient
    
    float4 BaseColor;
    // die Mischfarbe anhand des Reflektionsfaktors bilden,
    // theoretisch kann man den Faktor noch abschwchen mit dem Alpha-Wert aus der Refraktion,
    // der sich anhand der Wassertiefe ergeben wrde!
    BaseColor.xyz = lerp(EW_WATER_COLOR, ReflColor, r_alpha);
    
    // ohne wird der Reflektionskoeffizient als Transparenz genutzt,
    // wodurch stark-reflektierende Bereiche undurchsichtiger sind
    BaseColor.a = i.UWaterAFactor*r_alpha + WaterAlpha;
    
#if defined(USE_EXTRA_SHADING)    
    
    // zuletzt noch Diffuse- und Specular-Farben bestimmen
    // um noch eine Form von Shading zu beinhalten
    i.LightDirTS = -normalize(i.LightDirTS);
    
    float  dotLight    = dot(normal, i.LightDirTS);
    float4 colDiffuse  = SUN_LIGHT_DIFFUSE  * max(0, dotLight) + SUN_LIGHT_AMBIENT;
    
    float3 floatVec     = normalize(i.CamDirTS + i.LightDirTS);
    float  facSpec     = max(dot(floatVec, normal), 0);
    facSpec           = pow(facSpec, WaterPower);
    float4 colSpecular = WaterGloss * SUN_LIGHT_SPECULAR * facSpec;
    
    floatVec = normalize(i.CamDirTS + float3(0,0,10/g_NormStrength));   // 1/10 die View-Direction reinnehmen
    float  facSpecVI     = max(dot(floatVec, normal), 0);
    facSpecVI           = pow(facSpecVI, WaterPowerVI);
    float4 colSpecularVI = WaterGlossVI * SUN_LIGHT_SPECULAR * facSpecVI;
    
    BaseColor.xyz = BaseColor.xyz * colDiffuse + max(colSpecular, colSpecularVI);
//  #if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)  
    BaseColor.a   = max(BaseColor.a , max(WaterGlossVI*facSpecVI,WaterGloss*facSpec));
//  #endif    
#endif

	output.Color = BaseColor;

#if defined(USE_DEPTH_MAP_MRT)
	output.Depth.x = 1.0f - i.Depth;		// Projected Depth
	output.Depth.y = i.PosWVS.z;			// Linear VS Depth
	output.Depth.a = 1.0f;					// Don't blend the depth values
#endif

    return output;
}

// ================================================================================================
/*technique Reflection
<
    bool UsesNIRenderState = true;
>
{
    pass p0
    {
        CullMode = <EW_CULL_MODE>;
        ZWriteEnable = false;
        FillMode = solid;

        VertexShader = compile SHAD_VS_MINVER_1_1 VS_Reflect();
        PixelShader  = compile SHAD_PS_MINVER_2_0 PS_Reflect();
    }
}*/


// ================================================================================================
technique Reflection
<
    bool UsesNIRenderState = true;
>
{
    pass p0
    {
        CullMode = <EW_CULL_MODE>;
        ZWriteEnable = false;
        FillMode = solid;

        VertexShader = compile SHAD_VS_MINVER_1_1 VS_WaterSimple();
        PixelShader  = compile SHAD_PS_MINVER_2_0 PS_WaterSimple();
    }
}

// ================================================================================================
technique WaterReflection
< 
    bool UsesNIRenderState = true; 
>
{
    pass water0
    {
        CullMode = <EW_CULL_MODE>;
        ZWriteEnable = false;
        FillMode = solid;
        
        VertexShader = compile SHAD_VS_MINVER_2_0 VS_WaterRefl();
        PixelShader  = compile SHAD_PS_MINVER_2_0 PS_WaterRefl();
    }
}
