// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2011 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: mad
// date: 01.03.2011

// ================================================================================================
#include "Skinning.h"

// ================================================================================================
const float3    SelectionColor      : GLOBAL = float3(1,1,0);
const bool      ColorSolid          : GLOBAL = false;

// ================================================================================================
static const float glowScale    = 2.5;
static const float glowConstant = 0.0;
static const float glowExponent = 2.0;

static const bool  addSecondGlow = true;
static const float glowScale2    = 5.0;
static const float glowConstant2 = -0.7;
static const float glowExponent2 = 10.0;
static const bool  showStreak    = true;

// ================================================================================================
#define HighlightStates\
    FillMode            = solid;\
    ShadeMode           = Gouraud;\
	CullMode            = cw;\
	ZEnable             = true;\
	ZWriteEnable        = false;\
	AlphaTestEnable     = true;\
	AlphaBlendEnable    = true;\
	SrcBlend            = SrcAlpha;\
    DestBlend           = InvSrcAlpha;\
    Lighting            = false;\
	SpecularEnable      = false;\
	FogEnable           = false;\
	AlphaRef            = 10;
    
// ================================================================================================
struct VS_OutHighlight
{    
    float4 Pos          : POSITION;
    float4 Tex01        : TEXCOORD0;    // texture set 0 & 1
    float3 TSLightDir   : TEXCOORD2;    // Texture-/TangentSpace light direction
    float3 TSViewDir    : TEXCOORD3;
    float2 ScreenPos    : TEXCOORD4;
};

// ================================================================================================
// VERTEXSHADER
// ================================================================================================
VS_OutHighlight VS_Highlight(/*uniform int NumBones, VS_InSkinnedNBT i*/ VS_InSkinning_4Tex i)
{
    VS_OutHighlight o = (VS_OutHighlight)0;
    //float3x3 matTangent;
    float3   position;
    
	/*
    if (NumBones > 0)
    {
        VS_OutSkinnedNBT os = SkinningNBT(i, NumBones);
        position = os.Pos;
        o.Pos = mul(float4(position, 1), matViewProj);
        o.ScreenPos = o.Pos.xy / o.Pos.w;
        
        matTangent[0] = os.Tangent;
        matTangent[1] = os.Binormal;
        matTangent[2] = os.Normal;
    }
    else
    { 
	*/
        position = mul(i.Pos, matWorld);

		// Wind animation
		float2 direction = 1.0f - i.Color.xy;
		position.xy += ComputeWindAnimationDisplacement(position.xyz, direction);

        o.Pos = mul(float4(position, 1), matViewProj);
        o.ScreenPos = o.Pos.xy / o.Pos.w;
        
		/*
        matTangent[0] = normalize(mul(i.Tangent, (float3x3)matWorld));
        matTangent[1] = normalize(mul(i.Binormal, (float3x3)matWorld));
        matTangent[2] = normalize(mul(i.Normal, (float3x3)matWorld));
    }
	*/
    
    //o.Tex01.xy = i.Tex0.xy;
	o.Tex01.xy = i.Tex.xy;
    if (numLayer > 1)
        o.Tex01.zw = i.Tex1.xy;
    else
        o.Tex01.zw = o.Tex01.xy;
    
    float3 ViewDir = normalize(position - CamPos); // world space view direction
    //o.TSLightDir = mul(matTangent, LightDir);      // world space light dir to tangent space light dir
    //o.TSViewDir  = mul(matTangent, ViewDir);
	o.TSViewDir = ViewDir;
    
    return o;
}
// ================================================================================================
VS_OutHighlight VS_HighlightSkinned(uniform int NumBones, VS_InSkinnedNBT i)
{
    VS_OutHighlight o = (VS_OutHighlight)0;
    float3x3 matTangent;
    float3   position;
    
    VS_OutSkinnedNBT os = SkinningNBT(i, NumBones);
    position = os.Pos;
    o.Pos   = mul(float4(position, 1), matViewProj);
    o.ScreenPos = o.Pos.xy / o.Pos.w;
    
    o.Tex01.xy = i.Tex0.xy;
    if (numLayer > 1)
        o.Tex01.zw = i.Tex1.xy;
    else
        o.Tex01.zw = o.Tex01.xy;
    
    matTangent[0] = os.Tangent;
    matTangent[1] = os.Binormal;
    matTangent[2] = os.Normal;
    
    float3 ViewDir = normalize(position - CamPos); // world space view direction
    //o.TSLightDir = mul(matTangent, LightDir);      // world space light dir to tangent space light dir
    o.TSViewDir  = mul(matTangent, ViewDir);
    
    return o;
}

// ================================================================================================
const float fStreakStrenght = 0.85;
const float fStreakInvThickness = 50;
const float fStreakSpeed = 0.35f;
const bool  bStreakLeftAndRight = true;

// ================================================================================================
// PIXELSHADER
// ================================================================================================
float4 PS_Highlight(VS_OutHighlight i) : COLOR
{
    float4 result = float4(0,0,0,1);
    float  fAlpha = tex2D(SamplerBase, i.Tex01.xy).a;

    if (ColorSolid)
    {
        float outAlpha = (fAlpha > 0.2) ? 1 : 0;
        result = float4(SelectionColor, outAlpha);
    }
    else
    {
		// by jow entfernt: produziert Shader-Fehler und wird nicht benutzt
        //float3 Normal = tex2D(Sampler0, i.Tex01.xy) * 2 - 1; // "decompress" normal
		float3 Normal = float3(1.0f, 0.0f, 0.0f);

        float3 fvView = normalize(i.TSViewDir);
        float  NDotL  = saturate(dot(normalize(Normal), -fvView));
    
        // older version:
        //float glowStrength = saturate( pow(1 - NDotL, 2)) * 0.7;
    
        // new version with additional params...
        float glowStrength = saturate( pow(1 - NDotL, glowExponent)) * glowScale + glowConstant;
        if (addSecondGlow)
        {
            glowStrength += saturate( pow(1 - NDotL, glowExponent2)) * glowScale2 + glowConstant2;
        }
        // limit to max 0 alpha
        glowStrength = min(fAlpha, max(glowStrength, 0));
    
        // --------------------
        // some special stuff
        /*
        if (showStreak)
        {
            float2 fNormalizedScreenPos = i.ScreenPos*0.5 + 0.5;
        
            float fTimeMod = fmod(Time*fStreakSpeed, 1.0f);
            if (bStreakLeftAndRight)
                fTimeMod = abs((fTimeMod - 0.5) * 2);
        
            float2 fSHighLight = 1 - (abs(fNormalizedScreenPos - float2(fTimeMod, fTimeMod)) * fStreakInvThickness);
            //glowStrength = max(glowStrength, max(fSHighLight.x, fSHighLight.y) * fStreakStrenght);
            glowStrength = max(glowStrength, fSHighLight.x * fStreakStrenght);
        }
        */
        // --------------------
    
        result = float4(SelectionColor, glowStrength);
    }

    return result;
}

// ================================================================================================
technique Highlight
<
    string NBTMethod = "ATI";
    bool UsesNIRenderState = true;
>
{
    pass p0
	{
		HighlightStates
		
		//VertexShader = compile SHAD_VS_MINVER_1_1  VS_Highlight(0);
		VertexShader = compile SHAD_VS_MINVER_1_1  VS_Highlight();
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_Highlight();
	}
}

// ================================================================================================
technique Highlight_Skinning
<
    int BonesPerPartition = MAX_BONES;
    string NBTMethod = "ATI";
    bool UsesNIRenderState = true;
>
{
    pass p0
	{
		HighlightStates
		
		VertexShader = compile SHAD_VS_MINVER_1_1 VS_HighlightSkinned(4);
		// VertexShader = compile SHAD_VS_MINVER_1_1 VS_Highlight(4);
		PixelShader  = compile SHAD_PS_MINVER_2_0 PS_Highlight();
	}
}

// ================================================================================================
const float4x4 matWVP              : WORLDVIEWPROJECTION;
const float3   HighLightColor0     : GLOBAL;
const float3   HighLightColor1     : GLOBAL;

// ================================================================================================
struct VS_ScreenQuadOut
{
    float4 vPos : POSITION;
    float2 vTex : TEXCOORD0;
};

// ================================================================================================
VS_ScreenQuadOut VS_ScreenQuad(float4 inPos: POSITION, float2 inTex: TEXCOORD)
{
    VS_ScreenQuadOut o = (VS_ScreenQuadOut)0;
    
    o.vPos = mul(inPos, matWVP);
    o.vTex = inTex;
    return o;
}

// ================================================================================================
float4 PS_HighlightPostCombine(float2 texCoord : TEXCOORD0) : COLOR
{
    float3 src = tex2D(SamplerBase, texCoord.xy).xyz;
    
    float3 color = 1;
    if (src.x > 0)
        color = src.x * HighLightColor0 + (1 - src.x);
    else if (src.y > 0)
        color = src.y * HighLightColor1 + (1 - src.y);
    
    float alpha = max(src.x, max(src.y, src.z));

    return float4(color, alpha);
}

// ================================================================================================
technique Highlight_PostFX
{
    pass p0
	{
		FillMode            = solid;
		ShadeMode           = flat;
	    CullMode            = cw;
	    ZEnable             = false;
	    ZWriteEnable        = false;
	    Lighting            = false;
	    AlphaTestEnable     = true;
	    AlphaRef            = 15;
	    AlphaBlendEnable    = true;
	    
	    //ADDITIV: 
	    //SrcBlend          = One;
	    //DestBlend         = One;
        
        //Mul:
	    SrcBlend            = SrcAlpha;
        DestBlend           = InvSrcAlpha;

#if defined(SHADERMODEL3) && !defined(MIN_SHADER_VERSION)
	    VertexShader = compile SHAD_VS_MINVER_3_0 VS_ScreenQuad();
#else
	    VertexShader = null;
#endif
        PixelShader  = compile SHAD_PS_MINVER_2_0 PS_HighlightPostCombine();
	}
}
