// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2008 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: mad
// date: 21.07.2009

#include "GlobalFX.h"

// ================================================================================================
#if PIXELSHADER_VERSION >= 30 && !defined(MIN_SHADER_VERSION)  // variance shadow mapping here
//#   define VARIANCE_SHADOW_MAPPING // -> handled by app now
#   if defined(VARIANCE_SHADOW_MAPPING)
//#   define USE_EVSM
#   endif // VARIANCE_SHADOW_MAPPING
#else
// when not supported, undef VARIANCE_SHADOW_MAPPING ...
#   ifdef VARIANCE_SHADOW_MAPPING
#   undef VARIANCE_SHADOW_MAPPING
#   endif // VARIANCE_SHADOW_MAPPING
#endif    // PIXELSHADER_VERSION ...

static const float2 g_EVSMExponents = float2(14.0f, 14.0f); // float2(42.0f, 42.0f);
static const float  g_EVSM_Derivation = 0.001f; //0.0001f;

// ================================================================================================
#define SHADOWVAL_OFFSET        0.0013
#define VSM_EPSILON             1e-6    // 0.00006 // 0.00001
#define LOCAL_VSM_FILTERMODE    0       // 0 == NONE, 1 == bilerp, 2 == sampled(8/16??)
                                        // if texture is to small (or pixels too large) than bilerp may
                                        // result in artifacts, cause it moves to the next pixel on shadowmap,
                                        // which can be several cm/units on graphics -> bad blurring

// ================================================================================================
const float4x4  matLightViewProj     : GLOBAL;
const float     g_fNearToFarDistance : GLOBAL = 1.0;
const float     g_fMaxLightDistance  : GLOBAL;
const float     ShadowMapSize        : GLOBAL = 512.0;
const float     ShadowFactor         : GLOBAL = 0.0;

// ================================================================================================
#if !defined(USE_INDIVIDUAL_SHADOW_TEXTURE_SLOT)
	texture ShadowTex<int NTMIndex = 5;>;
#endif

//-----------------------------------
// standard sampler for shadow
//-----------------------------------
sampler2D ShadowMap = sampler_state
{
    Texture = (ShadowTex);

#if defined(VARIANCE_SHADOW_MAPPING)
#   if defined(USE_EVSM)
    
    AddressU = CLAMP;
    AddressV = CLAMP;
    BorderColor = float4(1.0f, 1.0f, 1.0f, 1.0f);
	
#   else // USE_EVSM
    
    AddressU = BORDER;
    AddressV = BORDER;
    BorderColor = float4(1.0f, 1.0f, 1.0f, 1.0f);
    
#   endif // USE_EVSM

    MagFilter = LINEAR;
    MinFilter = ANISOTROPIC;
    MipFilter = LINEAR;
    MaxAnisotropy = 4;

#else
    AddressU = BORDER;
    AddressV = BORDER;
    BorderColor = float4(0.0f, 0.0f, 0.0f, 0.0f);

    MinFilter = 1;
    MagFilter = 1;
    MipFilter = 0;
    MaxAnisotropy = 1;
#endif

};
                                        
// ================================================================================================
float4 PerformTransformInLightSpace(float4 WorldPosition)
{
    //return mul(WorldPosition, matLightViewProj);
    float4 v = mul(WorldPosition, matLightViewProj);
    
    // da man diesen Wert nur fr die proj. Texturcoords bentigt
    // und den Tiefenwert, hier schon paar berechnungen vornehmen, die im
    // Pixelshader unntig sind
    v.y = -v.y;             // fr TexCoords - y umdrehen
#if defined(VARIANCE_SHADOW_MAPPING)
    //v.z = abs(1 - v.z);
#else
    //v.z = abs(1 - v.z);     // fr Tiefe     - diese auch umdrehen fr schnelleren Vergleich
	v.z = saturate(1 - v.z);
#endif    
    return v;
}

// ================================================================================================
float GetDepthInLightSpace(float4 LightSpacePosition)
{
    return (LightSpacePosition.z / LightSpacePosition.w);
}

// ================================================================================================
float GetProjDepthValue(float3 LightVSPosition, sampler2D ShadowMap)
{
    float2 vProjTexCoords = LightVSPosition.xy;
    // vProjTexCoords.y  = -vProjTexCoords.y;
    vProjTexCoords.xy = vProjTexCoords.xy*0.5 + 0.5;

    return tex2D(ShadowMap, vProjTexCoords.xy).x - SHADOWVAL_OFFSET;   // + kleinen Offset
}

// ================================================================================================
float GetShadowValueSTD(float p_fDepthInLS, float3 p_LightVSPosition, sampler2D p_ShadowMap, float p_fEpsilon)
{
	// by jow: added shadow factor
    float fShadow = 1.0f;
    float fDepth = GetProjDepthValue(p_LightVSPosition, p_ShadowMap);
    // if (fDepth > fDepthInLS || fDepthInLS > 1.0 || fDepthInLS < 0.0)
    //    fShadow = 1;
    if (fDepth > p_fDepthInLS && p_fDepthInLS <= 1.0)
        fShadow = ShadowFactor;

    return fShadow;
}

// ================================================================================================
static const float2 texOffset[] = {
    float2( 0.910, 0.140),
    float2(-0.320, 0.560),
    float2( 0.640,-0.052),
    float2(-0.145,-0.745),
    float2( 0.312,-0.028),
    float2(-0.725, 0.728),
    float2( 0.239, 0.312),
    float2(-0.439, 0.912),
    
    float2( 1.910, 1.140),
    float2(-1.320, 1.560),
    float2( 1.640,-1.052),
    float2(-1.145,-1.745),
    float2( 1.312,-1.028),
    float2(-1.725, 1.728),
    float2( 1.239, 1.312),
    float2(-1.439, 1.912)
};

#define USED_SAMPLES 16
// ================================================================================================
float4 tex2D_sampled(sampler2D p_Sampler, float2 p_texCoord, float2 p_TexPixelSize, float p_fudge)
{
    float4 result = float4(0,0,0,0);
    
    float2 vDelta = (1 + p_fudge)/p_TexPixelSize;
    for (int i=0; i<USED_SAMPLES; ++i)
    {
        float2 fTex = p_texCoord + (texOffset[i] * vDelta);
        result += tex2D(p_Sampler, fTex);
    }
    
    result /= (float)USED_SAMPLES;
    return result;
}

// ================================================================================================
float4 tex2D_bilerp(sampler2D p_Sampler, float2 p_texCoord, float2 p_TexPixelSize, float p_fudge)
{
    float2 fInvTexSizeHalf = float2(0.5/p_TexPixelSize.x, 0.5/p_TexPixelSize.y);
    float4 texOffsets  = float4(-fInvTexSizeHalf.x+p_fudge, -fInvTexSizeHalf.y+p_fudge,
                                 fInvTexSizeHalf.x-p_fudge,  fInvTexSizeHalf.y-p_fudge);
    float4 offsetCoord = float4(p_texCoord.xy, p_texCoord.xy) + texOffsets;
    
    float2 fracCoord = frac(offsetCoord.xy * p_TexPixelSize);
    
    float4 s00 = tex2D(p_Sampler, offsetCoord.xy);
    float4 s10 = tex2D(p_Sampler, offsetCoord.zy);
    float4 s01 = tex2D(p_Sampler, offsetCoord.xw);
    float4 s11 = tex2D(p_Sampler, offsetCoord.zw);
    
    s00 = lerp(s00, s10, fracCoord.x);    // was mix
    s01 = lerp(s01, s11, fracCoord.x);
    s00 = lerp(s00, s01, fracCoord.y);
    
    return s00;
}

// ================================================================================================
float ChebyshevUpperBound(float2 moments, float mean, float minVariance)
{
    // Compute variance
    float variance = moments.y - (moments.x * moments.x);
    variance = max(variance, minVariance);
    
    // Compute probabilistic upper bound
    float d = mean - moments.x;
    float pMax = variance / (variance + (d * d));
    
    // One-tailed Chebyshev
    return (mean <= moments.x ? 1.0f : pMax);
}

// ================================================================================================
float2 WarpDepth(float depth, float2 exponents)
{
    // Rescale depth into [-1, 1]
    depth = 2.0f * depth - 1.0f;
    float pos =  exp( exponents.x * depth);
    float neg = -exp(-exponents.y * depth);
    return float2(pos, neg);
}

// ================================================================================================
float GetShadowValueVSM(float p_fDepthInLS, float3 p_LightVSPosition, sampler2D p_ShadowMap, float p_fEpsilon)
{
    float2 vProjTexCoords = p_LightVSPosition.xy;
    vProjTexCoords.xy = vProjTexCoords.xy*0.5 + 0.5;
    
#if defined(USE_EVSM)
    float4 vMoments;
#else   // !USE_EVSM
    float2 vMoments;
#endif  // USE_EVSM
    
#if LOCAL_VSM_FILTERMODE == 1
    vMoments = tex2D_bilerp (p_ShadowMap, vProjTexCoords.xy, float2(ShadowMapSize, ShadowMapSize), 0.00000001);
#elif LOCAL_VSM_FILTERMODE == 2
    vMoments = tex2D_sampled(p_ShadowMap, vProjTexCoords.xy, float2(ShadowMapSize, ShadowMapSize), 0.00000001);
#else   // any other value
    vMoments = tex2D(p_ShadowMap, vProjTexCoords.xy);
#endif
    //vMoments.y = vMoments.x*vMoments.x;
    
    p_fDepthInLS = saturate(p_fDepthInLS);
    
    /* first implementation for inequation
    float fLit = (p_fDepthInLS <= vMoments.x)?1.0:0.0;
    
    float fE_x2 = vMoments.y;
    float fEx_2 = vMoments.x*vMoments.x;
    
    float fVariance = min(max(fE_x2 - fEx_2, 0.0) + p_fEpsilon, 1.0);
    float fmD = vMoments.x - p_fDepthInLS;
    float fPmax = fVariance / (fVariance + fmD*fmD);
    
    return max(fLit, fPmax);
    // */
    
#if defined(USE_EVSM)
    vMoments = vMoments * 2 - 1;
    // exponential variance shadow mapping
    float2 exponents = g_EVSMExponents;
    float2 warpedDepth = WarpDepth(p_fDepthInLS, exponents);

	// Derivative of warping at depth
    float2 depthScale = g_EVSM_Derivation * exponents * warpedDepth;
    float2 minVariance = depthScale * depthScale;

	// Compute the upper bounds of the visibility function both for x and y
    float posContrib = ChebyshevUpperBound(vMoments.xz, warpedDepth.x, minVariance.x);
    float negContrib = ChebyshevUpperBound(vMoments.yw, warpedDepth.y, minVariance.y);

    return min(posContrib, negContrib);
    
#else   // !USE_EVSM
    
    // standard variance shadow mapping
    return ChebyshevUpperBound(vMoments.xy, p_fDepthInLS, p_fEpsilon);
    
#endif  // USE_EVSM
}

// ================================================================================================
// 0 = Shadow, 1 = Light
// ================================================================================================
float GetObjectInShadowFactor(float fDepthInLS, float3 LightVSPosition, sampler2D ShadowMap)
{
#if defined(VARIANCE_SHADOW_MAPPING)
    float f = GetShadowValueVSM(fDepthInLS, LightVSPosition, ShadowMap, VSM_EPSILON);
    return max(f, ShadowFactor);
#else // use standard version
    return GetShadowValueSTD(fDepthInLS, LightVSPosition, ShadowMap, SHADOWVAL_OFFSET);
#endif
}
