// BALIVE LIMITED PROPRIETARY INFORMATION //
// This software is supplied under the terms of a license agreement or
// nondisclosure agreement with B-Alive GmbH and may not 
// be copied or disclosed except in accordance with the terms of that 
// agreement.
//
//      Copyright (c) 2004-2008 B-Alive GmbH.
//      All Rights Reserved.
//
// B-Alive GmbH
// http://www.b-alive.de
//
// author: mad
// date: 21.07.2009

// ================================================================================================
#include "Skinning.h"

// ================================================================================================
// by jow: kein ZEnable/ZWriteEnable, da die Depthmap bei 8192x8192 Auflsung zu viel Speicher frisst. Deswegen BlendOp Max.
#define SetStates \
    FillMode            = <GROUND_FILL_MODE>; \
	ShadeMode           = Gouraud; \
	ZEnable             = false; \
	ZWriteEnable        = false; \
	AlphaTestEnable     = true; \
	AlphaRef            = 30; \
	AlphaFunc           = greater; \
	AlphaBlendEnable    = true; \
	SrcBlend            = One; \
	DestBlend           = One; \
	BlendOp				= Max; \
	Lighting            = false

// ================================================================================================
const float     WIND				: GLOBAL;

// params for clouds
const float4x4  TEX1_TRANSFORM      : GLOBAL;
const float4x4  TEX2_TRANSFORM      : GLOBAL;

// ================================================================================================
void GenerateShadowDepthValue(inout float4 p_OutColor, float p_fDepth)
{
#if defined(VARIANCE_SHADOW_MAPPING)
    p_OutColor = float4(p_fDepth,0,0,1);
#else
    p_OutColor = float4(1 - p_fDepth, 0, 0, 1);
#endif
}

// ================================================================================================
// copy from groundtile_shared
struct VSIn
{
   float3 Pos				: POSITION;
   float3 Normal			: NORMAL;
   float4 TexBlendNCWeight	: COLOR0;
   float2 TexSurface		: TEXCOORD0;
   float2 WaterHeight		: TEXCOORD1;
};

// ================================================================================================
struct VS_PosNormUV0
{
    float4 vPosition: POSITION;
    float3 vNormal  : NORMAL;
	float4 vColor	: COLOR;
    float2 vTex0    : TEXCOORD0;
    float2 vTex1    : TEXCOORD1;
    float2 vTex2    : TEXCOORD2;
    float2 vTex3    : TEXCOORD3;
};


// ================================================================================================
struct VS_PosDepth
{
    float4 vPosition : POSITION;
    float3 vNormal   : TEXCOORD0;
    float2 vTex      : TEXCOORD1;
    //float  fDepth    : TEXCOORD2;
    float4 vDepth    : TEXCOORD2; // debug
	float3 vPositionVS : TEXCOORD3;
};

// ================================================================================================
struct VS_PosDepthDummy
{
    float4 vPosition : POSITION;
    float3 vNormal   : TEXCOORD0;
    float2 vTex      : TEXCOORD1;
    float  fDepth    : TEXCOORD2;
    //float4 fDepth    : TEXCOORD2; // debug
    float4 vDummy    : TEXCOORD3;
	float3 vPositionVS : TEXCOORD4;
};

// ================================================================================================
struct VS_AnimGrass
{
    float4 vPosition    : POSITION;
    float4 vColor       : COLOR;
    float2 vTex         : TEXCOORD0;
    float3 vDirection   : NORMAL0;
};

// ================================================================================================
struct VS_InClouds
{
    float4 Position : POSITION;
    float2 TexCoord0: TEXCOORD0;
    float2 TexCoord1: TEXCOORD1;
};

#define PS_OUTPUT PS_OutColor

struct PS_OutColor
{
	float4 Color : COLOR0;
};

// ================================================================================================
// VERTEXSHADER
// ================================================================================================
VS_PosDepth VS_ShadowMapGen_Skinning(uniform int NumBones, VS_InSkinning i)
{
    VS_OutNormal os = Skinning(i.Pos, i.BlendWeights, i.BlendIndices, i.Normal, NumBones);
    
    // ----------------------------
    // HACK: damit der Compiler die VARS nicht kickt...
    float3 vTmp0 = i.Normal;
    vTmp0 = normalize(vTmp0);
    // ----------------------------
    
    VS_PosDepth o;
    o.vPosition = mul(float4(os.Pos, 1), matViewProj);
    
    //o.fDepth = o.vPosition.z / o.vPosition.w;
    o.vDepth = float4(o.vPosition.z / o.vPosition.w,0,0,0);
    //o.fDepth.x  = o.vPosition.z/ o.vPosition.w;
    //o.fDepth.yzw = os.Pos.xyz;
    
    o.vNormal =  i.Normal;

	// nur mit der Viewmatrix transformieren, da die Position in os.Pos durch das skinning bereits in World Space ist
	o.vPositionVS = mul(float4(os.Pos, 1), matView).xyz;

    o.vTex =     i.Tex;
    return o;
}

// ================================================================================================
VS_PosDepthDummy VS_ShadowMapGen_SkinningNBT(uniform int NumBones, VS_InSkinnedNBT i)
{
    VS_OutSkinnedNBT os = SkinningNBT(i, NumBones);
    
    // ----------------------------
    // HACK: damit der Compiler die VARS nicht kickt...
    float3 vTmp0 = os.Normal;
    vTmp0 = normalize(vTmp0);
    vTmp0 += os.Binormal;
    vTmp0 += os.Tangent;
    // ----------------------------
    
    VS_PosDepthDummy o = (VS_PosDepthDummy)0;
    o.vPosition  = mul(float4(os.Pos, 1), matViewProj);
    
    o.fDepth     = o.vPosition.z / o.vPosition.w;
    //o.fDepth.x   = o.vPosition.z/ o.vPosition.w;
    //o.fDepth.yzw = os.Pos.xyz;
    
    o.vNormal    = os.Normal;

	// nur mit der Viewmatrix transformieren, da die Position in os.Pos durch das skinning bereits in World Space ist
	o.vPositionVS = mul(float4(os.Pos, 1), matView).xyz;

    o.vTex       = i.Tex0;
    o.vDummy.x   = i.Binormal.x + i.Tangent.x + i.Tex1.x  + i.Tex2.x  + i.Tex3.x;
    o.vDummy.w   = 1;
    return o;
}

// ================================================================================================
VS_PosDepth VS_ShadowMapGen(VS_PosNormUV0 i, uniform bool bTextured)
{
    // ----------------------------
    // HACK: damit der Compiler die VARS nicht kickt...
    float3 vTmp0 = i.vNormal;
    vTmp0 = normalize(vTmp0);
    // ----------------------------
    
    VS_PosDepth o = (VS_PosDepth)0;
    float4 PosWorld = mul(i.vPosition, matWorld);

	// Wind animation
	float2 direction = 1.0f - i.vColor.xy;
	PosWorld.xy += ComputeWindAnimationDisplacement(PosWorld.xyz, direction);

    o.vPosition = mul(PosWorld, matViewProj);
    //o.fDepth = o.vPosition.z / o.vPosition.w;
    o.vDepth   = float4(o.vPosition.z / o.vPosition.w, 0,0,0);
    o.vDepth.z = i.vTex1.x + i.vTex2.y + i.vTex3.x; // dummy so that this params remain!
    //o.fDepth.x   = o.vPosition.z/ o.vPosition.w;
    //o.fDepth.yzw = i.vPosition.xyz;
    if (bTextured)
    {
        o.vNormal =  i.vNormal;
        o.vTex =     i.vTex0;
    }
	
	o.vPositionVS = mul(PosWorld, matView).xyz;

    return o;
}

// ================================================================================================
VS_PosDepthDummy VS_ShadowMapGenNBT(VS_InNBT i)
{
    // ----------------------------
    // HACK: damit der Compiler die VARS nicht kickt...
    float3 vTmp0 = i.Normal;
    vTmp0 = normalize(vTmp0);
    vTmp0 += normalize(i.Binormal);
    vTmp0 += normalize(i.Tangent);
    // ----------------------------
    
    VS_PosDepthDummy o = (VS_PosDepthDummy)0;
    
    float4 PosWorld = mul(i.Pos, matWorld);
    o.vPosition = mul(PosWorld, matViewProj);
    
    o.fDepth = o.vPosition.z / o.vPosition.w;
    //o.fDepth.x   = o.vPosition.z/ o.vPosition.w;
    //o.fDepth.yzw = PosWorld.xyz;
    
    o.vNormal =  i.Normal;
	
	o.vPositionVS = mul(i.Pos, matWorldView).xyz;
	
    o.vTex =     mul(float4(i.Tex0.xy, 1, 1), matTex);
    o.vDummy.xyz = i.Binormal + i.Tangent + i.Tex1.xxy + i.Tex2.yxy + i.Tex3.xyy;
    o.vDummy.w   = 1;
    return o;
}

// ================================================================================================
VS_PosDepth VS_ShadowMapGen_ExtAreaGround(VS_PosColUV0 i)
{
    // ----------------------------
    // HACK: damit der Compiler die VARS nicht kickt...
    float3 vTmp0 = i.vColor.xyz;
    vTmp0 = normalize(vTmp0);
    vTmp0.xy = i.vTex;
    vTmp0.xy += 0.5;
    // ----------------------------
    
    VS_PosDepth o;
    o.vPosition = mul(i.vPosition, matWorldViewProjection);
    
    //o.fDepth = o.vPosition.z / o.vPosition.w;
    o.vDepth = o.vPosition;
    //o.fDepth.x   = o.vPosition.z/ o.vPosition.w;
    //o.fDepth.yzw = i.vPosition.xyz;
    
    o.vNormal =  i.vColor;
	
	o.vPositionVS = mul(i.vPosition, matWorldView).xyz;
	
    o.vTex =     i.vTex;
    return o;
}

// ================================================================================================
VS_PosDepth VS_ShadowMapGen_GroundTile(VSIn i)
{
    // ----------------------------
    // HACK: damit der Compiler die VARS nicht kickt...
    float4 vTmp0 = i.TexBlendNCWeight;
    vTmp0.xyz *= 1.5;
    vTmp0.xy += i.TexSurface;
    vTmp0.zw += 0.5 * i.WaterHeight;
    vTmp0.w   = 0;
    // ----------------------------

    VS_PosDepth o;
    //o.vPosition = mul(i.Pos, matWorldViewProjection);
    o.vPosition = mul(float4( i.Pos, 1.0f ), matWorldViewProjection);
    o.vDepth = o.vPosition;

    o.vNormal =  i.Normal     + i.TexBlendNCWeight.xyz;
	
	o.vPositionVS = mul(float4(i.Pos, 1), matWorldView).xyz;
	
    o.vTex =     i.TexSurface + i.WaterHeight;
    
    return o;
}

// ================================================================================================
VS_PosDepth VS_ShadowMapGen_AnimatedGrass(VS_AnimGrass i)
{
    // ----------------------------
    // HACK: damit der Compiler die VARS nicht kickt...
    float3 vTmp0 = i.vColor.xyz;
    vTmp0 = normalize(vTmp0);
    vTmp0.xy = i.vTex;
    vTmp0.xy += 0.5;
    // ----------------------------

    float fWindFactor = WIND * 1.5 + 0.5;
    float4 vPosition = i.vPosition;
    vPosition.xy    += i.vDirection.xy * fWindFactor * sin(Time * i.vDirection.z);
    
    //float4 vPosition = i.vPosition;
    ////vPosition.xy    += i.vDirection.xy * cos(Time) * max(0.2 ,i.vDirection.z);
    //vPosition.xy    += i.vDirection.xy * cos(Time * i.vDirection.z);

    VS_PosDepth o;
    o.vPosition = mul(vPosition, matWorldViewProjection);
    
    o.vDepth = o.vPosition;
    
    o.vNormal =  i.vColor*i.vDirection;

	o.vPositionVS = mul(vPosition, matWorldView).xyz;

    o.vTex =     i.vTex;
    return o;
}

// ================================================================================================
VS_PosDepth VS_ShadowMapGen_Panorama(VS_PosColUV0 i)
{
    VS_PosDepth o;
    o.vPosition = mul(i.vPosition, matWorldViewProjection);
    
    //float4 tmp  = mul(matWorldView, i.vPosition);
    //o.vNormal = tmp.xyz; //float3(0,0,1);
    
    o.vDepth  = float4(o.vPosition.z / o.vPosition.w, 0,0,0);
    o.vTex    = i.vTex;
    o.vNormal = float3(0,0,1);

	o.vPositionVS = mul(i.vPosition, matWorldView).xyz;

    return o;
}


// ================================================================================================
VS_PosDepthDummy VS_ShadowMapGen_Clouds(VS_InClouds i)
{
    VS_PosDepthDummy o = (VS_PosDepthDummy)0;
    o.vPosition = mul(float4(i.Position.xyz, 1), matWorldViewProjection);
    
    // here we need the normal texture param for the first texcoord and a second
    // texcoord is passed via the dummy-param
    o.vTex      = i.TexCoord0; //mul(TEX1_TRANSFORM, i.TexCooord);
    o.vDummy.xy = i.TexCoord1; //mul(TEX2_TRANSFORM, i.TexCooord);

	o.vPositionVS = mul(i.Position, matWorldView).xyz;

    return o;
}

// ================================================================================================
// PIXELSHADER
// ================================================================================================
PS_OUTPUT PS_ShadowMapGen(VS_PosDepth i, uniform bool bTextured, uniform bool bSpecialLayer = false)
{
	PS_OUTPUT output;
	output.Color = float4(1,0,0,1);

	GenerateShadowDepthValue(output.Color, i.vDepth.x);
    if (bTextured)
    {
        float4 colTex = tex2D(SamplerBase, i.vTex);
        colTex.rgb = dot(i.vNormal, colTex.rgb);
        output.Color.a = colTex.a;

        if (bSpecialLayer)
        {
            // special decay map...
            float alpha =  tex2D(Sampler4, i.vTex).a;
            output.Color.a   = ((alpha + 0.001) >= g_fSecondColorDecay) ? output.Color.a : 0;
        }
    }

	output.Color.g = i.vPositionVS.z;

	return output;
	/*
    float4 result = float4(1,0,0,1);
    
    //result.r  = 1 - i.fDepth;
    //result.r  = 1 - i.vDepth.x;
    GenerateShadowDepthValue(result, i.vDepth.x);
    if (bTextured)
    {
        float4 colTex = tex2D(SamplerBase, i.vTex);
        colTex.rgb = dot(i.vNormal, colTex.rgb);
        result.a = colTex.a;

        if (bSpecialLayer)
        {
            // special decay map...
            float alpha =  tex2D(Sampler4, i.vTex).a;
            result.a   = ((alpha + 0.001) >= g_fSecondColorDecay) ? result.a : 0;
        }
    }
    return result;
	*/
}

// ================================================================================================
PS_OUTPUT PS_ShadowMapGenGround(VS_PosDepth i)
{
	PS_OUTPUT output;
	output.Color = float4(1,0,0,1);

	GenerateShadowDepthValue(output.Color, (i.vDepth.z/i.vDepth.w));

	output.Color.g = i.vPositionVS.z;

	return output;

	/*
    float4 result = float4(1,0,0,1);

    //result.r = 1 - i.fDepth;
    //result.r  = 1 - (i.vDepth.z/i.vDepth.w);
    GenerateShadowDepthValue(result, (i.vDepth.z/i.vDepth.w));
    
    return result;
	*/
}

// ================================================================================================
PS_OUTPUT PS_ShadowMapGen_Dummy(VS_PosDepthDummy i)
{
	PS_OUTPUT output;
	output.Color = float4(1,0,0,1);

	GenerateShadowDepthValue(output.Color, i.fDepth);

	float4 colTex = tex2D(SamplerBase, i.vTex);
    output.Color.a = colTex.a;

	output.Color.g = i.vPositionVS.z;

	return output;
	/*
    float4 result = float4(1,0,0,1);
    
    //result.r = 1 - i.fDepth;
    GenerateShadowDepthValue(result, i.fDepth);
    //result.r  = 1 - (i.fDepth.x);
    //result.gb = pow(i.fDepth.yz*0.00025, 2);
    
    
    float4 colTex = tex2D(SamplerBase, i.vTex);
    result.a = colTex.a;
    
    // result.a = 1 + abs(i.vDummy.x);   // HACK - damit der Param genutzt wird
    // result.a = 1;
    return result;
	*/
}

// ================================================================================================
PS_OUTPUT PS_ShadowMapGen_NBTSpecial(VS_PosDepthDummy i)
{
	PS_OUTPUT output;
	output.Color = float4(1,0,0,1);

	GenerateShadowDepthValue(output.Color, i.fDepth);

	float4 colTex = tex2D(SamplerBase, i.vTex);
    output.Color.a = colTex.a;
    // special decay map...
    float alpha =  tex2D(Sampler4, i.vTex).a;
    output.Color.a   = ((alpha + 0.001) >= g_fSecondColorDecay) ? output.Color.a : 0;

	output.Color.g = i.vPositionVS.z;

	return output;
	/*
    float4 result = float4(1,0,0,1);
    
    //result.r = 1 - i.fDepth;
    GenerateShadowDepthValue(result, i.fDepth);
    //result.r  = 1 - (i.fDepth.x);
    //result.gb = pow(i.fDepth.yz*0.00025, 2);
    
    
    float4 colTex = tex2D(SamplerBase, i.vTex);
    result.a = colTex.a;
    // special decay map...
    float alpha =  tex2D(Sampler4, i.vTex).a;
    result.a   = ((alpha + 0.001) >= g_fSecondColorDecay) ? result.a : 0;
    
    // result.a = 1 + abs(i.vDummy.x);   // HACK - damit der Param genutzt wird
    // result.a = 1;
    return result;
	*/
}

// ================================================================================================
PS_OUTPUT PS_ShadowMapGenGrass(VS_PosDepth i)
{
	PS_OUTPUT output;
	output.Color = float4(1,0,0,1);
	
	GenerateShadowDepthValue(output.Color, (i.vDepth.z / i.vDepth.w));

	float4 colTex = tex2D(Sampler0, i.vTex);
    output.Color.a = colTex.a;

	output.Color.g = i.vPositionVS.z;

	return output;

	/*
    float4 result = float4(1,0,0,1);
    
    //result.r  = 1 - i.fDepth;
    //result.r  = 1 - (i.vDepth.z / i.vDepth.w);
    GenerateShadowDepthValue(result, (i.vDepth.z / i.vDepth.w));
    
    float4 colTex = tex2D(Sampler0, i.vTex);
    result.a = colTex.a;
    
    return result;
	*/
}

// ================================================================================================
PS_OUTPUT PS_ShadowMapGenClouds(VS_PosDepthDummy i)
{
	PS_OUTPUT output;
	output.Color = float4(1,0,0,1);

	GenerateShadowDepthValue(output.Color, i.fDepth);

	float4 colTex1 = tex2D(Sampler0, mul(TEX1_TRANSFORM, float4(i.vTex,0,1)));
    float4 colTex2 = tex2D(Sampler1, mul(TEX2_TRANSFORM, float4(i.vTex,0,1)));
    float4 colMask = tex2D(Sampler2, i.vDummy.xy);

	output.Color.a = colTex1.a*colTex2.a*colMask.a;

	output.Color.g = i.vPositionVS.z;
	
	return output;
	
    /*float4 result = float4(1,0,0,1);
    
    //result.r  = 1 - i.fDepth;
    GenerateShadowDepthValue(result, i.fDepth);
    //result.r  = 1 - (i.vDepth.z / i.vDepth.w);
    
    float4 colTex1 = tex2D(Sampler0, mul(TEX1_TRANSFORM, float4(i.vTex,0,1)));
    float4 colTex2 = tex2D(Sampler1, mul(TEX2_TRANSFORM, float4(i.vTex,0,1)));
    float4 colMask = tex2D(Sampler2, i.vDummy.xy);
    result.a = colTex1.a*colTex2.a*colMask.a;
    
    return result;*/
	
}

// ================================================================================================
// technique
// ================================================================================================
technique ShadowMapGeneration
<
    bool UsesNIRenderState = true;
>
{
    pass p0
	{
		SetStates;
	    		
		VertexShader = compile SHAD_VS_MINVER_1_1  VS_ShadowMapGen(true);
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGen(true);
	}
}

// ================================================================================================
technique ShadowMapGeneration_NoTex
<
    bool UsesNIRenderState = true;
>
{
    pass p0
	{
		SetStates;
	    		
		VertexShader = compile SHAD_VS_MINVER_1_1  VS_ShadowMapGen(false);
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGen(false);
	}
}

// ================================================================================================
technique ShadowMapGeneration_GroundTile
<
    bool UsesNIRenderState = true;
>
{
    pass p0
	{
		SetStates;
	    		
		VertexShader = compile SHAD_VS_MINVER_1_1  VS_ShadowMapGen_GroundTile();
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGenGround();
	}
}

// ================================================================================================
technique ShadowMapGeneration_ExtAreaGround
<
    bool UsesNIRenderState = true;
>
{
    pass p0
	{
		SetStates;
	    		
		VertexShader = compile SHAD_VS_MINVER_1_1  VS_ShadowMapGen_ExtAreaGround();
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGenGround();
	}
}

// ================================================================================================
technique ShadowMapGeneration_Grass
<
    bool UsesNIRenderState = true;
>
{
    pass p0
	{
        FillMode            = <GROUND_FILL_MODE>;
		CullMode            = None;
	    ZEnable             = true;
	    ZWriteEnable        = true;
	    AlphaTestEnable     = true;
	    AlphaRef            = 80;
	    AlphaFunc           = greater;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    		
		VertexShader = compile SHAD_VS_MINVER_1_1  VS_ShadowMapGen_ExtAreaGround();
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGenGrass();
	}
}

// ================================================================================================
technique ShadowMapGeneration_AnimatedGrass
<
    bool UsesNIRenderState = true;
>
{
    pass p0
	{
        FillMode            = <GROUND_FILL_MODE>;
		CullMode            = None;
	    ZEnable             = true;
	    ZWriteEnable        = true;
	    AlphaTestEnable     = true;
	    AlphaRef            = 80;
	    AlphaFunc           = greater;
	    AlphaBlendEnable    = false;
	    Lighting            = false;
	    		
		VertexShader = compile SHAD_VS_MINVER_1_1  VS_ShadowMapGen_AnimatedGrass();
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGenGrass();
	}
}

// ================================================================================================
technique ShadowMapGeneration_Skinning
<
    int BonesPerPartition = MAX_BONES;
    string NBTMethod = "none";
    bool UsesNIRenderState = true;
>
{
    pass p0
	{
		SetStates;
	    		
		VertexShader = compile SHAD_VS_MINVER_1_1  VS_ShadowMapGen_Skinning(4);
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGen(true);
	}
}

// ================================================================================================
technique ShadowMapGenerationNBT
<
	string NBTMethod = "ATI";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        SetStates;
	    
	    VertexShader = compile SHAD_VS_MINVER_1_1  VS_ShadowMapGenNBT();
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGen_Dummy();
    }
}

// ================================================================================================
technique ShadowMapGenerationNBT_Special
<
	string NBTMethod = "ATI";
	bool UsesNIRenderState = true;
>
{
    pass p0
    {
        SetStates;
	    
	    VertexShader = compile SHAD_VS_MINVER_1_1  VS_ShadowMapGenNBT();
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGen_NBTSpecial();
    }
}

// ================================================================================================
technique ShadowMapGeneration_SkinningNBT
<
    int BonesPerPartition = MAX_BONES;
    string NBTMethod = "ATI";
	bool UsesNIRenderState = true;
>
{
    pass p0
	{
		SetStates;
	    		
		VertexShader = compile SHAD_VS_MINVER_2_0  VS_ShadowMapGen_SkinningNBT(4);
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGen_Dummy();
	}
}

// ================================================================================================
technique ShadowMapGeneration_SkinningNBTSpecial
<
    int BonesPerPartition = MAX_BONES;
    string NBTMethod = "ATI";
	bool UsesNIRenderState = true;
>
{
    pass p0
	{
		SetStates;
	    		
		VertexShader = compile SHAD_VS_MINVER_1_1  VS_ShadowMapGen_SkinningNBT(4);
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGen_NBTSpecial();
	}
}

// ================================================================================================
technique ShadowMapGeneration_Panorama
{
	pass p0
	{
		FillMode            = <GROUND_FILL_MODE>;
		ShadeMode           = Gouraud;
	    CullMode            = CW;
	    ZEnable             = true;
	    ZWriteEnable        = true;
	    AlphaTestEnable     = true;
	    AlphaRef            = 0;
	    AlphaFunc           = greater;
	    AlphaBlendEnable    = true;
	    SrcBlend            = SrcAlpha;
	    DestBlend           = InvSrcAlpha;
	    Lighting            = false;
	    		
		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_ShadowMapGen_Panorama();
		PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_ShadowMapGen(true);
	}
}

// ================================================================================================
technique ShadowMapGeneration_Clouds
{
	pass p0
	{
		FillMode            = <GROUND_FILL_MODE>;
		ShadeMode           = Gouraud;
	    CullMode            = CW;
	    AlphaTestEnable     = false;
	    AlphaBlendEnable    = true;
	    SrcBlend            = SrcAlpha;
	    DestBlend           = InvSrcAlpha;
	    Lighting            = false;
        ZEnable             = false;
        ZWriteEnable        = false;
        AlphaTestEnable     = true;
	    AlphaRef            = 80;
	    AlphaFunc           = greater;
	    	    		
		VertexShader = compile SHAD_VS_MINVER_1_1  VS_ShadowMapGen_Clouds();
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGenClouds();
	}
}

// ================================================================================================
technique ShadowMapGeneration_Special
<
    bool UsesNIRenderState = true;
>
{
    pass p0
	{
		SetStates;
	    		
		VertexShader = compile SHAD_VS_MINVER_1_1  VS_ShadowMapGen(true);
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGen(true, true);
	}
}

// ================================================================================================
technique ShadowMapGeneration_SkinningSpecial
<
    int BonesPerPartition = MAX_BONES;
    string NBTMethod = "none";
    bool UsesNIRenderState = true;
>
{
    pass p0
	{
		SetStates;
	    		
		VertexShader = compile SHAD_VS_MINVER_1_1  VS_ShadowMapGen_Skinning(4);
		PixelShader  = compile SHAD_PS_MINVER_2_0  PS_ShadowMapGen(true, true);
	}
}

// ================================================================================================
// ================================================================================================
const DWORD     EW_CULL_MODE: GLOBAL = 2;

// ================================================================================================
struct VS_InWater
{
    float4 Pos : POSITION;
    float2 Tex : TEXCOORD0;
};

// ================================================================================================
VS_PosDepth VS_WaterDepth(VS_InWater i)
{
    VS_PosDepth o = (VS_PosDepth)0;
    //float4 PosWorld = mul(i.Pos, matWorld);
    //o.vPosition = mul(PosWorld, matViewProjection);
    o.vPosition = mul(i.Pos, matWorldViewProjection);
    o.vDepth    = float4(o.vPosition.z , o.vPosition.w, 0, 0);
    o.vDepth.w  = i.Tex.x; // dummy so that this params remain!

	o.vPositionVS = mul(i.Pos, matWorldView).xyz;

    return o;
}

// ================================================================================================
PS_OUTPUT PS_WaterDepth(VS_PosDepth i)
{
	PS_OUTPUT output;
	output.Color = float4(1,0,0,1);

	GenerateShadowDepthValue(output.Color, i.vDepth.x / i.vDepth.y);

	output.Color.g = i.vPositionVS.z;

	return output;

    //float4 result = float4(1,0,0,1);
    //GenerateShadowDepthValue(result, i.vDepth.x / i.vDepth.y);
    //return result;
}

// ================================================================================================
technique WaterDepth
{
    pass p0
    {
        CullMode            = <EW_CULL_MODE>;
        ZWriteEnable        = false;
        FillMode            = solid;
        AlphaBlendEnable    = false;
        
        VertexShader = compile SHAD_VS_MINVER_2_0 VS_WaterDepth();
        PixelShader  = compile SHAD_PS_MINVER_2_0 PS_WaterDepth();
    }
}

// ================================================================================================
// ================================================================================================
VS_PosDepth VS_ShadowMapGen_Billboard(VS_PosColUV0 p_vsIn)
{
    VS_PosDepth o = (VS_PosDepth)0;
    o.vPosition = mul(p_vsIn.vPosition, matWorldViewProjection);
    o.vTex.xy   = p_vsIn.vTex;
    o.vDepth    = float4(o.vPosition.z / o.vPosition.w,0,0,0); //o.vPosition;

    // ----------------------------
    // HACK: damit der Compiler die VARS nicht kickt...
    float4 vTmp0 = p_vsIn.vColor * 2.f;
    // ----------------------------

	o.vPositionVS = mul(p_vsIn.vPosition, matWorldView).xyz;

    return o;
};

// ================================================================================================
PS_OUTPUT PS_ShadowMapGen_Billboard(VS_PosDepth i)
{
	PS_OUTPUT output;
	output.Color = float4(1,0,0,1);

	GenerateShadowDepthValue(output.Color, (i.vDepth.z / i.vDepth.w));

	float4 colTex = tex2D(Sampler0, i.vTex.xy);
    output.Color.a = colTex.a;

	output.Color.g = i.vPositionVS.z;

	return output;

    //float4 result = float4(1,0,0,1);
   // GenerateShadowDepthValue(result, (i.vDepth.z / i.vDepth.w));
    
    //float4 colTex = tex2D(Sampler0, i.vTex.xy);
    //result.a = colTex.a;

	//return result;
}

// ================================================================================================
technique ShadowMapGeneration_BillboardUnlit
{
	pass p0
	{
		FillMode            = <GROUND_FILL_MODE>;
		ShadeMode           = Flat;
	    CullMode            = CW;
	    ZEnable             = true;
	    ZWriteEnable        = true;
	    AlphaTestEnable     = true;
	    AlphaRef            = 0;
	    AlphaFunc           = Greater;
	    AlphaBlendEnable    = false;
	    Lighting            = false;

		VertexShader = compile  SHAD_VS_MINVER_1_1  VS_ShadowMapGen_Billboard();
		PixelShader  = compile  SHAD_PS_MINVER_2_0  PS_ShadowMapGen_Billboard();
	}
}